Package: visStatistics
Type: Package
Title: Automated Selection and Visualisation of Statistical Hypothesis
        Tests
Version: 0.2.0
Authors@R: c(
    person("Sabine", "Schilling",
           email = "sabineschilling@gmx.ch",
           role = c("cre", "aut", "cph"),
           comment = c(ORCID = "0000-0002-8318-9421", year = "2026")),
    person("Peter", "Kauf",
           email = "peter.kauf@zhaw.ch",
           role = "ctb")
    )
Description: The right test, visualised. 'visStatistics' automatically
    selects and visualises statistical hypothesis tests comparing two vectors,
    based on their class, distribution, and sample size.
    Visual outputs, including box plots, bar charts, regression lines with
    confidence bands, mosaic plots, residual plots, and Q-Q plots, are
    annotated with relevant test statistics, assumption checks, and post-hoc
    analyses where applicable.
    The algorithmic workflow shifts attention from ad-hoc test selection to
    visual diagnostic assessment and statistical interpretation. 
    It is particularly suited for server-side R applications, where end users
    interact solely through a web
    interface to select data groups and receive a complete visual statistical
    analysis automatically. The same automation makes it useful in
    time-constrained contexts such as statistical consulting, where it reduces
    effort spent on test selection and leaves more room for interpretation.
    The implemented tests cover the most frequently applied inferential methods
    in biomedical research (Hayat et al. (2017)
    <doi:10.1371/journal.pone.0179032>).
    The test selection algorithm proceeds as follows: Input vectors of class
    numeric or integer are considered numerical; those of class factor are
    considered categorical; those of class ordered are considered ordinal.
    Assumptions of residual normality and homogeneity of variances are
    considered met if the corresponding test yields a p-value greater than the
    significance level alpha = 1 - conf.level.
    (1) When the response is numerical and the predictor is categorical, a test
    comparing central tendencies is selected. If every group contains more than 
    50 observations, the sampling distribution of the group means is assumed 
    approximately normal by the central limit theorem (Lumley et al. (2002)
    <doi:10.1146/annurev.publhealth.23.100901.140546>); otherwise, residual
    normality is assessed using shapiro.test() applied to the standardised
    residuals of lm(). If normality is not met, wilcox.test() is used when the
    predictor has two levels and kruskal.test() followed by
    pairwise.wilcox.test() otherwise. If normality is met, levene.test()
    assesses variance homogeneity. For two-level predictors, Student's
    t.test(var.equal = TRUE) is applied if variances are homogeneous and
    Welch's t.test() otherwise. For predictors with more than two levels,
    aov() followed by TukeyHSD() is applied if variances are homogeneous, and
    oneway.test() followed by games.howell() otherwise.
    (2) When both vectors are numerical, lm() is fitted by default
    (correlation = FALSE). If correlation = TRUE, Spearman rank correlation is
    performed.
    (3) When the response is ordinal, it is converted to numeric ranks and the
    non-parametric path from (1) is followed (Wilcoxon or Kruskal-Wallis).
    When both variables are ordinal and correlation = TRUE, Kendall's tau_b is
    used instead.
    (4) When both vectors are categorical, Cochran's rule (Cochran (1954)
    <doi:10.2307/3001666>) is applied to test independence either by
    chisq.test() or fisher.test().
License: MIT + file LICENSE
URL: https://github.com/shhschilling/visStatistics,
        https://shhschilling.github.io/visStatistics/
BugReports: https://github.com/shhschilling/visStatistics/issues
Imports: Cairo, graphics, grDevices, grid, multcompView, nortest,
        stats, utils, vcd
Suggests: knitr, rmarkdown, spelling, testthat (>= 3.0.0)
VignetteBuilder: knitr
BuildVignettes: true
Config/testthat/edition: 3
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-05-12 13:06:50 UTC; sschilli
Config/roxygen2/version: 8.0.0
Language: en-GB
Author: Sabine Schilling [cre, aut, cph] (ORCID:
    <https://orcid.org/0000-0002-8318-9421>, year: 2026),
  Peter Kauf [ctb]
Maintainer: Sabine Schilling <sabineschilling@gmx.ch>
Repository: CRAN
Date/Publication: 2026-05-12 18:40:02 UTC
Built: R 4.5.2; ; 2026-05-12 20:47:40 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,visstat)
S3method(print,games.howell)
S3method(print,visstat)
S3method(summary,visstat)
export(bp.test)
export(counts_to_cases)
export(games.howell)
export(gh_letters)
export(levene.test)
export(openGraphCairo)
export(saveGraphVisstat)
export(vis_anova)
export(vis_anova_assumptions)
export(vis_group_normality)
export(vis_lm_assumptions)
export(vis_numeric)
export(visstat)
export(visstat_core)
import(Cairo)
import(grDevices)
import(graphics)
import(grid)
import(multcompView)
import(stats)
import(utils)
import(vcd)
importFrom(nortest,ad.test)

<!-- pkgdown::start -->
<!-- pkgdown::start -->

# visStatistics: The right test, visualised.

The R package `visStatistics` allows for rapid **vis**ualisation and
statistical analysis of raw data. It automatically selects and
visualises the most appropriate **statistic**al hypothesis test between
two vectors of class `integer`, `numeric` or `factor`.

This workflow is particularly suited for browser-based interfaces that
rely on server-side R applications connected to secure databases, where
users have no direct access, or for quick data visualisation, e.g. in
statistical consulting projects.

# Installation of latest stable version from CRAN

#### 1. Install the package

    install.packages("visStatistics")

#### 2. Load the package

    library(visStatistics)

# Installation of the development version from GitHub

#### 1.Install `devtools` from CRAN if not already installed:

    install.packages("devtools")

#### 2. Load the `devtools` package:

    library(devtools)

#### 3. Install the `visStatistics` package from GitHub:

    install_github("shhschilling/visStatistics")

#### 4. Load the `visStatistics` package:

    library(visStatistics)

#### 5. View help for the main function:

    ? visstat

# Getting Started

The function `visstat()` accepts input in two ways:

    # Standardised form (recommended):
    visstat(x, y)

    # Backward-compatible form:
    visstat(dataframe, "name_of_y", "name_of_x")

In the standardised form, `x` and `y` must be vectors of class
`"numeric"`, `"integer"`, or `"factor"`.

In the backward-compatible form, `"name_of_x"` and `"name_of_y"` must be
character strings naming columns in a `data.frame` named `dataframe`.
These column must be of class `"numeric"`, `"integer"`, or `"factor"`.
This is equivalent to writing:

    visstat(dataframe[["name_of_x"]], dataframe[["name_of_y"]])

To simplify the notation, throughout the remainder, data of class
`numeric` or `integer` are both referred to by their common `mode`
**`numeric`**, while data of class `factor` are referred to as
**`categorical`**.

The interpretation of `x` and `y` depends on their classes:

- If one is numeric and the other is a factor, the numeric must be
  passed as response `y` and the factor as predictor `x`. This supports
  tests for central tendencies.

- If both are numeric, a simple linear regression model is fitted with
  `y` as the response and `x` as the predictor.

- If both are factors, a test of association is performed (Chi-squared
  or Fisher’s exact). The test is symmetric, but the plot layout depends
  on which variable is supplied as `x`.

`visstat()` selects the appropriate statistical test and generates
visualisations accompanied by the main test statistics.

# Examples

    library(visStatistics)

## Numerical response and categorical predictor

When the response is numerical and the predictor is categorical, test of
central tendencies are selected.

### Welch two sample t-test

#### InsectSprays data set — both input forms

    insect_sprays_ab <- InsectSprays[InsectSprays$spray %in% c("A", "B"), ]
    insect_sprays_ab$spray <- factor(insect_sprays_ab$spray)

    # Standardised
    visstat(insect_sprays_ab$spray, insect_sprays_ab$count)

<img src="man/figures/README-insect-sprays-1.png" width="100%" />



    # Backward-compatible function call resulting in same output
    # visstat(insect_sprays_ab,"count", "spray")

#### mtcars data set

    mtcars$am <- as.factor(mtcars$am)
    t_test_statistics <- visstat(mtcars$am, mtcars$mpg)

<img src="man/figures/README-mtcars-1.png" width="100%" />

    # t_test_statistics

### Wilcoxon rank sum test

    grades_gender <- data.frame(
      sex = factor(rep(c("girl", "boy"), times = c(21, 23))),
      grade = c(
        19.3, 18.1, 15.2, 18.3, 7.9, 6.2, 19.4, 20.3, 9.3, 11.3,
        18.2, 17.5, 10.2, 20.1, 13.3, 17.2, 15.1, 16.2, 17.0, 16.5, 5.1,
        15.3, 17.1, 14.8, 15.4, 14.4, 7.5, 15.5, 6.0, 17.4, 7.3, 14.3,
        13.5, 8.0, 19.5, 13.4, 17.9, 17.7, 16.4, 15.6, 17.3, 19.9, 4.4, 2.1
      )
    )

    wilcoxon_statistics <- visstat(grades_gender$sex, grades_gender$grade)

<img src="man/figures/README-sex-grades2-1.png" width="100%" />

### One-way test

    one_way_npk <- visstat(npk$block,npk$yield)

<img src="man/figures/README-npk-onewy-1.png" width="100%" /><img src="man/figures/README-npk-onewy-2.png" width="100%" />

### Kruskal-Wallis test

    visstat(iris$Species, iris$Petal.Width)

<img src="man/figures/README-iris-kruskal-1.png" width="100%" /><img src="man/figures/README-iris-kruskal-2.png" width="100%" />
The generated graphs can be saved in all available formats of the
`Cairo` package. Here we save the graphical output of type “pdf” in the
`plotDirectory` `tempdir()`:

    visstat(
    iris$Species,iris$Petal.Width,graphicsoutput = "pdf",plotDirectory = tempdir()
    )

## Numerical response and numerical predictor: Linear Regression

    linreg_cars <- visstat(cars$speed ,cars$dist)

<img src="man/figures/README-lin-reg-dist-speed-1.png" width="100%" /><img src="man/figures/README-lin-reg-dist-speed-2.png" width="100%" />

Increasing the confidence level `conf.level` from the default 0.95 to
0.99 leads two wider confidence and prediction bands:

<img src="man/figures/README-pressure-1.png" width="100%" /><img src="man/figures/README-pressure-2.png" width="100%" />

## Both varibles categorical

### Pearson’s Chi-squared test

Count data sets are often presented as multidimensional arrays, so -
called contingency tables, whereas `visstat()` requires a `data.frame`
with a column structure. Arrays can be transformed to this column wise
structure with the helper function `counts_to_cases()`:

    hair_eye_color_df <- counts_to_cases(as.data.frame(HairEyeColor))
    visstat(hair_eye_color_df$Eye, hair_eye_color_df$Hair)

<img src="man/figures/README-pearson-1.png" width="100%" /><img src="man/figures/README-pearson-2.png" width="100%" />

### Fisher’s exact test

    hair_eye_color_male <- HairEyeColor[, , 1]
    # Slice out a 2 by 2 contingency table
    black_brown_hazel_green_male <- hair_eye_color_male[1:2, 3:4]
    # Transform to data frame
    black_brown_hazel_green_male <- counts_to_cases(as.data.frame(black_brown_hazel_green_male))
    # Fisher test
    fisher_stats <- visstat(black_brown_hazel_green_male$Eye,black_brown_hazel_green_male$Hair)

<img src="man/figures/README-haireye-fisher-1.png" width="100%" /><img src="man/figures/README-haireye-fisher-2.png" width="100%" />

# Decision logic

The choice of statistical tests depends on whether the data of the
selected columns are numeric or categorical, the number of levels in the
categorical variable, and the distribution of the data. The function
prioritizes interpretable visual output and tests that remain valid
under the the following decision logic:

## Numerical response and categorical predictor

When the response is numeric and the predictor is categorical, a
statistical hypothesis test of central tendencies is selected.

- If the categorical predictor has exactly two levels, Welch’s t - test
  (`t.test()`), is applied whenever both groups contain more than 30
  observations, with the validity of the test supported by the
  approximate normality of the sampling distribution of the mean under
  the central limit theorem \[@Rasch:2011vl @Lumley2002dsa\]. For
  smaller samples, group - wise normality is assessed using the
  Shapiro - Wilk test (`shapiro.test()`) at the significance level*α*.
  If both groups are found to be approximately normally distributed
  according to the Shapiro - Wilk test, Welch’s t-test is applied;
  otherwise, the Wilcoxon rank-sum test (`wilcox.test()`) is used.

- For predictors with more than two levels, an ANOVA model (`aov()`) is
  initially fitted. The normality of residuals is evaluated using both
  the Shapiro–Wilk test (`shapiro.test()`) and the Anderson–Darling test
  (`ad.test()`); residuals are considered approximately normal if at
  least one of the two tests yields a result exceeding the significance
  threshold *α*. If this condition is met, Bartlett’s test
  (`bartlett.test()`) is then used to assess homoscedasticity. When
  variances are homogeneous (*p* &gt; *α*), ANOVA is applied with
  Tukey’s HSD (`TukeyHSD()`) for post-hoc comparison. If variances
  differ significantly (*p* ≤ *α*), Welch’s one - way test
  (`oneway.test()`) is used, also followed by Tukey’s HSD. If residuals
  are not normally distributed according to both tests (*p* ≤ *α*), the
  Kruskal-Wallis test (`kruskal.test()`) is selected, followed by
  pairwise Wilcoxon tests (`pairwise.wilcox.test()`). A graphical
  overview of the decision logic used is provided in below figure.

<img src="man/figures/decision_tree.png" width="100%" 
     alt="Decision tree used to select the appropriate statistical test.">

<p style="font-style: italic; font-size: 90%; margin-top: 0.5em;">
Decision tree used to select the appropriate statistical test for a
categorical predictor and numeric response, based on the number of
factor levels, normality, and homoscedasticity.
</p>

## Numerical response and numerical predictor: Linear Regression

When both the response and predictor are numeric, a simple linear
regression model (`lm()`) is fitted and analysed in detail, including
residual diagnostics, formal tests, and the plotting of fitted values
with confidence bands. Note that only one explanatory variable is
allowed, as the function is designed for two-dimensional visualisation.

## Both variables categorical

When both variables are categorical, no direction is assumed (though one
is still referred to as the for consistency). `visstat()` tests the null
hypothesis that both variables are independent using either
`chisq.test()` or `fisher.test()`. The choice of test is based on
Cochran’s rule \[@Cochran\], which advises that
the*χ*<sup>2</sup>approximation is reliable only if no expected cell
count is zero and no more than 20 percent of cells have expected counts
below 5.

For a more detailed description of the underlying decision logic see

    vignette("visStatistics")

# Limitations

The main purpose of this package is a decision-logic based automatic
visualisation of statistical test results. Therefore, except for the
user-adjustable `conf.level` parameter, all statistical tests are
applied using their default settings from the corresponding base R
functions. As a consequence, paired tests are currently not supported
and `visstat()` does not allow to study interactions terms between the
different levels of an independent variable in an analysis of variance.
Focusing on the graphical representation of tests, only simple linear
regression is implemented, as multiple linear regressions cannot be
visualised.

## Implemented tests

### Numerical response and categorical predictor

#### Main tests

`t.test()`, `wilcox.test()`, `aov()`, `oneway.test()`, `kruskal.test()`

#### Normality assumption check

`shapiro.test()` and `ad.test()`

#### Homoscedasticity assumption check

`bartlett.test()`

#### Post-hoc tests-`TukeyHSD()` (used following `aov()`and `oneway.test()`)

- `pairwise.wilcox.test()` (used following `kruskal.test()`)

### Numerical response and numerical predictor

When both the response and predictor are numerical, a simple linear
regression model is fitted:`lm()`

Note that multiple linear regression models are not implemented, as the
package focuses on the visualisation of data, not model building. \###
Categorical response and categorical predictor

When both variables are categorical, `visstat()` tests the null
hypothesis of independence using one of the following:-`chisq.test()`
(default for larger samples) - `fisher.test()` (used for small expected
cell counts based on Cochran’s rule) <!-- pkgdown::end -->

<!-- pkgdown::start -->

# visStatistics: The right test, visualised.

<!-- Imagine you have a data base, which for confidentiality reasons, can not be directly accessed by collaborating researchers. But you want to give your collaborators to give  the possibility, via a web interface, to slice out groups of the data set, visualize it and get a first statistical analysis. This was the task for which `visstatistics` was originally developed: a fully automated  "quick and dirty"  visualisation and statistical analysis tool, particularly suited for browser-based interfaces that -->

<!-- rely on  server-side R applications connected to secure databases, where users have no direct accesss.  -->

<!-- The R package `visStatistics` allows for rapid **vis**ualisation and statistical -->

<!-- analysis of raw data. It automatically selects and visualises the most -->

<!-- appropriate **statistic**al hypothesis test between two vectors of class -->

<!-- `integer`, `numeric` or `factor`. -->

<!-- (an input vector of class -->

<!-- `character` will be internally transformed to class `factor`). -->

<!-- While numerous R packages provide statistical testing functionality, few are -->

<!-- designed with pedagogical accessibility as a primary concern.  -->

<!-- `visStatistics`  -->

<!-- addresses this gap -->

<!-- by automating the test selection process and presenting results using annotated, -->

<!-- publication-ready visualisations.  -->

<!-- This helps the user to focus on interpretation rather than technical execution. -->

<!-- Its easy access should be of extreme use  -->

# Purpose

`visStatistics` is an R package for automated statistical test selection
and visualisation. It selects appropriate hypothesis tests for pairs of
variables (integer, numeric, or factor), runs the analysis, and produces
annotated, publication-ready plots.

The package was originally developed for researchers who could not
directly access sensitive data but needed to perform visualisations and
statistical analyses of selected groups via a web interface connected to
a protected database — a setting that required a fully automated
workflow. The same automation makes it useful in time-constrained
contexts such as statistical consulting, where it reduces effort spent
on test selection and leaves more room for interpretation. The package
covers most hypothesis tests taught in undergraduate statistics.

# Installation of latest stable version from CRAN

#### 1. Install the package

``` r
install.packages("visStatistics")
```

#### 2. Load the package

``` r
library(visStatistics)
```

# Installation of the development version from GitHub

#### 1.Install `devtools` from CRAN if not already installed:

``` r
install.packages("devtools")
```

#### 2. Load the `devtools` package:

``` r
library(devtools)
```

#### 3. Install the `visStatistics` package from GitHub:

``` r
pak::pak("shhschilling/visStatistics")
```

#### 4. Load the `visStatistics` package:

``` r
library(visStatistics)
```

#### 5. View help for the main function:

``` r
? visstat
```

#### 6. Study all the details in the packages’ vignette:

``` r
vignette("visStatistics")
```

# Getting Started

The function `visstat()` accepts input in three ways:

``` r
# Standardised form:
visstat(x, y)

# Formula interface: 
visstat(y ~ x, data = df) 


# Backward-compatible form:
visstat(dataframe, "namey", "namex")
```

In the standardised form, `x` and `y` must be vectors of class
`"numeric"`, `"integer"`, or `"factor"`.

In the formula interface, the formula `y ~ x` specifies the response `y`
and predictor `x` variables, and `data` is a data frame containing these
variables.

In the backward-compatible form, `"namex"` and `"namey"` must be
character strings naming columns in a `data.frame` named `dataframe`.
These column must be of class `"numeric"`, `"integer"`, or `"factor"`.
This is equivalent to writing:

``` r
visstat(dataframe[["namex"]], dataframe[["namey"]])
```

To simplify the notation, throughout the remainder, data of class
`numeric` or `integer` are both referred to by their common `mode`
**`numeric`**, while data of class `factor` are referred to as
**`categorical`**.

The interpretation of `x` and `y` depends on their classes:

- If one is numeric and the other is a factor, the numeric must be
  passed as response `y` and the factor as predictor `x`. This supports
  tests for central tendencies.

- If both are numeric, a simple linear regression model is fitted with
  `y` as the response and `x` as the predictor.

- If both are factors (and not both ordered), a test of association is
  performed (Chi-squared or Fisher’s exact). The test is symmetric, but
  the plot layout depends on which variable is supplied as `x`.

- If both are factors and `y` is additionally of class `ordered`, a non
  parametric test is performed: a Wilcoxon test in the case of two
  factor levels in `x`, a Kruskal-Wallis-Test for more than two factor
  levels in `x`.

- If **both** `x` and `y` are of class `ordered`, the order of the
  levels carries information that Chi-squared / Fisher would discard.
  `visstat()` instead tests for a monotone association via Kendall’s
  $\tau_b$ (`cor.test(..., method = "kendall")`) and visualises the
  relationship with a jittered rank–rank scatter plus a mosaic plot.

# Decision logic

The choice of statistical tests depends on whether the data of the
selected columns are numeric or categorical, the number of levels in the
categorical variable, and the distribution of the data.  
For a detailed description of the underlying decision logic, please
refer to the package vignette:

``` r
vignette("visStatistics")
```

Below figure gives a graphical overview of the main tests implemented.

<div style="border: 1px solid #666; padding: 10px; display: inline-block; text-align: left;">

<img src="vignettes/figures/overview.png" width="100%" 
     alt="Decision logic of main tests.">

<p style="font-style: italic; font-size: 90%; margin-top: 0.5em;">

Overview of the implemented statistical tests based on the class of the
variables.

</p>

</div>

A graphical summary of the decision logic used for categorial predictors
and numerical responses resulting in tests of central tendencies is
given in below figure:

<div style="border: 1px solid #666; padding: 10px; display: inline-block; text-align: left;">

<img src="vignettes/figures/decision_tree.png" width="100%" 
     alt="Decision tree used to select the appropriate statistical test.">

<p style="font-style: italic; font-size: 90%; margin-top: 0.5em;">

Comparing central tendencies: Decision tree used to select the
appropriate statistical test for a categorical predictor and numeric
response, based on the number of factor levels, normality, and
homoscedasticity.

</p>

</div>

# Examples

``` r
library(visStatistics)
```

## Numerical response and categorical predictor

When the response is numerical and the predictor is categorical, test of
central tendencies are selected.

### Welch two sample t-test

<!-- #### InsectSprays data set — both input forms -->

<!-- ```{r insect-sprays, eval = TRUE} -->

<!-- insect_sprays_ab <- InsectSprays[InsectSprays$spray %in% c("A", "B"), ] -->

<!-- insect_sprays_ab$spray <- factor(insect_sprays_ab$spray) -->

<!-- visstat(insect_sprays_ab$spray, insect_sprays_ab$count) -->

<!-- ``` -->

#### mtcars data set

``` r
 mtcars$am <- as.factor(mtcars$am)
 t_test_statistics <- visstat(mtcars$am, mtcars$mpg)
```

<img src="man/figures/README-mtcars-1.png" alt="Welch two-sample t-test comparing fuel efficiency (mpg) between automatic and manual transmission. Shows boxplot with individual points, statistical results, and diagnostic plots for normality and variance homogeneity." width="80%" /><img src="man/figures/README-mtcars-2.png" alt="Welch two-sample t-test comparing fuel efficiency (mpg) between automatic and manual transmission. Shows boxplot with individual points, statistical results, and diagnostic plots for normality and variance homogeneity." width="80%" /><img src="man/figures/README-mtcars-3.png" alt="Welch two-sample t-test comparing fuel efficiency (mpg) between automatic and manual transmission. Shows boxplot with individual points, statistical results, and diagnostic plots for normality and variance homogeneity." width="80%" />

### Wilcoxon rank sum test

``` r
grades_gender <- data.frame(
  sex = factor(rep(c("girl", "boy"), times = c(21, 23))),
  grade = c(
    19.3, 18.1, 15.2, 18.3, 7.9, 6.2, 19.4, 20.3, 9.3, 11.3,
    18.2, 17.5, 10.2, 20.1, 13.3, 17.2, 15.1, 16.2, 17.0, 16.5, 5.1,
    15.3, 17.1, 14.8, 15.4, 14.4, 7.5, 15.5, 6.0, 17.4, 7.3, 14.3,
    13.5, 8.0, 19.5, 13.4, 17.9, 17.7, 16.4, 15.6, 17.3, 19.9, 4.4, 2.1
  )
)

wilcoxon_statistics <- visstat(grades_gender$sex, grades_gender$grade)
```

<img src="man/figures/README-sex-grades2-1.png" alt="Wilcoxon rank sum test comparing grades between girls and boys. Displays boxplot with overlaid individual points, test statistics, and diagnostic plots for assessing distributional differences." width="80%" /><img src="man/figures/README-sex-grades2-2.png" alt="Wilcoxon rank sum test comparing grades between girls and boys. Displays boxplot with overlaid individual points, test statistics, and diagnostic plots for assessing distributional differences." width="80%" />

### Fisher’s one way ANOVA

``` r
fisher_one_way_npk <- visstat(npk$block,npk$yield)
```

<img src="man/figures/README-npk-onewy-1.png" alt="Fisher's one-way ANOVA comparing crop yield across different experimental blocks. Shows boxplot for each block with individual data points, ANOVA results, and post-hoc comparisons with confidence intervals." width="80%" /><img src="man/figures/README-npk-onewy-2.png" alt="Fisher's one-way ANOVA comparing crop yield across different experimental blocks. Shows boxplot for each block with individual data points, ANOVA results, and post-hoc comparisons with confidence intervals." width="80%" />

### Kruskal-Wallis test

``` r
kruskal_iris=visstat(iris$Species, iris$Petal.Width)
```

<img src="man/figures/README-iris-kruskal-1.png" alt="Kruskal-Wallis test comparing petal width across three iris species. Displays boxplot with individual observations for each species, test statistics, and post-hoc pairwise comparisons." width="80%" /><img src="man/figures/README-iris-kruskal-2.png" alt="Kruskal-Wallis test comparing petal width across three iris species. Displays boxplot with individual observations for each species, test statistics, and post-hoc pairwise comparisons." width="80%" />

## Ordered response and categorical predictor:

A response variable of classes `ordered` and `factor` is internally
transformed to a `numeric` response and Wilcoxon test is performed for a
predictor with two classes, otherwise a Kruskal-Wallis-Test.

### Wilcoxon with ordinal response

``` r
set.seed(123)

# Create predictor: Customer segment (2 groups)
segment <- factor(rep(c("Budget", "Premium"), each = 50))

# Create response: Likert scale ratings (1-5)
satisfaction_numeric <- c(
  sample(1:5, 50, replace = TRUE, prob = c(0.15, 0.25, 0.30, 0.20, 0.10)),  # Budget
  sample(1:5, 50, replace = TRUE, prob = c(0.05, 0.10, 0.20, 0.35, 0.30))   # Premium
)

# Create dataframe with ORDERED response
survey_data <- data.frame(
  segment = segment,
  satisfaction = ordered(satisfaction_numeric)  # Declare as ordered
)

# triggers warnings and use Wilcoxon test
result <- visstat(survey_data$segment,survey_data$satisfaction)
```

<img src="man/figures/README-ordinal-1.png" alt="Wilcoxon rank sum test comparing satisfaction ratings (Likert scale 1-5) between budget and premium customer segments. Shows distribution of ordinal responses with test statistics and visualisations of rank differences." width="80%" />

### Kruskal-Wallis test

``` r
set.seed(123)

# Create predictor: Service class (3 groups)
service_class <- factor(rep(c("Economy", "Business", "First"), each = 50))

# Create response: Likert scale ratings (1-5)
comfort_numeric <- c(
  sample(1:5, 50, replace = TRUE, prob = c(0.35, 0.30, 0.20, 0.10, 0.05)),  # Economy
  sample(1:5, 50, replace = TRUE, prob = c(0.10, 0.20, 0.35, 0.25, 0.10)),  # Business
  sample(1:5, 50, replace = TRUE, prob = c(0.05, 0.10, 0.20, 0.30, 0.35))   # First
)

# Create dataframe with ordered response
flight_data <- data.frame(
  service_class = service_class,
  comfort = ordered(comfort_numeric)  # Declare as ordered
)

# triggers warning and uses Kruskal-Wallis test
result <- visstat(flight_data$service_class, flight_data$comfort)
```

<img src="man/figures/README-unnamed-chunk-3-1.png" alt="Kruskal-Wallis test comparing comfort ratings (Likert scale 1-5) across three airline service classes: economy, business, and first. Shows distribution of ordinal responses with test statistics and post-hoc pairwise rank comparisons." width="80%" />

## Numerical response and numerical predictor: Linear Regression

``` r
vis_women <- visstat(women$height, women$weight,conf.level=0.99)
```

<img src="man/figures/README-lin-reg-dist-speed-1.png" alt="Linear regression analysis of weight versus height for women. Shows scatter plot with fitted regression line, 99% confidence band for the regression line, and prediction band. Includes residual plot and Q-Q plot for diagnostic assessment." width="80%" /><img src="man/figures/README-lin-reg-dist-speed-2.png" alt="Linear regression analysis of weight versus height for women. Shows scatter plot with fitted regression line, 99% confidence band for the regression line, and prediction band. Includes residual plot and Q-Q plot for diagnostic assessment." width="80%" />

### Pearson’s Chi-squared test

Count data sets are often presented as multidimensional arrays, so -
called contingency tables, whereas `visstat()` requires a `data.frame`
with a column structure. Arrays can be transformed to this column wise
structure with the helper function `counts_to_cases()`:

``` r
hair_eye_color_df <- counts_to_cases(as.data.frame(HairEyeColor))
visstat(hair_eye_color_df$Eye, hair_eye_color_df$Hair)
```

<img src="man/figures/README-pearson-1.png" alt="Pearson's Chi-squared test for independence between hair and eye colour. Shows bar chart of observed counts by colour combination, mosaic plot displaying proportional cell frequencies with Pearson residuals, and test statistics for association." width="80%" /><img src="man/figures/README-pearson-2.png" alt="Pearson's Chi-squared test for independence between hair and eye colour. Shows bar chart of observed counts by colour combination, mosaic plot displaying proportional cell frequencies with Pearson residuals, and test statistics for association." width="80%" />

### Fisher’s exact test

``` r
hair_eye_color_male <- HairEyeColor[, , 1]
# Slice out a 2 by 2 contingency table
black_brown_hazel_green_male <- hair_eye_color_male[1:2, 3:4]
# Transform to data frame
black_brown_hazel_green_male <- counts_to_cases(as.data.frame(black_brown_hazel_green_male))
# Fisher test
fisher_stats <- visstat(black_brown_hazel_green_male$Eye,black_brown_hazel_green_male$Hair)
```

<img src="man/figures/README-haireye-fisher-1.png" alt="Fisher's exact test for 2x2 contingency table of hair and eye colour in males (black/brown hair versus hazel/green eyes). Displays observed frequencies, mosaic plot showing cell proportions, and exact test statistics for association with small sample sizes." width="80%" />

## Both variables ordered: Rank correlation

When both predictor and response are of class `ordered`, `visstat()`
tests for a monotone association via Kendall’s $\tau_b$ and produces a
jittered rank–rank scatter together with a mosaic plot.

``` r
set.seed(1)
n <- 120
xs <- sample(1:5, n, replace = TRUE)
ys <- pmin(5, pmax(1, xs + sample(-1:1, n, replace = TRUE)))
likert_levels <- c("strongly disagree", "disagree", "neutral",
                   "agree", "strongly agree")
attitude  <- ordered(likert_levels[xs], levels = likert_levels)
intention <- ordered(likert_levels[ys], levels = likert_levels)
kendall_result <- visstat(intention, attitude)
```

<img src="man/figures/README-kendall-ordered-1.png" alt="Kendall's tau-b rank correlation between two five-point Likert items. Shows a jittered rank-rank scatter annotated with tau-b, the p-value, and sample size, plus a mosaic plot whose tile ordering reflects the level ordering of the ordered factors." width="80%" />

# Saving the graphical output

All generated graphics can be saved in any file format supported by
`Cairo()`, including “png”, “jpeg”, “pdf”, “svg”, “ps”, and “tiff” in
the user specified `plotDirectory`.

If the optional argument `plotName` is not given, the naming of the
output follows the pattern `"testname_namey_namex."`, where `"testname"`
specifies the selected test or visualisation and `"namey"` and `"namex"`
are character strings naming the selected data vectors `y` and `x`,
respectively. The suffix corresponding to the chosen `graphicsoutput`
(e.g., `"pdf"`, `"png"`) is then concatenated to form the complete
output file name.

In the following example, we store the graphics in `png` format in the
`plotDirectory` `tempdir()` with the default naming convention:

``` r
#Graphical output written to plotDirectory: In this example 
# a bar chart to visualise the Chi-squared test and mosaic plot showing
# Pearson's residuals named 
#chi_squared_or_fisher_Hair_Eye.png and mosaic_complete_Hair_Eye.png resp. 
save_fisher <- visstat(black_brown_hazel_green_male, "Hair", "Eye",
        graphicsoutput = "png", plotDirectory = tempdir())
```

The full file path of the generated graphics are stored as the attribute
`"plot_paths"` on the returned object of class `"visstat"`.

``` r
paths <- attr(save_fisher, "plot_paths")
print(paths)
#> [1] "/var/folders/5c/n85wqnh95l50qbp3s9l0rp_w0000gn/T//RtmptoHBnl/chi_squared_or_fisher_Hair_Eye.png"
```

Remove the graphical output from `plotDirectory`:

``` r
file.remove(paths)
#> [1] TRUE
```

<!-- ## Implemented tests -->

<!-- ### Numerical response and categorical predictor -->

<!-- #### Main tests -->

<!-- ` t.test()`, `wilcox.test()`, `lm()` ,`aov()`, `oneway.test()`, `kruskal.test()` -->

<!-- #### Normality assumption check -->

<!-- `shapiro.test()` and `ad.test()` -->

<!-- #### Homoscedasticity assumption check -->

<!-- `levene.test()` and `bartlett.test()`for ANOVA and `bp.test()` for linear regression -->

<!-- #### Post-hoc tests -->

<!-- -`TukeyHSD()` following `aov()`, -->

<!-- -`games.howell()` following `oneway.test()`, -->

<!-- -`pairwise.wilcox.test()` following `kruskal.test()`. -->

<!-- ### Numerical response and predictor -->

<!-- `lm()`, `cor()` -->

<!-- ### Both variables categorical -->

<!-- `chisq.test()`, `fisher.test()`  -->

## ----setup, include=FALSE-----------------------------------------------------
library(visStatistics)
  knitr::opts_chunk$set(
    fig.width = 7.5,      # Smaller device = smaller text
    fig.height = 5,
    out.width = '100%')

## ----install-github, eval = FALSE---------------------------------------------
# install_github("shhschilling/visStatistics")

## ----load, eval = FALSE-------------------------------------------------------
# library(visStatistics)

## ----npk, fig.show='hide'-----------------------------------------------------
#Standardised form
visstat(npk$block,npk$yield)
# Using formula interface
 visstat(yield~block,data=npk)
#Backward-compatible form
 visstat(npk,"yield","block")

## ----fig-overview, echo=FALSE, results='asis'---------------------------------
if (knitr::opts_knit$get("rmarkdown.pandoc.to") == "html") {
  cat('
<div style="border: 1px solid #666; padding: 10px; display: inline-block; text-align: center;">
  <img src="figures/overview.png" width="100%"
       alt="Overview of implemented tests .">
  <p style="font-style: italic; font-size: 90%; margin-top: 0.5em;">
    Overview of the implemented statistical tests based on the class of the variables.
  </p>
</div>
')
} else {
  cat('
\\begin{center}
\\fbox{%
  \\begin{minipage}{0.95\\linewidth}
    \\centering
    \\includegraphics[width=\\linewidth]{figures/overview.png}\\\\
    \\vspace{0.5em}
    \\textit{Decision tree used to select the appropriate statistical test for a categorical predictor and numeric response, based on the number of factor levels, normality, and homoscedasticity.}
  \\end{minipage}
}
\\end{center}
')
}

## ----fig-decision-switch, echo=FALSE, results='asis'--------------------------
if (knitr::opts_knit$get("rmarkdown.pandoc.to") == "html") {
  cat('
<div style="border: 1px solid #666; padding: 10px; display: inline-block; text-align: center;">
  <img src="figures/decision_tree.png" width="100%"
       alt="Decision tree used to select the appropriate statistical test.">
  <p style="font-style: italic; font-size: 90%; margin-top: 0.5em;">
    Decision tree used to select the appropriate statistical test for a categorical
    predictor and numeric response, based on the sample sizes, number of factor levels, normality,
    and homoscedasticity.
  </p>
</div>
')
} else {
  cat('
\\begin{center}
\\fbox{%
  \\begin{minipage}{0.95\\linewidth}
    \\centering
    \\includegraphics[width=\\linewidth]{figures/decision_tree.png}\\\\
    \\vspace{0.5em}
    \\textit{Decision tree used to select the appropriate statistical test for a categorical predictor and numeric response, based on the sample sizes, number of factor levels, normality, and homoscedasticity.}
  \\end{minipage}
}
\\end{center}
')
}

## -----------------------------------------------------------------------------
mtcars$am <- as.factor(mtcars$am)
t_test_statistics <- visstat(mtcars$am, mtcars$mpg)

## -----------------------------------------------------------------------------
grades_gender <- data.frame(
  sex = as.factor(c(rep("girl", 21), rep("boy", 23))),
  grade = c(
    19.3, 18.1, 15.2, 18.3, 7.9, 6.2, 19.4,
    20.3, 9.3, 11.3, 18.2, 17.5, 10.2, 20.1, 13.3, 17.2, 15.1, 16.2, 17.0,
    16.5, 5.1, 15.3, 17.1, 14.8, 15.4, 14.4, 7.5, 15.5, 6.0, 17.4,
    7.3, 14.3, 13.5, 8.0, 19.5, 13.4, 17.9, 17.7, 16.4, 15.6, 17.3, 19.9, 4.4, 2.1
  )
)

wilcoxon_statistics <- visstat(grades_gender$sex, grades_gender$grade)

## ----ordinal, fig.show='hide'-------------------------------------------------
set.seed(123)

# Create predictor: Customer segment (2 groups)
segment <- factor(rep(c("Budget", "Premium"), each = 50))

# Create response: Likert scale ratings (1-5)
satisfaction_numeric <- c(
  sample(1:5, 50, replace = TRUE, prob = c(0.15, 0.25, 0.30, 0.20, 0.10)),  # Budget
  sample(1:5, 50, replace = TRUE, prob = c(0.05, 0.10, 0.20, 0.35, 0.30))   # Premium
)

# Create dataframe with ORDERED response
survey_data <- data.frame(
  segment = segment,
  satisfaction = ordered(satisfaction_numeric)  # Declare as ordered
)

# triggers warnings and use Wilcoxon test
wilcox_ordered <- visstat(survey_data, "satisfaction", "segment")

## -----------------------------------------------------------------------------
anova_npk <- visstat(npk$block,npk$yield,conf.level=0.95)

## ----results='hide'-----------------------------------------------------------
visstat(iris$Species, iris$Petal.Width)

## ----ordinal-kruskal, fig.show='hide'-----------------------------------------
set.seed(123)

# Predictor: customer segment (3 groups)
segment <- factor(rep(c("Budget", "Standard", "Premium"), each = 50))

# Response: Likert scale ratings (1-5), with a deliberate trend across segments
comfort_numeric <- c(
  sample(1:5, 50, replace = TRUE, prob = c(0.30, 0.30, 0.20, 0.15, 0.05)),  # Budget
  sample(1:5, 50, replace = TRUE, prob = c(0.10, 0.20, 0.40, 0.20, 0.10)),  # Standard
  sample(1:5, 50, replace = TRUE, prob = c(0.05, 0.10, 0.20, 0.35, 0.30))   # Premium
)

# Dataframe with ORDERED response
survey_data_3 <- data.frame(
  segment = segment,
  comfort = ordered(comfort_numeric)  # Declare as ordered
)

# triggers warning and uses Kruskal-Wallis test
kruskal_ordered <- visstat(survey_data_3, "comfort", "segment")

## -----------------------------------------------------------------------------
linreg_swiss <- visstat(swiss$Examination, swiss$Fertility, conf.level = 0.99)

## ----fig.show='hide'----------------------------------------------------------
result_ozone0 <- visstat(airquality$Wind, airquality$Ozone)

## -----------------------------------------------------------------------------
result_ozone1 <- visstat(airquality$Wind, airquality$Ozone, correlation = TRUE)

## -----------------------------------------------------------------------------
# Remove zeros to satisfy Gamma requirements
airquality_clean <- subset(airquality, Ozone > 0)
# Gamma model with log mapping
model_gamma <- glm(Ozone ~ Wind, data = airquality_clean, family = Gamma(link = "log"))
summary(model_gamma)

## ----echo=FALSE---------------------------------------------------------------
# Plotting the data with Gamma model overlay
plot(airquality_clean$Wind, airquality_clean$Ozone, log = "y", 
     pch = 19, col = "darkgrey", xlab = "Wind (mph)", ylab = "Ozone (ppb) [log scale]",
     main = "Gamma GLM Fit (Log Link)")
# Generate predictions for the overlay
wind_seq <- seq(min(airquality_clean$Wind), max(airquality_clean$Wind), length.out = 100)
preds <- predict(model_gamma, newdata = data.frame(Wind = wind_seq), type = "response")
lines(wind_seq, preds, col = "red", lwd = 2)

legend("topright", legend = c("Data", "Gamma GLM (log link)"), 
       col = c("darkgrey", "red"), pch = c(19, NA), lty = c(NA, 1), lwd = c(NA, 2))

## -----------------------------------------------------------------------------
# Extract standardised  residuals
std_dev_res <- rstandard(model_gamma, type = "deviance")
# Validate using the Shapiro-Wilk normality test
shapiro.test(std_dev_res)
# Validate using the Anderson-Darling normality test
nortest::ad.test(std_dev_res)

## -----------------------------------------------------------------------------
HairEyeColourDataFrame <- counts_to_cases(as.data.frame(HairEyeColor))

## -----------------------------------------------------------------------------
hair_eye_colour_df <- counts_to_cases(as.data.frame(HairEyeColor))
visstat(hair_eye_colour_df$Eye, hair_eye_colour_df$Hair)

## -----------------------------------------------------------------------------
hair_black_brown_eyes_brown_blue <- HairEyeColor[1:2, 1:2, ]
# Transform to data frame
hair_black_brown_eyes_brown_blue_df <- counts_to_cases(as.data.frame(hair_black_brown_eyes_brown_blue))
# Chi-squared test with Yates' continuity correction

visstat(hair_black_brown_eyes_brown_blue_df$Eye, hair_black_brown_eyes_brown_blue_df$Hair)

## ----fisher-data-prep---------------------------------------------------------
hair_eye_colour_male <- HairEyeColor[, , 1]
# Slice out a 2 by 2 contingency table
black_brown_hazel_green_male <- hair_eye_colour_male[1:2, 3:4]
# Transform to data frame
black_brown_hazel_green_male <- counts_to_cases(as.data.frame(black_brown_hazel_green_male))
# Fisher test
fisher_stats <- visstat(black_brown_hazel_green_male$Eye, black_brown_hazel_green_male$Hair)

## ----fisher-or----------------------------------------------------------------
fisher_stats$estimate   # odds ratio
fisher_stats$conf.int   # 95% CI

## ----kendall-example----------------------------------------------------------
set.seed(42)
n <- 150
# Latent scores with deliberate negative monotone association:
# higher alcohol consumption (xs) -> lower academic performance (ys)
xs <- sample(1:5, n, replace = TRUE)
ys <- pmin(5, pmax(1, (6 - xs) + sample(-1:1, n, replace = TRUE)))
likert_levels  <- c("never", "rarely", "sometimes", "often", "always")
likert_levels2 <- c("poor", "fair", "ok", "good", "great")
alcohol     <- ordered(likert_levels[xs],  levels = likert_levels)
performance <- ordered(likert_levels2[ys], levels = likert_levels2)
kendall_result <- visstat(performance, alcohol, correlation = TRUE)

## ----fisher-save-graphics-----------------------------------------------------
# Graphical output written to plotDirectory: In this example
# a single bar chart showing absolute counts.
# Output file: chi_squared_or_fisher_Hair_Eye.png
save_fisher = visstat(black_brown_hazel_green_male$Eye, black_brown_hazel_green_male$Hair,
        graphicsoutput = "png", plotDirectory = tempdir())

## ----show-path2---------------------------------------------------------------
paths <- attr(save_fisher, "plot_paths")
print(paths)

## ----eval=TRUE----------------------------------------------------------------
file.remove(paths)

## ----visstat-methods, fig.show='hide', results='hide'-------------------------
iris_kruskal <- visstat(iris$Species, iris$Petal.Width)

## ----visstat-methods-print-summary--------------------------------------------
print(iris_kruskal)
summary(iris_kruskal)

## ----visstat-methods-plot-paths-----------------------------------------------
iris_kruskal_stored <- visstat(iris$Species, iris$Petal.Width,
                               graphicsoutput = "pdf",
                               plotName = "iris_kruskal",
                               plotDirectory = tempdir())
plot(iris_kruskal_stored)

## ----visstat-methods-plot-replay----------------------------------------------
plot(iris_kruskal)

## ----visstat-methods-cleanup, echo = FALSE------------------------------------
file.remove(attr(iris_kruskal_stored, "plot_paths"))

#' ANOVA or Welch's ANOVA with appropriate post-hoc tests
#'
#' Internal function that performs ANOVA or Welch's one-way test and 
#' corresponding post-hoc comparisons. Uses TukeyHSD for equal variances 
#' (Fisher's ANOVA) and Games-Howell for unequal variances (Welch's ANOVA).
#'
#' @param samples Numeric vector; the dependent variable.
#' @param fact Factor; the grouping variable.
#' @param conf.level Numeric; confidence level for tests and intervals (default: 0.95).
#' @param samplename Character; label for y-axis (default: "").
#' @param factorname Character; label for x-axis (default: "").
#' @param cex Numeric; character expansion factor for plot elements (default: 1).
#'
#' @return A list with components:
#' \describe{
#'   \item{summary statistics of ANOVA}{Summary of Fisher's ANOVA or Welch's oneway test}
#'   \item{post-hoc analysis}{TukeyHSD object or Games-Howell results in compatible format}
#'   \item{conf.level}{The confidence level used}
#' }
#'
#' @details
#' The function first tests for homogeneity of variance using Levene's test.
#' If variances are equal (p > alpha), Fisher's one-way ANOVA with Tukey's HSD
#' post-hoc is performed. If variances are unequal (p <= alpha), Welch's
#' one-way ANOVA with Games-Howell post-hoc is performed.
#'
#' The function produces a box plot with jittered points and group means
#' (red diamonds for the parametric branches), annotated with a compact
#' letter display showing which groups differ significantly.
#' @examples
#' # Example with equal variances (uses Fisher's ANOVA + TukeyHSD)
#' data(PlantGrowth)
#' result1 <- vis_anova(PlantGrowth$weight, PlantGrowth$group, 
#'                      samplename = "Weight", factorname = "Group")
#' 
#' # Example with unequal variances (uses Welch's ANOVA + Games-Howell)
#' # Create data with heterogeneous variances
#' set.seed(123)
#' group_a <- rnorm(20, mean = 10, sd = 1)
#' group_b <- rnorm(20, mean = 15, sd = 5)  # Much larger variance
#' group_c <- rnorm(20, mean = 12, sd = 2)
#' values <- c(group_a, group_b, group_c)
#' groups <- factor(rep(c("A", "B", "C"), each = 20))
#' result2 <- vis_anova(values, groups, 
#'                      samplename = "Value", factorname = "Group")
#'
#' @export

vis_anova <- function(samples,
                      fact,
                      conf.level = conf.level,
                      samplename = "",
                      factorname = "",
                      cex = 1) {
  if (missing(conf.level)) {
    conf.level <- 0.95
  }
  
  oldparanova <- par(no.readonly = TRUE)
  on.exit(par(oldparanova))
  
  alpha <- 1 - conf.level
  
  samples3 <- na.omit(samples)
  fact <- subset(fact, !is.na(samples))
  samples <- samples3
  n_classes <- length(unique(fact))

  meanna <- function(x) {
    mean(x, na.rm = T)
  }

  m <- tapply(samples, fact, meanna)
  # tests
  an <- aov(samples ~ fact)
  summaryAnova <- summary(an)
  oneway <- oneway.test(samples ~ fact)
  # check for homogeneity
  bartlett_test <- bartlett.test(samples ~ fact)
  p_bart <- bartlett_test$p.value
  levene_test <- levene.test(samples,fact)
  p_levene <- levene_test$p.value
  
  
  
  if (p_levene > 1 - conf.level)
  {
    p_aov <- summaryAnova[[1]][["Pr(>F)"]][1]
    F_value <- sprintf("%.2f", summaryAnova[[1]]$`F value`[1])
    label_aov <- "Fisher's one-way ANOVA"
    summarystat <- summaryAnova
    post_hoc_anova <- TukeyHSD(an, conf.level = conf.level)
  } else {
    # Unequal variances - use Welch's ANOVA with Games-Howell post-hoc
    p_aov <- oneway$p.value
    F_value <- sprintf("%.2f", oneway$statistic)
    label_aov <- "Welch's one-way ANOVA"
    summarystat <- oneway
    
    # Use Games-Howell for post-hoc (correct for unequal variances)
    gh_result <- games.howell(samples, fact, conf.level = conf.level)
    
    # Convert to format needed by multcompLetters (line 846)
    comparison_names <- paste0(gh_result$group2, "-", gh_result$group1)
    result_matrix <- cbind(
      diff = gh_result$mean_diff,
      lwr = gh_result$ci_lower,
      upr = gh_result$ci_upper,
      `p adj` = gh_result$p_adj
    )
    rownames(result_matrix) <- comparison_names
    post_hoc_anova <- list(fact = result_matrix)
  }
  
  
  
  
  # Y-axis limits (with extra lower margin for legend space)
  lower_margin <- 0.45
  upper_margin <- 0.2
  margins <- calc_min_max_of_y_axis(samples, lower_margin, upper_margin)
  mi <- margins[[1]]
  ma <- margins[[2]]

  # For legend positioning (spread from actual plot range)
  spread <- ma - mi


  box_cols <- rep_len(c(colorscheme(1), colorscheme(3)), n_classes)

  show_jitter  <- max(tapply(samples, fact, length)) <= jitter_max_n

  b <- boxplot(
    samples ~ fact,
    xlim = c(0, n_classes + 1),
    ylim = c(mi, ma),
    col = box_cols,
    ylab = samplename,
    xlab = factorname,
    las = 2,
    outline = !show_jitter  # outliers via boxplot only when jitter is suppressed
  )

  if (show_jitter) {
    stripchart(
      samples ~ fact,
      vertical = TRUE,
      method = "jitter",
      col = adjustcolor(box_cols, red.f = 0.55, green.f = 0.55, blue.f = 0.55),
      pch = 1,
      cex = 0.7,
      add = TRUE
    )
  }

  # Group means -- parametric branch tests means, so mark them explicitly
  points(seq_len(n_classes), m,
         pch = 16, col = "red", cex = 1.3)

  if (n_classes > 6) {
    n_labels <- c(paste("N =", b$n[1]), as.character(b$n[-1]))
  } else {
    n_labels <- paste("N =", b$n)
  }
  text(seq_len(n_classes), ma, n_labels)
  
  
  
  s <- multcompLetters(post_hoc_anova[[1]][, 4], threshold = alpha)
  
  ord <- c()
  
  v <- attributes(s$Letters)$names
  f_levels <- sort(unique(fact))
  for (i in 1:n_classes) {
    ord[i] <- which(v == f_levels[i])
  }
  
  text(seq(1:n_classes + 1),
       mi,
       s$Letters[ord],
       col = colors()[81],
       lwd = 2)
  
  
  mtext(
    paste0(label_aov, "\nF = ", F_value, ", p = ", signif(p_aov, 2)))



  # Legend: mean marker (top) + significance letters with post-hoc method
  # and alpha (split on two lines for compactness in narrow windows).
  # All post-hoc / alpha info lives here, so the title can stay compact.
  posthoc_name <- ifelse(p_levene > alpha, "Tukey's HSD", "Games-Howell")
  legend("bottomleft",
         legend = c("group mean",
                    "a, b, ...: significance letters",
                    paste0("(", posthoc_name, ", alpha = ", signif(alpha, 2), ")")),
         pch = c(16, NA, NA),
         col = c("red", NA, NA),
         text.col = c("red", colors()[81], colors()[81]),
         bty = "n",
         cex = 0.9,
         inset = 0.08)
  
  
  
  
  my_list <-
    list(
      # "summary statistics of Fisher's one-way ANOVA" = summaryAnova,
      # "summary statistics of Welch's one-way ANOVA (not assuming equal  variances)" = oneway,
      "summary statistics of ANOVA" = summarystat,
      "post-hoc analysis " = post_hoc_anova,
      "conf.level" = conf.level
    )
  
  return(my_list)
}

#' Games-Howell Post-Hoc Test
#'
#' Performs pairwise comparisons using the Games-Howell test, which does not
#' assume equal variances or equal sample sizes. This is the appropriate
#' post-hoc test to use after a significant Welch's ANOVA.
#'
#' @param samples Numeric vector; the dependent variable.
#' @param groups Factor or vector; the grouping variable.
#' @param conf.level Numeric; confidence level for confidence intervals (default: 0.95).
#'
#' @return A data frame with columns:
#' \describe{
#'   \item{group1}{First group in comparison}
#'   \item{group2}{Second group in comparison}
#'   \item{mean_diff}{Difference in means (group1 - group2)}
#'   \item{se}{Standard error of the difference}
#'   \item{t}{t-statistic}
#'   \item{df}{Degrees of freedom (Welch-Satterthwaite)}
#'   \item{p_value}{Unadjusted p-value}
#'   \item{p_adj}{Holm-adjusted p-value for multiple comparisons}
#'   \item{ci_lower}{Lower bound of confidence interval}
#'   \item{ci_upper}{Upper bound of confidence interval}
#'   \item{significant}{Logical; TRUE if p_adj < (1 - conf.level)}
#' }
#'
#' @details
#' The Games-Howell test uses the Welch-Satterthwaite approximation for
#' degrees of freedom and does not pool variances. P-values are adjusted
#' using the Holm method to control family-wise error rate.
#'
#' @examples
#' # Convert dose to factor
#' ToothGrowth$dose <- as.factor(ToothGrowth$dose)
#' 
#' # Perform Games-Howell test
#' result <- games.howell(ToothGrowth$len, ToothGrowth$dose)
#' print(result)
#'
#' @export
games.howell <- function(samples, groups, conf.level = 0.95) {
  
  # Input validation
  if (!is.numeric(samples)) {
    stop("samples must be numeric")
  }
  
  if (length(samples) != length(groups)) {
    stop("samples and groups must have the same length")
  }
  
  # Clean data
  complete_cases <- complete.cases(samples, groups)
  samples <- samples[complete_cases]
  groups <- as.factor(groups[complete_cases])
  
  # Check minimum requirements
  group_levels <- levels(groups)
  k <- length(group_levels)
  
  if (k < 2) {
    stop("At least 2 groups required")
  }
  
  # Calculate group statistics
  n <- tapply(samples, groups, length)
  means <- tapply(samples, groups, mean)
  vars <- tapply(samples, groups, var)
  
  # Generate all pairwise comparisons
  comparisons <- combn(k, 2)
  n_comparisons <- ncol(comparisons)
  
  # Initialize results
  results <- data.frame(
    group1 = character(n_comparisons),
    group2 = character(n_comparisons),
    mean_diff = numeric(n_comparisons),
    se = numeric(n_comparisons),
    t = numeric(n_comparisons),
    df = numeric(n_comparisons),
    p_value = numeric(n_comparisons),
    ci_lower = numeric(n_comparisons),
    ci_upper = numeric(n_comparisons),
    stringsAsFactors = FALSE
  )
  
  # Perform pairwise comparisons
  for (i in 1:n_comparisons) {
    g1_idx <- comparisons[1, i]
    g2_idx <- comparisons[2, i]
    
    g1 <- group_levels[g1_idx]
    g2 <- group_levels[g2_idx]
    
    # Mean difference
    mean_diff <- means[g1] - means[g2]
    
    # Standard error (Welch formula - no pooling)
    se <- sqrt(vars[g1]/n[g1] + vars[g2]/n[g2])
    
    # Degrees of freedom (Welch-Satterthwaite approximation)
    df <- (vars[g1]/n[g1] + vars[g2]/n[g2])^2 / 
      ((vars[g1]/n[g1])^2 / (n[g1] - 1) + 
         (vars[g2]/n[g2])^2 / (n[g2] - 1))
    
    # t-statistic
    t_stat <- mean_diff / se
    
    # Two-tailed p-value
    p_value <- 2 * pt(abs(t_stat), df, lower.tail = FALSE)
    
    # Confidence interval
    alpha <- 1 - conf.level
    t_crit <- qt(1 - alpha/2, df)
    ci_lower <- mean_diff - t_crit * se
    ci_upper <- mean_diff + t_crit * se
    
    # Store results
    results[i, ] <- list(
      group1 = g1,
      group2 = g2,
      mean_diff = mean_diff,
      se = se,
      t = t_stat,
      df = df,
      p_value = p_value,
      ci_lower = ci_lower,
      ci_upper = ci_upper
    )
  }
  
  # Adjust p-values using Holm method
  results$p_adj <- p.adjust(results$p_value, method = "holm")
  
  # Add significance indicator
  results$significant <- results$p_adj < (1 - conf.level)
  
  # Set class for potential print method
  class(results) <- c("games.howell", "data.frame")
  
  return(results)
}

#' @exportS3Method
print.games.howell <- function(x, digits = 4, ...) {
  cat("\nGames-Howell Post-Hoc Test\n")
  cat("==========================\n\n")
  
  # Format output
  output <- x
  output$mean_diff <- round(output$mean_diff, digits)
  output$se <- round(output$se, digits)
  output$t <- round(output$t, digits)
  output$df <- round(output$df, 2)
  output$p_value <- format.pval(output$p_value, digits = digits)
  output$p_adj <- format.pval(output$p_adj, digits = digits)
  output$ci_lower <- round(output$ci_lower, digits)
  output$ci_upper <- round(output$ci_upper, digits)
  
  # Add significance stars
  output$sig <- ifelse(x$p_adj < 0.001, "***",
                       ifelse(x$p_adj < 0.01, "**",
                              ifelse(x$p_adj < 0.05, "*",
                                     ifelse(x$p_adj < 0.1, ".", ""))))
  
  print(as.data.frame(output), row.names = FALSE, ...)
  
  cat("\n---\n")
  cat("Signif. codes: 0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1\n")
  cat("P-values adjusted using Holm method\n\n")
  
  invisible(x)
}

#' Get compact letter display from Games-Howell results
#'
#' Converts Games-Howell test results into compact letter display
#' using multcompView. Groups sharing a letter are not significantly different.
#'
#' @param x A games.howell object from \code{games.howell()}
#' @param alpha Significance level (default: 0.05)
#'
#' @return A named vector with group names and their letter codes
#'
#' @examples
#' # Convert dose to factor
#' ToothGrowth$dose <- as.factor(ToothGrowth$dose)
#' result <- games.howell(ToothGrowth$len, ToothGrowth$dose)
#' letters <- gh_letters(result)
#' print(letters)
#'
#' @export
gh_letters <- function(x, alpha = 0.05) {
  
  if (!inherits(x, "games.howell")) {
    stop("x must be a games.howell object")
  }
  
  # Create a matrix of p-values for multcompView
  all_groups <- unique(c(x$group1, x$group2))
  k <- length(all_groups)
  
  # Initialize p-value matrix
  p_matrix <- matrix(1, nrow = k, ncol = k)
  rownames(p_matrix) <- all_groups
  colnames(p_matrix) <- all_groups
  
  # Fill in p-values
  for (i in 1:nrow(x)) {
    g1 <- x$group1[i]
    g2 <- x$group2[i]
    p_matrix[g1, g2] <- x$p_adj[i]
    p_matrix[g2, g1] <- x$p_adj[i]
  }
  
  # Convert to format for multcompView (TRUE = different)
  diff_matrix <- p_matrix < alpha
  
  # Use multcompView to get compact letter display
  letters <- multcompView::multcompLetters(diff_matrix)$Letters
  
  return(letters)
}

#' Visualize Numeric Relationships: Regression or Correlation Analysis
#'
#' This function provides unified visualization for numeric relationships between two continuous variables.
#' It can perform either regression analysis (with confidence and prediction bands) or
#' Spearman rank correlation analysis with appropriate visualizations and statistical output.
#' For regression, statistical assumptions are checked and warnings are issued if violated, but analysis proceeds.
#'
#' @param y Numeric vector. The response variable (dependent variable) for regression analysis, 
#'   or the y-axis variable for correlation analysis.
#' @param x Numeric vector. The predictor variable (independent variable) for regression analysis,
#'   or the x-axis variable for correlation analysis. Must have the same length as y.
#' @param correlation Logical. If FALSE (default), performs regression analysis with
#'   confidence and prediction bands. If TRUE, performs Spearman rank correlation analysis.
#' @param conf.level Numeric. Confidence level for statistical tests and intervals. 
#'   Must be between 0 and 1. Default is 0.95 (95 percent confidence level).
#' @param name_of_factor Character string. Label for the x-axis (independent variable). 
#'   If empty, defaults to the variable name.
#' @param name_of_sample Character string. Label for the y-axis (dependent variable). 
#'   If empty, defaults to the variable name.
#'
#' @return A list containing analysis results and assumption checks. Content depends on analysis type.
#'   For regression analysis: analysis_type, summary_regression, assumptions, warnings, r_squared, adj_r_squared.
#'   For correlation analysis: analysis_type, correlation_test, correlation_coefficient, assumptions,
#'   warnings, method_used.
#'
#' @details
#' Statistical Assumptions Checked:
#' Regression: Normality of residuals (Shapiro-Wilk test) and
#' homoscedasticity (Breusch-Pagan test). All regression analyses proceed even if
#' assumptions are violated, but appropriate warnings are issued.
#' Correlation: Spearman rank correlation requires no distributional assumptions.
#'
#' @examples
#' \dontrun{
#' # Generate sample data
#' set.seed(123)
#' x <- rnorm(50, mean = 10, sd = 2)
#' y <- 2 * x + rnorm(50, mean = 0, sd = 1)
#' 
#' # Regression analysis (default)
#' result1 <- vis_numeric(y, x, 
#'                       name_of_factor = "Predictor", 
#'                       name_of_sample = "Response")
#' 
#' # Spearman rank correlation
#' result2 <- vis_numeric(y, x, correlation = TRUE)
#' }
#'
#' @seealso \code{\link{cor.test}}, \code{\link{lm}}, \code{\link{shapiro.test}}
#' 
#' @author Sabine Schilling
#' @export
vis_numeric <- function(y,
                        x,
                        correlation = FALSE,
                        conf.level = 0.95,
                        name_of_factor = character(),
                        name_of_sample = character()) {
  
  # Input validation
  if (!is.numeric(y) || !is.numeric(x)) {
    stop("Both 'y' and 'x' must be numeric vectors")
  }
  
  if (length(y) != length(x)) {
    stop("'y' and 'x' must have the same length")
  }
  
  if (!is.logical(correlation) || length(correlation) != 1) {
    stop("'correlation' must be a single logical value (TRUE or FALSE)")
  }
  
  if (!is.numeric(conf.level) || length(conf.level) != 1 || 
      conf.level <= 0 || conf.level >= 1) {
    stop("'conf.level' must be a single number between 0 and 1")
  }
  
  # Define fallback colors if colorscheme() function not available
  if (!exists("colorscheme", mode = "function")) {
    colorscheme <- function(x) {
      if (x == 1) return(c("#1f77b4", "#ff7f0e"))  # blue, orange
      if (x == 2) return(c("#2ca02c", "#d62728"))  # green, red
      return(c("#1f77b4", "#ff7f0e"))  # default
    }
  }
  
  # Store and restore graphical parameters
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  
  alpha <- 1 - conf.level
  
  # Set default variable names if not provided
  if (length(name_of_factor) == 0 || name_of_factor == "") {
    name_of_factor <- "x"
  }
  if (length(name_of_sample) == 0 || name_of_sample == "") {
    name_of_sample <- "y"
  }
  
  # Remove all NAs from both vectors
  complete_cases <- !is.na(y) & !is.na(x)
  if (sum(complete_cases) < 3) {
    stop("At least 3 complete observations required after removing missing values")
  }
  
  x <- x[complete_cases]
  y <- y[complete_cases]
  n <- length(x)
  
  # Initialize warnings vector
  warnings_list <- character(0)
  
  if (!correlation) {
    # ========== REGRESSION ANALYSIS ==========
    
    # Fit regression model
    reg <- lm(y ~ x)
    reg_summary <- summary(reg)
    
    # Check regression assumptions
    assumptions <- list()
    
    # 1. Normality of residuals
    if (n >= 3 && n <= 5000) {
      assumptions$shapiro_residuals <- shapiro.test(rstandard(reg))
      if (assumptions$shapiro_residuals$p.value < alpha) {
        warnings_list <- c(warnings_list, 
                           paste("Normality of residuals violated (Shapiro-Wilk p =", 
                                 signif(assumptions$shapiro_residuals$p.value, 3), ")"))
      }
    } else {
      assumptions$shapiro_residuals <- list(p.value = NA, 
                                            method = "Sample size outside valid range for Shapiro-Wilk test")
    }
    
    # 2. Test for homoscedasticity using Breusch-Pagan test (correct for regression)
    bp_result <- tryCatch({
      bp.test(reg)
    }, error = function(e) {
      list(p.value = NA, method = paste("Breusch-Pagan test failed:", e$message))
    })
    
    assumptions$bp_homoscedasticity <- bp_result
    
    if (!is.na(bp_result$p.value) && bp_result$p.value < alpha) {
      warnings_list <- c(warnings_list, 
                         paste("Homoscedasticity violated (Breusch-Pagan p =", 
                               signif(bp_result$p.value, 3), ")"))
    }
    
    # Order data for plotting
    ord <- order(x)
    x_sorted <- x[ord]
    y_sorted <- y[ord]
    
    # Calculate prediction intervals
    conf_int <- predict(reg, interval = "confidence", level = conf.level)
    pred_int <- suppressWarnings(predict(reg, interval = "prediction", level = conf.level))
    
    y_conf_low <- conf_int[ord, 2]
    y_conf_up <- conf_int[ord, 3]
    y_pred_low <- pred_int[ord, 2]
    y_pred_up <- pred_int[ord, 3]
    fitted_sorted <- reg$fitted[ord]
    
    # Set up plot
    ma <- max(y_sorted, y_pred_up, na.rm = TRUE)
    mi <- min(y_sorted, y_pred_low, na.rm = TRUE)
    spread <- ma - mi
    
    par(mfrow = c(1, 1))
    plot(x_sorted, y_sorted, 
         ylim = c(mi - 0.1 * spread, ma + 0.4 * spread),
         xlab = name_of_factor,
         ylab = name_of_sample,
         main = "")
    
    polygon(c(x_sorted, rev(x_sorted)),
            c(y_conf_low, rev(y_conf_up)),
            col = adjustcolor(colorscheme(1)[1], alpha.f = 0.2),
            border = NA)
    
    # Add regression line and bands
    lines(x_sorted, fitted_sorted, col = colorscheme(2)[1], lwd = 2)
    lines(x_sorted, y_conf_low, col = colorscheme(1)[1], lwd = 2, lty = 2)
    lines(x_sorted, y_conf_up, col = colorscheme(1)[1], lwd = 2, lty = 2)
    lines(x_sorted, y_pred_low, col = colorscheme(1)[2], lwd = 2, lty = 3)
    lines(x_sorted, y_pred_up, col = colorscheme(1)[2], lwd = 2, lty = 3)
    
    legend("topleft", horiz = FALSE,
           c("regression line",
             paste(conf.level*100,"% confidence band"),
             paste(conf.level*100,"% prediction band")),
           lwd = 2,
           col = c(colorscheme(2)[1], colorscheme(1)[1], colorscheme(1)[2]),
           lty = c(1, 2, 3),
           bty = "n",
           cex = 0.7)
    
    # Create title
    conf_int_coeffs <- confint(reg, level = conf.level)
    
    title_text <- paste0("y = b1*x + b0, R-squared = ", sprintf("%.3f", reg_summary$r.squared),
                         ", conf. level = ", sprintf("%.2f", conf.level),
                         "\nslope b1 = ", sprintf("%.2f", reg$coefficients[2]),
                         ", CI [", sprintf("%.2f", conf_int_coeffs[2, 1]),
                         ", ", sprintf("%.2f", conf_int_coeffs[2, 2]), "]",
                         ", p = ", sprintf("%.2e", reg_summary$coefficients[2, 4]),
                         "\nintercept b0 = ", sprintf("%.2f", reg$coefficients[1]),
                         ", CI [", sprintf("%.2f", conf_int_coeffs[1, 1]),
                         ", ", sprintf("%.2f", conf_int_coeffs[1, 2]), "]",
                         ", p = ", sprintf("%.2e", reg_summary$coefficients[1, 4]))
    
    mtext(title_text, cex = 0.7)
    
    # Prepare return values
    result_list <- list(
      analysis_type = "regression",
      independent_variable_x = name_of_factor,
      dependent_variable_y = name_of_sample,
      summary_regression = reg_summary,
      assumptions = assumptions,
      warnings = warnings_list,
      r_squared = reg_summary$r.squared,
      adj_r_squared = reg_summary$adj.r.squared,
      sample_size = n
    )
    
  } else {
    # ========== CORRELATION ANALYSIS ==========
    
    # Always use Spearman: it subsumes Pearson (nearly identical results
    # for bivariate normal data) while remaining valid for non-normal
    # distributions and non-linear monotonic relationships.
    method_used <- "spearman"
    
    # Perform correlation test
    cor_test <- tryCatch({
      cor.test(x, y, method = method_used, conf.level = conf.level)
    }, warning = function(w) {
      # Handle warnings like "cannot compute exact p-value with ties"
      suppressWarnings(cor.test(x, y, method = method_used, conf.level = conf.level))
    })
    
    cor_coef <- cor_test$estimate
    p_value <- cor_test$p.value
    
    # Spearman correlation: no parametric assumptions required
    assumptions <- list(note = "Spearman correlation: no distributional assumptions required")
    
    # Create visualization
    ord <- order(x)
    x_sorted <- x[ord]
    y_sorted <- y[ord]
    
    # Monotonic trend line for visualization (regression on ranks,
    # mapped back to the original scale)
    trend_reg <- lm(rank(y) ~ rank(x))
    fitted_ranks <- trend_reg$fitted[ord]
    y_range <- max(y_sorted) - min(y_sorted)
    rank_range <- max(fitted_ranks) - min(fitted_ranks)
    if (rank_range > 0) {
      trend_fitted <- min(y_sorted) + (fitted_ranks - min(fitted_ranks)) * y_range / rank_range
    } else {
      trend_fitted <- rep(mean(y_sorted), length(fitted_ranks))
    }
    
    # Set up plot
    ma <- max(y_sorted, na.rm = TRUE)
    mi <- min(y_sorted, na.rm = TRUE)
    spread <- ma - mi
    
    par(mfrow = c(1, 1))

    plot(x_sorted, y_sorted,
         ylim = c(mi - 0.1 * spread, ma + 0.4 * spread),
         xlab = name_of_factor,
         ylab = name_of_sample,
         main = "")

    # Add trend line
    lines(x_sorted, trend_fitted, col = colorscheme(2)[2], lwd = 2, lty = 2)

    legend("topleft", horiz = FALSE,
           paste(tools::toTitleCase(method_used), "correlation trend line"),
           lwd = 2,
           col = colorscheme(2)[2],
           lty = 2,
           bty = "n",
           cex = 0.7)

    # Create title
    significance <- ifelse(p_value < alpha, "significant", "not significant")
    method_name <- tools::toTitleCase(method_used)

    mtext(bquote("Spearman" ~ rho ~ "=" ~ .(signif(cor_coef, 3)) ~
                 ", p =" ~ .(signif(p_value, 3))))
    
    # Prepare return values
    result_list <- list(
      analysis_type = paste(method_used, "correlation"),
      independent_variable_x = name_of_factor,
      dependent_variable_y = name_of_sample,
      correlation_test = cor_test,
      correlation_coefficient = cor_coef,
      p_value = p_value,
      method_used = method_used,
      assumptions = assumptions,
      warnings = warnings_list,
      significance = significance,
      sample_size = n
    )
  }
  
  # Issue warnings to console if any assumptions violated
  if (length(warnings_list) > 0) {
    warning("Statistical assumptions violated:\n",
            paste(warnings_list, collapse = "\n"),
            "\nAnalysis proceeded but interpret results cautiously.",
            call. = FALSE)
    if (!correlation && any(grepl("Normality|Homoscedasticity", warnings_list))) {
      message("RECOMMENDATION: Consider exploring alternatives outside visstat() such as data transformations, generalised linear models, or robust regression. For an assumption-free, non-causal alternative consider rerunning with correlation = TRUE.")
    }
  }
  
  return(result_list)
}

# MIT License-----
# Copyright (c) 2021 Sabine Schilling

# Plotting functions----

# Testing for normality and visualization ----

test_norm_vis <- function(x, y_axis_hist = c(0, 0.04)) {
  # store default graphical parameters------
  oldparnormvis <- par(no.readonly = TRUE)
  on.exit(par(oldparnormvis))
  par(mfrow = c(1, 2), oma = c(0, 0, 3, 0))
  # Remove NA from x
  x <- x[!is.na(x)]
  norm_dens <- function(z) {
    dnorm(z, mean(x), sd(x))
  }
  
  ymax <- max(norm_dens(x))
  # Plot histogramm of raw data
  otto <- hist(
    x,
    freq = FALSE,
    col = "grey",
    breaks = "Sturges",
    xlim = c(
      mean(x, na.rm = T) - 5 * sd(x, na.rm = T),
      mean(x, na.rm = T) +
        5 * sd(x, na.rm = T)
    ),
    ylim = c(0, 1.2 * ymax)
  )
  
  # normal distribution with mean and sd of given distribution
  curve(norm_dens,
        col = "red",
        add = TRUE,
        lwd = 2)
  
  
  # par(new = TRUE) #the next high-level plotting command does not clean the frame before drawing
  # as if it were on a new device.
  lines(density(x), col = "blue")
  
  legend(
    "topright",
    c("fitted", "estimated"),
    lty = 1,
    lwd = 2,
    col = c("red", "blue"),
    bty = "n"
  )
  box() # frame around current plot
  
  qqnorm(x)
  qqline(x, col = "red", lwd = 2)
  
  KS <- ad.test(x)
  p_KS <- signif(KS$p.value, 2)
  SH <- shapiro.test(x)
  p_SH <- signif(SH$p.value, 2)
  
  mtext(
    paste(
      "Shapiro-Wilk: p = ",
      p_SH,
      "\n Anderson-Darling: p = ",
      p_KS,
      "\n Nullhypothesis: Data is normally distributed"
    ),
    outer = TRUE
  )
  my_list <- list("Anderson-Darling" = KS, "Shapiro" = SH)
  
  return(my_list)
}



###### Two-Sample t-Test ###############################
two_sample_t_test <- function(samples,
                              fact,
                              alternative = c("two.sided", "less", "greater"),
                              paired = FALSE,
                              var.equal = FALSE,
                              conf.level = conf.level,
                              samplename = "",
                              factorname = "") {
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  alternative <- match.arg(alternative)
  
  
 
  
  if (!missing(conf.level) &&
      (length(conf.level) != 1 || !is.finite(conf.level) ||
       conf.level < 0 || conf.level > 1)) {
    stop("'conf.level' must be a single number between 0 and 1")  
  }
  
  
  
  if (missing(conf.level)) {
    conf.level <- 0.95
  }
  alpha <- 1 - conf.level
  levels <- unique(sort(fact))
  twosamples <- create_two_samples_vector(samples, fact)
  x <- twosamples$sample1and2
  x1 <- twosamples$sample1
  x2 <- twosamples$sample2
  # Check normality of both samples-----
  p1 <- test_norm(twosamples$sample1)
  p2 <- test_norm(twosamples$sample2)
  
  # margins of y -axis
  lower <- 0.2
  upper <- 0.2
  margins <- calc_min_max_of_y_axis(x, lower, upper)
  mi <- margins[[1]]
  ma <- margins[[2]]
  
  x <- cbind(x, factor(c(rep(1, length(
    x1
  )), rep(2, length(
    x2
  )))))
  
  show_jitter  <- max(length(x1), length(x2)) <= jitter_max_n

  b <- boxplot(
    samples ~ fact,
    xlab = factorname,
    ylab = samplename,
    ylim = c(mi, ma),
    col = colorscheme(1),
    outline = !show_jitter  # outliers via boxplot only when jitter is suppressed
  )

  if (show_jitter) {
    stripchart(
      samples ~ fact,
      vertical = TRUE,
      xlim = c(0, 3),
      ylim = c(mi, ma),
      col = adjustcolor(colorscheme(1), red.f = 0.55, green.f = 0.55, blue.f = 0.55),
      pch = 1,
      cex = 0.7,
      method = "jitter",
      add = TRUE
    )
  }

  # Group means -- parametric test compares means, so mark them explicitly
  points(c(1, 2), c(mean(x1), mean(x2)),
         pch = 16, col = "red", cex = 1.3)

  # Sample sizes above box plot
  text(1:length(b$n), c(ma, ma), paste("N =", b$n))

  t <- t.test(
    x1,
    x2,
    alternative = alternative,
    conf.level = conf.level,
    paired = FALSE,
    var.equal = var.equal,
    na.action = na.omit
  )

  p_value       <- signif(t$p.value, 2)
  test_statistic <- sprintf("%.2f", t$statistic)
  stat_name     <- names(t$statistic)

  # Title: line 1 = test name, line 2 = test statistic + p
  # (alpha and decision suffix omitted -- 2-group test has no post-hoc letters
  #  whose interpretation depends on alpha)
  mtext(paste0(t$method, "\n",
               stat_name, " = ", test_statistic,
               ", p = ", p_value))

  # Legend: just the mean marker (no significance letters in the 2-group case)
  legend("bottomleft",
         legend = "group mean",
         pch = 16,
         col = "red",
         text.col = "red",
         bty = "n",
         cex = 0.9,
         inset = 0.02)
  
  my_list <-
    list(
      "dependent variable (response)" = samplename,
      "independent variables (features)" = unique(fact),
      "t-test-statistics" = t,
      "Shapiro-Wilk-test_sample1" = p1,
      "Shapiro-Wilk-test_sample2" = p2
    )
  
  return(my_list)
}


# Two-Sample Wilcoxon-Test  ###############################
# One function with flags for greater, less, two sided and notch
two_sample_wilcoxon_test <- function(samples,
                                     fact,
                                     alternative = c("two.sided", "less", "greater"),
                                     conf.level = conf.level,
                                     notchf = FALSE,
                                     samplename = "",
                                     factorname = "",
                                     cex = 1) {
  oldparwilcox <- par(no.readonly = TRUE) # make a copy of current values
  on.exit(par(oldparwilcox))
  
  alternative <- match.arg(alternative)
  # Error handling ----
  
  
  if (missing(conf.level)) {
    conf.level <- 0.95
  }
  
  if (missing(notchf)) {
    notchf <- FALSE
  }
  
  if (!((length(conf.level) == 1L) && is.finite(conf.level) &&
        (conf.level > 0) && (conf.level < 1))) {
    return(warning("'conf.level' must be a single number between 0 and 1"))
  }
  
  if (!is.numeric(samples)) {
    return(warning("'samples' must be numeric"))
  }
  if (!is.null(fact)) {
    if (!is.factor(fact)) {
      return(warning("'fact' must be factorial"))
    }
  }
  
  # Store default graphical parameter
  # Define color palette
  colortuple2 <- colorscheme(2)
  
  
  
  
  
  # Create to numeric vectors
  twosamples <- create_two_samples_vector(samples, fact)
  x <- twosamples$sample1and2
  x1 <- twosamples$sample1
  x2 <- twosamples$sample2
  
  upper <- 0.2
  lower <- 0.05
  res <- calc_min_max_of_y_axis(x, upper, lower)
  mi <- res[[1]]
  ma <- res[[2]]
  
  x <- cbind(x, factor(c(rep(1, length(
    x1
  )), rep(2, length(
    x2
  )))))
  
  par(oma = c(0, 0, 2, 0)) # outer margin above: 2 lines..

  show_jitter  <- max(length(x1), length(x2)) <= jitter_max_n

  b <- boxplot(
    samples ~ fact,
    notch = notchf,
    col = colorscheme(1),
    ylim = c(mi, ma),
    ylab = samplename,
    xlab = factorname,
    outline = !show_jitter  # outliers via boxplot only when jitter is suppressed
  )

  if (show_jitter) {
    stripchart(
      samples ~ fact,
      vertical = TRUE,
      method = "jitter",
      col = adjustcolor(colorscheme(1), red.f = 0.55, green.f = 0.55, blue.f = 0.55),
      pch = 1,
      cex = 0.7,
      add = TRUE
    )
  }

  # Sample sizes above box plot
  text(1:length(b$n), c(ma, ma), paste("N =", b$n))

  t <- wilcox.test(samples ~ fact, alternative = alternative, na.action = na.omit)

  p_value       <- signif(t$p.value, 2)
  test_statistic <- sprintf("%.2f", t$statistic)
  stat_name     <- names(t$statistic)

  # Title: line 1 = test name, line 2 = test statistic + p
  # (alpha and decision suffix omitted -- 2-group test has no post-hoc letters
  #  whose interpretation depends on alpha)
  mtext(paste0(t$method, "\n",
               stat_name, " = ", test_statistic,
               ", p = ", p_value))
  
  
  
  my_list <-
    list(
      "dependent variable (response)" = samplename,
      "independent variables (features)" = unique(fact),
      "statsWilcoxon" = t,
      "statsBoxplot" = b
    )
  
  return(my_list)
}

# Two-Sample F-Test ###############################
# subtract means; two lines according to variances.
two_sample_FTest <- function(samples,
                             fact,
                             conf.int = conf.int,
                             alternative = "two.sided") {
  # if (missing(conf.int)) conf.int = 0.95
  #  if (missing(alternative)) alternative = "two.sided"
  # Store default graphical parameter
  oldparftest <- par(no.readonly = TRUE)
  on.exit(par(oldparftest))
  if (missing(conf.int)) {
    conf.int <- 0.95
  }
  
  alpha <- 1 - conf.int
  levels <- unique(sort(fact))
  
  x1 <- samples[fact == levels[1]]
  x2 <- samples[fact == levels[2]]
  
  x1 <- x1 - mean(x1, na.rm = T)
  
  x2 <- x2 - mean(x2)
  
  
  x <- c(x1, x2)
  spread <- max(x) - min(x)
  spread <- max(spread, var(x1), var(x2))
  
  mi <- min(x) - 0.3 * spread
  ma <- max(x) + 0.3 * spread
  
  x <- cbind(x, factor(c(rep(1, length(
    x1
  )), rep(2, length(
    x2
  )))))
  
  par(oma = c(0, 0, 3, 0))
  stripchart(
    x[, 1] ~ x[, 2],
    vertical = TRUE,
    xlim = c(0.5, 3),
    ylim = c(mi, ma),
    col = c("grey70", "grey80"),
    ylab = "centred samples",
    xlab = "",
    axes = FALSE
  )
  
  axis(side = 2)
  axis(side = 1,
       at = c(1, 2),
       labels = levels)
  box()
  
  lines(
    x = c(1.1, 1.1),
    y = c(-0.5 * var(x1), 0.5 * var(x1)),
    col = "blue",
    lwd = 5
  )
  lines(
    x = c(1.9, 1.9),
    y = c(-0.5 * var(x2), 0.5 * var(x2)),
    col = "blue",
    lwd = 5
  )
  
  legend(
    "topright",
    inset = 0.05,
    c("variances"),
    col = c("blue"),
    lwd = 2
  )
  
  t <- var.test(x1, x2, alternative = alternative)
  p_value <- t$p.value
  p_value <- signif(p_value, 3)
  
  test_statistic <- t$statistic
  test_statistic <- formatC(signif(test_statistic, digits = 2))
  
  
  mtext(
    paste(
      "Two Sample F-Test (",
      alternative,
      "): P = ",
      p_value,
      "\n Confidence Level = ",
      1 - alpha
    ),
    outer = TRUE
  )
}



# Chisquared Test of Fisher-test  ----


# vis_chi_squared_test: implemented in vis_samples_fact -----
vis_chi_squared_test <- function(samples,
                                 fact,
                                 samplename,
                                 factorname,
                                 cex = 1) {
  oldparchi <- par(no.readonly = TRUE)
  on.exit(par(oldparchi))
  
  colortuple <- colorscheme(1)
  ColorPalette <- colorscheme(3)
  if (missing(samplename)) {
    samplename <- character()
  }
  if (missing(factorname)) {
    factorname <- character()
  }
  
  counts <- makeTable(samples, fact, samplename, factorname)
  # check for minimal size of 2x2
  check_assumptions_chi <- check_assumptions_count_data(samples, fact)
  
  if (check_assumptions_chi == FALSE) {
    fisher_chi <- counts
    return(fisher_chi)
    
  } else {
    row_sum <- rowSums(counts)
    col_sum <- colSums(counts)
    count_labels <- dimnames(counts)[2]
    count_labels <- as.character(unlist(count_labels))
    
    category_names <- dimnames(counts)[1]
    category_names <- as.character(unlist(category_names))
    
    norm_counts <- (counts / row_sum) * 100 # row percentages

    fisher_chi <- fisher_chi(counts) # checks Cochran requirements; uses Fisher if violated

    # Fisher's exact test: show absolute counts; Chi² / Yates: show row percentages
    is_fisher <- isTRUE(grepl("Fisher", fisher_chi$method, ignore.case = TRUE))
    if (is_fisher) {
      bar_data  <- counts
      ylab_text <- "Count"
      max_val_y <- max(counts, na.rm = TRUE)
    } else {
      bar_data  <- norm_counts
      ylab_text <- "%"
      max_val_y <- max(norm_counts, na.rm = TRUE)
    }

    if (nrow(counts) < (length(ColorPalette) + 2)) {
      col_vec_browser <- c(colortuple, head(ColorPalette, n = nrow(counts) - 2))
    } else {
      col_vec_browser <- c(colortuple, rainbow(nrow(counts) - 2, s = 0.4, alpha = 1))
    }

    par(mfrow = c(1, 1), oma = c(0, 0, 3, 0))

    maxlabels <- length(levels(samples))
    if (maxlabels > 7 |
        grepl("basis", samplename) | grepl("source", samplename) |
        grepl("basis", factorname) | grepl("source", factorname) |
        grepl("genotyped", samplename) |
        grepl("genotyped", factorname)) {
      labelsize <- 0.3 * cex
    } else if (maxlabels > 5) {
      labelsize <- 0.7 * cex
    } else {
      labelsize <- cex
    }

    if (fisher_chi$method != "Fisher's Exact Test for Count Data") {
      chi2_fisher_text <- paste0(
        fisher_chi$method,
        "\nChi-squared = ", round(fisher_chi$statistic, 2),
        ", p-value = ", signif(fisher_chi$p.value, 3)
      )
    } else {
      chi2_fisher_text <- paste0(
        fisher_chi$method,
        "\np-value = ", signif(fisher_chi$p.value, 3)
      )
    }

    titletext <- chi2_fisher_text

    if (nrow(counts) > 3) {
      ma <- 1.3 * max_val_y
      legendsize <- 0.7 * cex
    } else {
      # Fisher shows N= labels above each bar; 1.2x gives enough headroom so
      # the label on the tallest bar stays inside the yaxs="i" coordinate range.
      ma <- if (is_fisher) 1.2 * max_val_y else 1.1 * max_val_y
      legendsize <- cex
    }

    bp <- barplot(
      bar_data,
      names.arg = count_labels,
      xlim = c(-0.5, ncol(counts) + 1),
      ylim = c(0, ma),
      width = 1 / (nrow(counts) + 1),
      space = c(0, 1),
      col = col_vec_browser,
      ylab = ylab_text,
      xlab = samplename,
      beside = TRUE,
      cex.axis = 1,
      cex.names = labelsize
    )

    # N labels: Fisher branch only — Pearson chi² is followed by a mosaic that
    # already displays counts. Place "N = <count>" above every individual bar.
    if (is_fisher) {
      bar_x <- as.vector(bp)
      bar_n <- as.vector(counts)
      bar_h <- as.vector(bar_data)
      text(bar_x, bar_h + strheight("N", units = "user"),
           paste("N =", bar_n),
           cex = labelsize * 0.8)
    }

    box()
    mtext(titletext)
    category_names <- as.character(category_names)
    legend(
      "topright",
      inset = 0.05,
      title = factorname,
      category_names,
      col = col_vec_browser,
      bty = "n",
      lwd = 2,
      cex = legendsize
    )
    
    return(fisher_chi)
  }
}




###### Visualize Kruskal_Wallis ###############################
## performs Kruskal Wallis and post-hoc Wilcoxon:

vis_Kruskal_Wallis <- function(samples,
                               fact,
                               conf.level = conf.level,
                               samplename = "",
                               factorname = "",
                               cex = 1,
                               notch = F) {
  oldparkruskal <- par(no.readonly = TRUE)
  on.exit(par(oldparkruskal))
  
  if (missing(conf.level)) {
    conf.level <- 0.95
  }
  alpha <- 1 - conf.level
  # remove rows with NAs in samples
  samples3 <- na.omit(samples)
  fact <- subset(fact, !is.na(samples))
  samples <- samples3
  n_classes <- length(unique(fact))
  # Box / stripchart colors -- start with the t-test pair (green, blue)
  # and extend with RColorBrewer Set 3 for additional groups.
  box_cols <- rep_len(c(colorscheme(1), colorscheme(3)), n_classes)

  kk <- kruskal.test(samples ~ fact)

  # Y-axis limits (with extra lower margin for legend space)
  lower_margin <- 0.45
  upper_margin <- 0.2
  margins <- calc_min_max_of_y_axis(samples, lower_margin, upper_margin)
  mi <- margins[[1]]
  ma <- margins[[2]]

  # For legend positioning (spread from actual plot range)
  spread <- ma - mi


  show_jitter  <- max(tapply(samples, fact, length)) <= jitter_max_n

  b <- boxplot(
    samples ~ fact,
    notch = isTRUE(notch),
    xlim = c(0, n_classes + 1),
    ylim = c(mi, ma),
    col = box_cols,
    las = 1,
    xlab = factorname,
    ylab = samplename,
    cex.lab = cex,
    cex.axis = 0.8 * cex,
    boxwex = 0.5,
    outline = !show_jitter  # outliers via boxplot only when jitter is suppressed
  )

  if (show_jitter) {
    stripchart(
      samples ~ fact,
      vertical = TRUE,
      method = "jitter",
      col = adjustcolor(box_cols, red.f = 0.55, green.f = 0.55, blue.f = 0.55),
      pch = 1,
      cex = 0.7,
      add = TRUE
    )
  }

  if (n_classes > 6) {
    n_labels <- c(paste("N =", b$n[1]), as.character(b$n[-1]))
  } else {
    n_labels <- paste("N =", b$n)
  }
  text(seq_len(n_classes), ma, n_labels)

  post_hoc_kruskal <- sig_diffs_nongauss(samples, fact, conf.level = conf.level)
  s <- multcompLetters(post_hoc_kruskal[[1]][, 4], threshold = alpha)

  ord <- integer(n_classes)
  v <- attributes(s$Letters)$names
  f_levels <- sort(unique(fact))
  for (i in 1:n_classes) {
    ord[i] <- which(v == f_levels[i])
  }

  text(seq_len(n_classes),
       mi,
       s$Letters[ord],
       col = colors()[81],
       cex = cex,
       lwd = 2)

  # Title: line 1 = test name only, line 2 = H + p
  # (post-hoc method + alpha live in the legend below to avoid duplication)
  kk_value <- sprintf("%.2f", as.numeric(kk$statistic))
  mtext(paste0("Kruskal-Wallis test\nH = ", kk_value,
               ", p = ", signif(kk$p.value, 2)))

  # Legend: significance letters (no mean marker -- Kruskal tests ranks).
  # All post-hoc / alpha info lives here, so the title can stay compact.
  legend("bottomleft",
         legend = c("a, b, ...: significance letters",
                    paste0("(pairwise Wilcoxon, Holm adj., alpha = ",
                           signif(alpha, 2), ")")),
         text.col = colors()[81],
         bty = "n",
         cex = 0.9,
         inset = 0.08)

  my_list <-
    list("Kruskal Wallis rank sum test" = kk,
         "post-hoc by pairwise Wilcoxon rank sum test " = post_hoc_kruskal)
  return(my_list)
}


##### Visualize Regression und trumpet curves ###############################
vis_regr_trumpets <- function(x, y, conf.level) {
  if (missing(conf.level)) {
    conf.level <- 0.95
  }
  oldparreg <- par(no.readonly = TRUE)
  on.exit(par(oldparreg))
  reg <- lm(y ~ x)
  summary(reg)
  
  ## error bands:
  y_conf_low <- conf_band(x, reg, conf.level, -1)
  y_conf_up <- conf_band(x, reg, conf.level, 1)
  
  ma <- max(y, reg$fitted)
  mi <- min(y, reg$fitted)
  spread <- ma - mi
  
  lower <- 0.1
  upper <- 0.4
  margins <- calc_min_max_of_y_axis(y, lower, upper)
  mi <- margins[[1]]
  ma <- margins[[2]]
  
  
  par(oma = c(0, 0, 5, 0))
  plot(x, y, ylim = c(mi, ma))
  points(x,
         reg$fitted,
         type = "l",
         col = 2,
         lwd = 2)
  
  points(
    x,
    y_conf_low,
    type = "l",
    lwd = 2,
    lty = 2,
    col = colors()[84]
  )
  points(
    x,
    y_conf_up,
    type = "l",
    lwd = 2,
    lty = 2,
    col = colors()[84]
  )
  legend(
    "bottomright",
    c(
      "regr. line",
      paste("confidence band for alpha=", 1 - conf.level)
    ),
    lwd = 2,
    col = c(2, colors()[84], colors()[85]),
    lty = c(1, 2, 3),
    bty = "n"
  )
  s <- summary(reg)
  mtext(
    paste(
      "Regression: ax + b. confidence band for alpha = ",
      1 - conf.level,
      "\n \n"
    ),
    outer = TRUE,
    cex = 1.5
  )
  mtext(
    paste(
      "\n \n a = ",
      signif(reg$coefficients[2], 2),
      ", p = ",
      signif(s$coefficients[2, 4], 2),
      "\n b = ",
      signif(reg$coefficients[1], 2),
      ", p = ",
      signif(s$coefficients[1, 4], 2),
      "\n R^2 = ",
      signif(summary(reg)$r.squared, 4)
    ),
    outer = TRUE
  )
  
  par(mfrow = c(1, 2), oma = c(0, 0, 3, 0))
  plot(
    reg$fitted,
    residuals(reg),
    main = "Residuals vs. Fitted",
    xlab = "Fitted Values",
    ylab = "Residuals"
  )
  abline(h = 0, col = 1, lwd = 2)
  
  qqnorm(residuals(reg), ylab = "Sample Quantiles of  Residuals")
  qqline(residuals(reg), col = "red", lwd = 2)
  
  KS <- ad.test(residuals(reg))
  p_KS <- signif(KS$p.value, 2)
  SH <- shapiro.test(residuals(reg))
  p_SH <- signif(SH$p.value, 2)
  
  mtext(
    paste(
      "Residual Analysis\n Shapiro-Wilk: p = ",
      p_SH,
      "\n  Anderson-Darling: p = ",
      p_KS
    ),
    outer = TRUE
  )
}

###### Visualize Residuals ###############################
vis_resid <- function(resid, fitted) {
  oldparresid <- par(no.readonly = TRUE)
  on.exit(par(oldparresid))
  
  par(mfrow = c(1, 2), oma = c(0, 0, 3, 0))
  plot(fitted, resid, main = "Residuals vs. Fitted")
  abline(h = 0, col = 1, lwd = 2)
  
  qqnorm(resid)
  qqline(resid, col = "red", lwd = 2)
  
  KS <- ad.test(resid)
  p_KS <- signif(KS$p.value, 2)
  SH <- shapiro.test(resid)
  p_SH <- signif(SH$p.value, 2)
  
  mtext(
    paste(
      "Residual Analysis\n Shapiro-Wilk: p = ",
      p_SH,
      "\n Anderson-Darling: p = ",
      p_KS
    ),
    outer = TRUE
  )
}


###### Visualize Regression ###############################

# only normality assumptions of standardised residuals
vis_normality_assumptions <- function(y, x, conf.level = 0.95) {
  oldparreg <- par(no.readonly = TRUE)
  on.exit(par(oldparreg))
  alpha <- 1 - conf.level
  # P = alpha
  
  # remove all NAs from both vectors
  xna <- x[!is.na(y) & !is.na(x)]
  yna <- y[!is.na(y) & !is.na(x)]
  
  x <- xna
  y <- yna
  
  ord <- order(x)
  x <- sort(x)
  y <- y[ord]
  
  reg <- lm(y ~ x)
  resreg <- summary(reg)
  
  
  par(mfrow = c(1, 2), oma = c(0, 0, 4, 0))
  plot(
    reg$fitted,
    rstandard(reg),
    main = "std. Residuals vs. Fitted",
    xlab = "Fitted Values",
    ylab = "standardised Residuals"
  )
  abline(h = 0, col = 1, lwd = 2)
  
  qqnorm(rstandard(reg), ylab = "Sample Quantiles of Std. Residuals")
  qqline(rstandard(reg), col = "red", lwd = 2)
  
  AD <- ad.test(rstandard(lm(y ~ x)))
  p_AD <- signif(AD$p.value, 2)
  SH <- shapiro.test(rstandard(lm(y ~ x)))
  p_SH <- signif(SH$p.value, 2)
  if (p_SH < alpha) {
    mtext(
      paste(
        "Residual Analysis\n Shapiro-Wilk: p = ",
        p_SH,
        ", Anderson-Darling: p = ",
        p_AD,
        "\n Requirement of normally distributed residuals not met "
      ),
      outer = TRUE
    )
  } else {
    mtext(
      paste(
        "Residual Analysis\n Shapiro-Wilk: p = ",
        p_SH,
        ", Anderson-Darling: p = ",
        p_AD
      ),
      outer = TRUE
    )
  }
  
  my_list <- list(
    "summary_regression" = resreg,
    "shapiro_test_residuals" = SH,
    "ad_test_residuals" = AD
  )
  
  
  return(my_list)
}





# Mosaic plots-----
vis_mosaic <- function(samples,
                       fact,
                       name_of_sample = character(),
                       name_of_factor = character(),
                       minperc = 0.05,
                       numbers = TRUE,
                       shade = TRUE) {
  oldparmosaic <- par(no.readonly = TRUE)
  oldparmosaic$new <- FALSE
  on.exit(par(oldparmosaic))
  
  if (missing(minperc)) {
    # minperc is the minimum percentage a column has to contribute to be displayed
    minperc <- 0.05
  }
  if (missing(numbers)) {
    # numbers are shown in rectangle of category
    numbers <- TRUE
  }
  
  counts <- makeTable(samples, fact, name_of_sample, name_of_factor)
  check_assumptions <- check_assumptions_count_data(samples, fact)
  if (check_assumptions == FALSE) {
    my_list <- counts
    return(my_list)
  } else {
    ## Mosaic plot
    ## The height of the box is the same for all boxes in the same row and
    # is equal to the total count in that row.
    #
    # The width of the box is the proportion of individuals in the row which fall into that cell.
    n_samples <- length(unique(samples))
    n_fact <- length(unique(fact))
    maxfactors <- max(n_samples, n_fact)
    threshold <- 6
    
    # Calculate adaptive font sizes based on number of categories
    if (maxfactors <= 4) {
      label_fontsize <- 8
      cell_fontsize <- 7
    } else if (maxfactors <= 6) {
      label_fontsize <- 7
      cell_fontsize <- 6
    } else if (maxfactors <= 8) {
      label_fontsize <- 6
      cell_fontsize <- 5
    } else {
      label_fontsize <- 4
      cell_fontsize <- 3
    }
    
    # Common labeling arguments for readable labels
    label_args <- list(
      gp_labels = gpar(fontsize = label_fontsize),
      gp_varnames = gpar(fontsize = label_fontsize + 1, fontface = "bold")
    )
    
    if (n_samples < threshold & n_fact < threshold) {
      res <- mosaic(
        counts,
        shade = shade,
        legend = shade,
        labeling_args = label_args,
        pop = FALSE
      )
      
      tab <- as.table(ifelse(counts < 0.005 * sum(counts), NA, counts))
      if (numbers == TRUE) {
        labeling_cells(text = tab, margin = 0, 
                       gp_text = gpar(fontsize = cell_fontsize))(counts)
      }
    } else {
      ## Eliminate rows and columns distributing less than minperc total number of counts
      rowSum <- rowSums(counts)
      colSum <- colSums(counts)
      total <- sum(counts)
      
      countscolumn_row_reduced <- as.table(counts[which(rowSum > minperc * total), which(colSum > minperc * total)])
      
      # check dimensions after reduction: must be a contingency table
      test <- dim(as.table(countscolumn_row_reduced))
      if (is.na(test[2])) {
        countsreduced <- counts
      } else {
        countsreduced <- countscolumn_row_reduced
      }
      
      res <- mosaic(
        countsreduced,
        shade = shade,
        legend = shade,
        labeling_args = label_args,
        pop = FALSE
      )
      
      if (numbers == TRUE & maxfactors <= 10) {
        labeling_cells(text = countsreduced, margin = 0,
                       gp_text = gpar(fontsize = cell_fontsize))(countsreduced)
      }
    }
    my_list <- list("mosaic_stats" = res)
    
    return(my_list)
  }
}




# Helper functions--------------------------------------

# Check for type of samples and fact
type_sample_fact <- function(samples, fact) {
  typesample <- class(samples)
  typefactor <- class(fact)
  listsf <- list("typesample" = typesample, "typefactor" = typefactor)
  return(listsf)
}

# helper function odds ratio
# calculation of odds ratio
odds_ratio <- function(a, b, c, d, alpha, zerocorrect) {
  attr(odds_ratio, "help") <-
    "odds_ratio calculates odds ratio OR=(a/b)/(c/d) and corresponding upper and lower confidence intervalls\n INPUT: a = group 1 positive, c = group 2 positive, b=group 1 non positive, d = group 2 non positive, 1-alpha: confidence level, default alpha=0.05"
  
  # "odds_ratio calculates odds ratio OR=(a/b)/(c/d) and corresponding upper and lower confidence intervalls\n
  # INPUT: a=number of positives in  group 1, c=group 2 positive, b=group 1 non positive, d =group 2 non positive,default alpha=0.05, OR=(a/b)/(c/d)"\n
  # a,b,c,d can be vectors, elementwise calculation
  #
  
  
  if (missing(alpha)) {
    alpha <- 0.05
  }
  if (missing(zerocorrect)) {
    zerocorrect <- TRUE
  }
  # odds ratio:=OR=a/b/(c/d)
  
  # eliminate columns with zeros
  # a=c=0 or b=d 0: no positive or no negative cases in both groups
  # Higgins and Green 2011:
  
  if (zerocorrect == TRUE) {
    # eliminate columns with zeros, if
    # a=c=0 or b=d=0: no positive or no control cases in BOTH groups
    # Higgins and Green 2011:
    doublezero <- which(a == 0 &
                          c == 0 | b == 0 & d == 0, arr.ind = T)
    a[doublezero] <- NaN
    b[doublezero] <- NaN
    c[doublezero] <- NaN
    d[doublezero] <- NaN
    # Where zeros cause problems with computation of effects or standard errors, 0.5 is added to all cells (a, b, c, d)
    singlezero <- which(a == 0 |
                          b == 0 | c == 0 | d == 0, arr.ind = T)
    a[singlezero] <- a[singlezero] + 0.5
    b[singlezero] <- b[singlezero] + 0.5
    c[singlezero] <- c[singlezero] + 0.5
    d[singlezero] <- d[singlezero] + 0.5
  }
  
  oddA <- a / b
  oddB <- c / d
  
  OR <- oddA / oddB
  
  # confidence intervall
  # SE of ln(OR)
  SE <- sqrt(1 / a + 1 / b + 1 / c + 1 / d)
  alpha <- 0.05
  zalph <- qnorm(1 - alpha / 2)
  
  log_low <- log(OR) - zalph * SE
  log_up <- log(OR) + zalph * SE
  
  lowconf <- exp(log_low) # lower confidence
  upconf <- exp(log_up)
  
  output <- rbind(OR, lowconf, upconf, SE)
  my_list <- ("odds_ratio_statistics" <- output)
  return(my_list)
}

# create sorted table
makeTable <- function(samples, fact, samplename, factorname) {
  counts <- data.frame(fact, samples)
  colnames(counts) <- c(factorname, samplename)
  counts2 <- table(counts)
  # sort by column sums
  counts3 <- counts2[, order(colSums(counts2), decreasing = T)]
  # sort by row sums
  counts4 <- counts3[order(rowSums(counts3), decreasing = T), ]
  # remove columnns with all entries zero
  counts4 <- counts4[, colSums(counts4 != 0) > 0]
  
  return(counts4)
}

fisher_chi <- function(counts) {
  # if Cochran requirements for chi2 not given: fisher test is performed
  # if more than 20% of cells have EXPECTED count smaller 5 or one cell has expected count smaller than 1
  #

  suppressWarnings(chisq <- chisq.test(counts))
  expected_counts <- chisq$expected

  cochran_violated <- any(expected_counts < 1) | # at least one cell with expectation value smaller 1
    (sum(expected_counts < 5) / length(expected_counts) > 0.2 # more than 20% of cells have expected count smaller 5
     &
     dim(counts)[2] < 7) # Fisher breaks down for too large tables

  if (cochran_violated) {
    if (all(dim(counts) == 2)) {
      # 2x2 table: exact Fisher test — includes OR and CI in output
      testFisherChi <- fisher.test(counts)
    } else {
      # larger tables: simulate p-value (OR not defined)
      testFisherChi <- fisher.test(
        counts,
        workspace = 1e9,
        simulate.p.value = TRUE,
        hybrid = FALSE,
        B = 1e5
      )
    }
  } else {
    testFisherChi <- chisq
  }

  return(testFisherChi)
}

side_of_nh <- function(alternative) {
  if (alternative == "less") {
    compare <- c(">=")
  } else if (alternative == "greater") {
    compare <- c("<=")
  } else {
    compare <- c("equals")
  }
  return(compare)
}

create_two_samples_vector <- function(samples, fact) {
  # Creates column vector built out of two samples
  # samples all in one column and sorted
  levels <- unique(sort(fact))
  # two levels
  if (length(levels) > 2) {
    return(warning(
      "warning: create_two_samples_vector: only two level input allowed"
    ))
  } else {
    samples1 <- samples[fact == levels[1]]
    samples1 <- samples1[!is.na(samples1)]
    if (length(samples1) == 0) {
      return(warning("each group needs at least one entry"))
    } else {
      samples2 <- samples[fact == levels[2]]
      samples2 <- samples2[!is.na(samples2)]
      if (length(samples2) == 0) {
        return(warning("each group needs at least one entry"))
      } else {
        x <- c(samples1, samples2)
        my_list <- list(
          "sample1" = samples1,
          "sample2" = samples2,
          "sample1and2" = x
        )
        return(my_list)
      }
    }
  }
}

calc_min_max_of_y_axis <- function(samples,
                                   lowerExtramargin,
                                   upperExtramargin) {
  maximum <- max(samples, na.rm = T)
  minimum <- min(samples, na.rm = T)
  spread <- maximum - minimum
  min_y_axis <- minimum - lowerExtramargin * spread
  max_y_axis <- maximum + upperExtramargin * spread
  return(list(min_y_axis, max_y_axis))
}


check_assumptions_shapiro <- function(x) {
  x <- sort(x[complete.cases(x)])
  n <- length(x)
  rng <- x[n] - x[1L] # 1L is integer
  checkSize <- !(is.na(n) || n < 3L || n > 5000L) # FALSE or TRUE
  if (checkSize == FALSE) {
    warning("sample size must be between 3 and 5000")
    return(FALSE)
  }
  if (rng == 0) {
    warning("all 'x' values are identical")
    return(FALSE)
  }
  
  
  
  return(TRUE)
}

check_assumption_shapiro_size_range_two_samples <- function(x1, x2) {
  boolean1 <- check_assumptions_shapiro(x1)
  boolean2 <- check_assumptions_shapiro(x2)
  if (boolean1 == TRUE & boolean2 == TRUE) {
    return(TRUE)
  } else {
    return(FALSE)
  }
}


check_assumptions_count_data <- function(samples, fact) {
  counts <- table(samples, fact)
  sr <- rowSums(counts)
  sc <- colSums(counts)
  counts <- counts[sr > 0, sc > 0, drop = FALSE]
  nr <- as.integer(nrow(counts))
  nc <- as.integer(ncol(counts))
  if (is.null(dim(counts))) {
    warning("no entries in count table ")
    return(FALSE)
  } else if (is.na(nr) || is.na(nc) || is.na(nr * nc)) {
    warning("invalid nrow  or ncol in count data ", domain = NA)
    return(FALSE)
  } else if (nr <= 1L) {
    warning("need 2 or more non-zero row marginals")
    return(FALSE)
  } else if (nc <= 1L) {
    warning("need 2 or more non-zero column marginals")
    return(FALSE)
  } else {
    return(TRUE)
  }
}

sig_diffs_nongauss <- function(samples, fact, conf.level = conf.level) {
  # function to produce a table similar to that produced for TukeyHSD,
  # but for non-normally distributed data
  # calculate p values for each data classification based on pairwise.wilcox.test
  
  if (missing(conf.level)) {
    conf.level <- 0.95
  }
  
  ufactor <- levels(fact)
  pwt <- pairwise.wilcox.test(samples, fact, conf.level = conf.level)
  factormeans <- matrix(0, length(ufactor), 1)
  for (ii in 1:length(ufactor)) {
    pos <- which(fact == ufactor[ii])
    
    factormeans[ii] <- mean(samples[pos])
  }
  
  # make a matrix with a row for every possible combination of
  # 2 data classifications and populate it with the calculated
  # p values
  
  xcomb <- combn(length(ufactor), 2)
  tukeylike <- matrix(0, ncol(xcomb), 4)
  colnames(tukeylike) <- c("diff", "lwr", "upr", "p adj")
  tukeynames <- vector("list", ncol(xcomb))
  for (ii in 1:ncol(xcomb)) {
    tukeynames[ii] <-
      paste(ufactor[xcomb[2, ii]], "-", ufactor[xcomb[1, ii]], sep = "")
    
    p_value <- pwt$p.value[xcomb[2, ii] - 1, xcomb[1, ii]]
    
    if (is.na(p_value)) {
      p_value <- 1
    }
    
    tukeylike[ii, 4] <- p_value
    tukeylike[ii, 1] <- NA
    tukeylike[ii, 2] <- NA
    tukeylike[ii, 3] <- NA
  }
  rownames(tukeylike) <- tukeynames
  
  # re-format the table slightly so it is the same as that produced
  # by TukeyHSD and output
  
  tukeylike2 <- list(tukeylike)
  
  return(tukeylike2)
}


conf_band <- function(x, reg, conf.level = conf.level, up) {
  # reg: result of linear regression lm
  # up: fact plus or minus
  if (missing(conf.level)) {
    conf.level <- 0.95
  }
  
  if (missing(up)) {
    up <- 1
  }
  alpha <- 1 - conf.level
  a <- reg$coefficients[2] # slope
  b <- reg$coefficients[1] # constant
  md <- x - mean(x)
  
  result <- x # initialization
  
  # formula standard error of the regression line at point x:
  # https://stats.stackexchange.com/questions/101318/understanding-shape-and-calculation-of-confidence-bands-in-linear-regression
  # See also statistics script page 124: konfidenzint4erval
  
  # http://www.sthda.com/english/articles/40-regression-analysis/166-predict-in-r-model-predictions-and-confidence-intervals/
  for (i in 1:length(x)) {
    result[i] <- a * x[i] + b +
      up * qt(1 - alpha / 2, length(x) - 2) *
      # Standard error of the estimate
      sqrt(sum(reg$resid * reg$resid) / (length(x) - 2)) *
      #
      sqrt(1 / (length(x)) + md[i]^2 / sum(md * md))
  }
  return(result)
}

progn_band <- function(x, reg, conf.level, up) {
  if (missing(conf.level)) {
    conf.level <- 0.95
  }
  alpha <- 1 - conf.level
  if (missing(up)) {
    up <- 1
  }
  a <- reg$coefficients[2]
  b <- reg$coefficients[1]
  md <- x - mean(x)
  
  result <- x
  
  for (i in 1:length(x)) {
    result[i] <- a * x[i] + b +
      up * qt(1 - alpha / 2, length(x) - 2) *
      # Standard error of the estimate
      sqrt(sum(reg$resid * reg$resid) / (length(x) - 2)) *
      #
      sqrt(1 + 1 / (length(x)) + md[i]^2 / sum(md * md))
  }
  return(result)
}
calculate_comparepvalue <- function(p_value, conf.level) {
  if (p_value < 1 - conf.level) {
    comparepvalue <- "<"
  } else {
    comparepvalue <- ">"
  }
  return(comparepvalue)
}

# Check for normality with Shapiro-Wilk test without visualization----
test_norm <- function(x) {
  # Remove NA from x
  x <- x[!is.na(x)]
  #  KS = ks.test(x, pnorm, mean(x), sd(x))
  shapiro_wilk_test <- shapiro.test(x)
  # my_list = list("Kolmogorov-Smirnoff" = KS, "Shapiro" =SH)
  return(shapiro_wilk_test)
}

# Check length of distributions for t-test----
check_assumption_sample_size_t_test <- function(x1, x2, minimum_size) {
  # x1 sample 1
  # x2 sample 2
  # minimum_size:return TRUE if length> minimum_size
  if (length(x1) > minimum_size & length(x2) > minimum_size) {
    return(TRUE)
  } else {
    return(FALSE)
  }
}

# Visual constants ----

# Maximum per-group n for showing jitter overlay on box plots.
# Above this threshold the stripchart is suppressed and boxplot() reverts to
# its classical Tukey behaviour (outliers beyond 1.5*IQR shown as points).
jitter_max_n <- 50

# Define color scheme-----

#' \code{colorscheme(x)} selects color scheme of graphical output. Function parameter NULL lists all available color schemes, 1 a color tuple of green and blue
#' 2 a color tuple of dark green and turquoi, 3 a colorplaette as defined by RcolorBrewer

#'
#' @param colorcode selects color scheme. parameters NULL: list of all available color schemes, 1: colortuple, 2, colortuple2, 3, ColorPalette
#' @return selected color scheme, colors are given with their Hex Code #RRGGBB names
#' @noRd

colorscheme <- function(colorcode = NULL) {
  browserLightGreen <- "#B8E0B8" # matched part group0
  browserLightBlue <- "#B3D1EF" # matched part group1
  browserLightTurquois <- "#B3E1EF" # light turquois
  browserDarkGreen <- "#5CB85C" # dark green
  colortuple <- c(browserLightGreen, browserLightBlue)
  colortuple2 <- c(browserDarkGreen, browserLightTurquois)
  # from package RColorBrewer Set 3
  ColorPalette <- c(
    "#8DD3C7",
    "#FFFFB3",
    "#BEBADA",
    "#FB8072",
    "#80B1D3",
    "#FDB462",
    "#B3DE69",
    "#FCCDE5",
    "#D9D9D9",
    "#BC80BD",
    "#CCEBC5",
    "#FFED6F"
  )
  
  
  my_list <- list(
    "colortuple" = colortuple,
    "colortuple2" = colortuple2,
    "ColorPalette" = ColorPalette
  )
  
  if (is.null(colorcode)) {
    return(my_list)
  } else if (colorcode == 1) {
    return(colortuple)
  } else if (colorcode == 2) {
    return(colortuple2)
  } else if (colorcode == 3) {
    return(ColorPalette)
  } else {
    message("Choose valid parameter: NULL, 1,2 or 3")
  }
}

resetPar <- function() {
  dev.new
  while (!is.null(dev.list()))
    dev.off() # restores to default values
  oldpar <- par(no.readonly = TRUE)
  return(oldpar)
}

library(testthat)
library(nortest)  # for ad.test
library(multcompView)  # for multcompLetters
library(vcd) # for mosaic plots

# Test suite for test_and_visuals.R functions
# Note: Many functions create plots, so we focus on testing return values and error handling

# Helper function to create test data
create_test_data <- function() {
  set.seed(123)
  list(
    numeric_data = rnorm(30, mean = 10, sd = 2),
    factor_data = factor(rep(c("A", "B", "C"), each = 10)),
    binary_factor = factor(rep(c("Group1", "Group2"), each = 15)),
    count_data = factor(sample(c("Cat1", "Cat2", "Cat3"), 50, replace = TRUE)),
    x_regression = seq(1, 20, length.out = 30),
    y_regression = 2 * seq(1, 20, length.out = 30) + rnorm(30, 0, 1)
  )
}

# Tests for helper functions
test_that("colorscheme function works correctly", {
  # Test listing all schemes
  all_schemes <- colorscheme(NULL)
  expect_type(all_schemes, "list")
  expect_named(all_schemes, c("colortuple", "colortuple2", "ColorPalette"))
  
  # Test individual schemes
  scheme1 <- colorscheme(1)
  expect_type(scheme1, "character")
  expect_length(scheme1, 2)
  expect_match(scheme1[1], "^#[A-Fa-f0-9]{6}$")  # hex color format
  
  scheme2 <- colorscheme(2)
  expect_type(scheme2, "character")
  expect_length(scheme2, 2)
  
  scheme3 <- colorscheme(3)
  expect_type(scheme3, "character")
  expect_length(scheme3, 12)
  
  # Test invalid input
  expect_message(colorscheme(4), "Choose valid parameter")
})

test_that("create_two_samples_vector works correctly", {
  data <- create_test_data()
  
  result <- create_two_samples_vector(data$numeric_data, data$binary_factor)
  
  expect_type(result, "list")
  expect_named(result, c("sample1", "sample2", "sample1and2"))
  expect_length(result$sample1, 15)
  expect_length(result$sample2, 15)
  expect_length(result$sample1and2, 30)
  
  # Test with more than 2 levels
  expect_warning(
    create_two_samples_vector(data$numeric_data, data$factor_data),
    "only two level input allowed"
  )
})

test_that("calc_min_max_of_y_axis works correctly", {
  test_data <- c(1, 5, 10, 15, 20)
  
  result <- calc_min_max_of_y_axis(test_data, 0.1, 0.2)
  
  expect_type(result, "list")
  expect_length(result, 2)
  
  min_val <- result[[1]]
  max_val <- result[[2]]
  
  expect_true(min_val < min(test_data))
  expect_true(max_val > max(test_data))
  expect_type(min_val, "double")
  expect_type(max_val, "double")
})

test_that("check_assumptions_shapiro works correctly", {
  # Valid data
  valid_data <- rnorm(100)
  expect_true(check_assumptions_shapiro(valid_data))
  
  # Too small sample
  small_data <- c(1, 2)
  expect_warning(check_assumptions_shapiro(small_data), "sample size must be between 3 and 5000")
  expect_false(suppressWarnings(check_assumptions_shapiro(small_data)))
  
  # All identical values
  identical_data <- rep(5, 10)
  expect_warning(check_assumptions_shapiro(identical_data), "all 'x' values are identical")
  expect_false(suppressWarnings(check_assumptions_shapiro(identical_data)))
})

test_that("check_assumptions_count_data works correctly", {
  # Valid count data
  samples <- factor(c("A", "A", "B", "B", "C", "C"))
  fact <- factor(c("X", "Y", "X", "Y", "X", "Y"))
  expect_true(check_assumptions_count_data(samples, fact))
  
  # Invalid data - only one row
  samples_bad <- factor(c("A", "A", "A", "A"))
  fact_bad <- factor(c("X", "Y", "X", "Y"))
  expect_warning(
    check_assumptions_count_data(samples_bad, fact_bad),
    "need 2 or more non-zero row marginals"
  )
  expect_false(suppressWarnings(check_assumptions_count_data(samples_bad, fact_bad)))
})

test_that("makeTable works correctly", {
  samples <- factor(c("A", "A", "B", "B", "C"))
  fact <- factor(c("X", "Y", "X", "Y", "X"))
  
  result <- makeTable(samples, fact, "Sample", "Factor")
  
  expect_s3_class(result, "table")
  expect_equal(length(dimnames(result)), 2)  # Should have row and column names
  expect_true(all(colSums(result) > 0))  # no zero columns
  
  # Check that result is properly structured (don't assume specific order)
  expect_true(nrow(result) >= 2)  # Should have at least 2 rows
  expect_true(ncol(result) >= 2)  # Should have at least 2 columns
  
  # Check that the table contains our data
  expect_true(sum(result) == 5)  # Total count should be 5
})

test_that("test_norm works correctly", {
  # Normal data
  normal_data <- rnorm(100)
  result <- test_norm(normal_data)
  
  expect_s3_class(result, "htest")
  expect_equal(result$method, "Shapiro-Wilk normality test")
  expect_true("p.value" %in% names(result))
  
  # Data with NAs
  data_with_na <- c(rnorm(50), NA, NA)
  result_na <- test_norm(data_with_na)
  expect_s3_class(result_na, "htest")
})

test_that("check_assumption_sample_size_t_test works correctly", {
  x1 <- rnorm(10)
  x2 <- rnorm(15)
  
  # Sufficient sample sizes
  expect_true(check_assumption_sample_size_t_test(x1, x2, 5))
  
  # Insufficient sample sizes
  expect_false(check_assumption_sample_size_t_test(x1, x2, 12))
})

test_that("side_of_nh works correctly", {
  expect_equal(side_of_nh("less"), ">=")
  expect_equal(side_of_nh("greater"), "<=")
  expect_equal(side_of_nh("two.sided"), "equals")
})

test_that("calculate_comparepvalue works correctly", {
  expect_equal(calculate_comparepvalue(0.01, 0.95), "<")
  expect_equal(calculate_comparepvalue(0.1, 0.95), ">")
})

test_that("type_sample_fact works correctly", {
  samples <- rnorm(10)
  fact <- factor(rep(c("A", "B"), 5))
  
  result <- type_sample_fact(samples, fact)
  
  expect_type(result, "list")
  expect_named(result, c("typesample", "typefactor"))
  expect_equal(result$typesample, "numeric")
  expect_equal(result$typefactor, "factor")
})

# Tests for main statistical functions (focusing on return values and error handling)
test_that("test_norm_vis returns correct structure", {
  skip_if_not_installed("nortest")
  
  data <- rnorm(100, mean = 10, sd = 2)
  
  # Capture plot output and test return value
  result <- test_norm_vis(data)
  
  expect_type(result, "list")
  expect_named(result, c("Anderson-Darling", "Shapiro"))
  expect_s3_class(result$`Anderson-Darling`, "htest")
  expect_s3_class(result$Shapiro, "htest")
})

test_that("fisher_chi works correctly", {
  # Create a contingency table
  counts <- matrix(c(10, 5, 8, 12), nrow = 2)
  dimnames(counts) <- list(c("Row1", "Row2"), c("Col1", "Col2"))
  
  result <- fisher_chi(counts)
  
  expect_s3_class(result, "htest")
  expect_true("p.value" %in% names(result))
  
  # Check that method is one of the expected chi-square or Fisher test variants
  # The exact method name might vary depending on the implementation
  expect_true(grepl("Chi-squared|Fisher|chi", result$method, ignore.case = TRUE))
  
  # Check that we get a valid p-value
  expect_true(is.numeric(result$p.value))
  expect_true(result$p.value >= 0 && result$p.value <= 1)
})

test_that("fisher_chi 2x2: exact Fisher with OR and CI", {
  # Fisher's tea-tasting example: small counts -> Cochran violated -> exact Fisher
  counts <- matrix(c(7, 1, 1, 7), nrow = 2)
  dimnames(counts) <- list(c("milk_first", "tea_first"), c("correct", "wrong"))

  result <- fisher_chi(counts)

  expect_s3_class(result, "htest")
  expect_true(grepl("Fisher", result$method))
  # exact Fisher (not simulated) must return OR and CI
  expect_false(is.null(result$estimate))
  expect_false(is.null(result$conf.int))
  expect_true(is.numeric(result$estimate))
  expect_length(result$conf.int, 2)
})

test_that("fisher_chi larger table: simulate.p.value, no OR", {
  # 3x3 table with small counts -> Cochran violated -> simulated Fisher -> no OR
  counts <- matrix(c(1, 2, 1, 2, 1, 2, 1, 2, 1), nrow = 3)
  dimnames(counts) <- list(c("A", "B", "C"), c("X", "Y", "Z"))

  result <- fisher_chi(counts)

  expect_s3_class(result, "htest")
  expect_true(grepl("Fisher", result$method))
  # simulated Fisher does not return OR
  expect_null(result$estimate)
})

test_that("odds_ratio function works correctly", {
  # Test basic odds ratio calculation
  result <- odds_ratio(a = 10, b = 5, c = 8, d = 12, alpha = 0.05)
  
  expect_type(result, "double")
  expect_true(nrow(result) == 4)  # OR, lowconf, upconf, SE
  expect_true(all(is.finite(result[!is.nan(result)])))
})

# Tests for visualization functions (checking they run without error)
test_that("vis_chi_squared_test runs without error for valid data", {
  skip_if_not_installed("vcd")
  
  # Create categorical data
  samples <- factor(sample(c("Cat1", "Cat2"), 50, replace = TRUE))
  fact <- factor(sample(c("Group1", "Group2"), 50, replace = TRUE))
  
  expect_no_error({
    result <- vis_chi_squared_test(samples, fact, "Sample", "Factor")
  })
})

test_that("two_sample_t_test runs without error for valid data", {
  data <- create_test_data()
  
  expect_no_error({
    result <- two_sample_t_test(
      samples = data$numeric_data,
      fact = data$binary_factor,
      samplename = "Values",
      factorname = "Groups"
    )
  })
})

test_that("two_sample_wilcoxon_test runs without error for valid data", {
  data <- create_test_data()
  
  expect_no_error({
    result <- two_sample_wilcoxon_test(
      samples = data$numeric_data,
      fact = data$binary_factor,
      samplename = "Values",
      factorname = "Groups"
    )
  })
})

test_that("vis_anova runs without error for valid data", {
  skip_if_not_installed("multcompView")
  
  data <- create_test_data()
  
  expect_no_error({
    result <- vis_anova(
      samples = data$numeric_data,
      fact = data$factor_data,
      samplename = "Values",
      factorname = "Groups"
    )
  })
})

test_that("vis_Kruskal_Wallis runs without error for valid data", {
  skip_if_not_installed("multcompView")
  
  data <- create_test_data()
  
  expect_no_error({
    result <- vis_Kruskal_Wallis(
      samples = data$numeric_data,
      fact = data$factor_data,
      samplename = "Values",
      factorname = "Groups"
    )
  })
})



test_that("vis_normality_assumptions runs without error for valid data", {
  skip_if_not_installed("nortest")
  
  data <- create_test_data()
  
  expect_no_error({
    result <- vis_normality_assumptions(
      y = data$y_regression,
      x = data$x_regression
    )
  })
})

# Test error handling for main functions
test_that("two_sample_wilcoxon_test handles invalid inputs", {
  expect_warning(
    two_sample_wilcoxon_test(
      samples = c("not", "numeric"),
      fact = factor(c("A", "B")),
      conf.level = 0.95
    ),
    "'samples' must be numeric"
  )
  
  expect_warning(
    two_sample_wilcoxon_test(
      samples = c(1, 2),
      fact = c("A", "B"),  # not a factor
      conf.level = 0.95
    ),
    "'fact' must be factorial"
  )
  
  expect_warning(
    two_sample_wilcoxon_test(
      samples = c(1, 2),
      fact = factor(c("A", "B")),
      conf.level = 1.5  # invalid confidence level
    ),
    "'conf.level' must be a single number between 0 and 1"
  )
})

test_that("two_sample_t_test handles invalid confidence levels", {
  data <- create_test_data()
  
  expect_error(
    two_sample_t_test(
      samples = data$numeric_data,
      fact = data$binary_factor,
      conf.level = 1.5
    ),
    "'conf.level' must be a single number between 0 and 1"
  )
})

# Test confidence band functions
test_that("conf_band and progn_band work correctly", {
  x <- 1:10
  y <- 2 * x + rnorm(10, 0, 0.5)
  reg <- lm(y ~ x)
  
  conf_result <- conf_band(x, reg, conf.level = 0.95, up = 1)
  expect_type(conf_result, "double")
  expect_length(conf_result, length(x))
  
  progn_result <- progn_band(x, reg, conf.level = 0.95, up = 1)
  expect_type(progn_result, "double")
  expect_length(progn_result, length(x))
  
  # Prediction bands should be wider than confidence bands
  expect_true(all(progn_result >= conf_result))
})

# Test that functions handle missing parameters correctly
test_that("functions handle missing optional parameters", {
  data <- create_test_data()
  
  # Test functions with missing conf.level
  expect_no_error({
    two_sample_wilcoxon_test(
      samples = data$numeric_data,
      fact = data$binary_factor
    )
  })
  
  expect_no_error({
    vis_anova(
      samples = data$numeric_data,
      fact = data$factor_data
    )
  })
})

# Test graphics parameter restoration
test_that("functions restore graphics parameters", {
  # Save original parameters
  original_par <- par(no.readonly = TRUE)
  
  # Run a function that changes parameters
  data <- create_test_data()
  test_norm_vis(data$numeric_data)
  
  # Check that most important parameters are restored
  current_par <- par(no.readonly = TRUE)
  
  # Key parameters should be restored (allowing for minor differences)
  expect_equal(current_par$mfrow, original_par$mfrow)
  expect_equal(current_par$oma, original_par$oma)
})

# Performance test for large datasets
test_that("functions handle reasonably large datasets", {
  skip_on_cran()  # Skip on CRAN to avoid long test times
  
  large_data <- rnorm(1000)
  large_factor <- factor(rep(c("A", "B"), 500))
  
  expect_no_error({
    result <- test_norm(large_data)
  })
  
  expect_no_error({
    result <- create_two_samples_vector(large_data, large_factor)
  })
})

#' Breusch-Pagan Test for Heteroscedasticity
#'
#' @description Performs the Breusch-Pagan test for heteroscedasticity in 
#' linear regression models. Tests the null hypothesis that the error variance 
#' is constant (homoscedasticity) against the alternative that the error 
#' variance depends on the fitted values.
#'
#' @param model A fitted linear model object (from \code{lm()}).
#'
#' @return An object of class \code{"htest"} with components:
#' \item{statistic}{the value of the chi-squared test statistic.}
#' \item{parameter}{degrees of freedom.}
#' \item{p.value}{the p-value of the test.}
#' \item{method}{a character string indicating the test performed.}
#' \item{data.name}{a character string giving the name of the model.}
#'
#' @details 
#' The Breusch-Pagan test regresses the squared standardized residuals on 
#' the fitted values. The test statistic is:
#' 
#' \deqn{BP = n \cdot R^2}
#' 
#' where:
#' \itemize{
#'   \item \eqn{n} = sample size
#'   \item \eqn{R^2} = coefficient of determination from auxiliary regression of 
#'         \eqn{e_i^2} on \eqn{\hat{y}_i}
#' }
#' 
#' Under the null hypothesis of homoscedasticity, the test statistic follows 
#' a chi-squared distribution: \eqn{BP \sim \chi^2(p-1)} where \eqn{p} is the 
#' number of parameters in the model (including intercept).
#' 
#' Large values of the test statistic (small p-values) provide evidence against 
#' homoscedasticity.
#' 
#' @references
#' Breusch, T. S., and Pagan, A. R. (1979). A simple test for heteroscedasticity 
#' and random coefficient variation. Econometrica, 47(5), 1287-1294.
#' DOI: 10.2307/1911963
#' 
#' @examples
#' # Example with homoscedastic errors
#' set.seed(123)
#' x <- runif(100)
#' y <- 2 + 3*x + rnorm(100, sd = 1)
#' model1 <- lm(y ~ x)
#' bp.test(model1)  # Should not reject (p > 0.05)
#' 
#' # Example with heteroscedastic errors (variance increases with x)
#' set.seed(456)
#' x <- runif(100)
#' y <- 2 + 3 *x + rnorm(100, sd = 0.5 + 2*x)
#' model2 <- lm(y ~ x)
#' bp.test(model2)  # Should reject (p < 0.05)
#'
#' @export

bp.test <- function(model) {
  
  # Error handling
  if (!inherits(model, "lm")) {
    stop("model must be an object of class 'lm'")
  }
  
  # Extract residuals and fitted values
  residuals <- residuals(model)
  fitted_values <- fitted(model)
  n <- length(residuals)
  
  # Number of parameters (including intercept)
  p <- length(coef(model))
  
  # Standardize residuals
  sigma2 <- sum(residuals^2) / n
  std_residuals <- residuals / sqrt(sigma2)
  
  # Square the standardized residuals
  sq_std_residuals <- std_residuals^2
  
  # Auxiliary regression: sq_std_residuals ~ fitted_values
  aux_model <- lm(sq_std_residuals ~ fitted_values)
  
  # Get R-squared from auxiliary regression
  r_squared <- summary(aux_model)$r.squared
  
  # Calculate test statistic: n * R^2
  bp_statistic <- n * r_squared
  
  # Degrees of freedom (number of predictors in auxiliary regression)
  # For simple linear regression with 1 predictor, df = 1
  df <- p - 1
  
  # Calculate p-value from chi-squared distribution
  p_value <- pchisq(bp_statistic, df = df, lower.tail = FALSE)
  
  # Return htest object
  structure(
    list(
      statistic = c(BP = bp_statistic),
      parameter = c(df = df),
      p.value = p_value,
      method = "Breusch-Pagan test for heteroscedasticity",
      data.name = deparse(substitute(model))
    ),
    class = "htest"
  )
}

test_that("vis_lm_assumptions returns correct structure for ANOVA", {
  # Setup test data
  ToothGrowth$dose <- as.factor(ToothGrowth$dose)
  
  # Run function
  result <- vis_lm_assumptions(ToothGrowth$len, ToothGrowth$dose)
  
  # Test structure - should have 6 elements
  expect_named(result, c("summary_anova", "shapiro_test", "ad_test", 
                         "levene_test", "bartlett_test", "bp_test"))
  
  # For ANOVA (regression=FALSE): levene and bartlett exist, bp_test is NULL
  expect_s3_class(result$levene_test, "htest")
  expect_s3_class(result$bartlett_test, "htest")
  expect_null(result$bp_test)
  
  # Check class of summary_anova
  expect_s3_class(result$summary_anova, "summary.aov")
  
  # Check normality tests
  expect_s3_class(result$shapiro_test, "htest")
  expect_true(is.list(result$ad_test) || is.character(result$ad_test))
})

test_that("vis_lm_assumptions returns correct structure for regression", {
  # Run function with regression=TRUE
  result <- vis_lm_assumptions(mtcars$mpg, as.factor(mtcars$cyl), regression = TRUE)
  
  # Test structure - should have 6 elements
  expect_named(result, c("summary_anova", "shapiro_test", "ad_test", 
                         "levene_test", "bartlett_test", "bp_test"))
  
  # For regression (regression=TRUE): bp_test exists, levene and bartlett are NULL
  expect_null(result$levene_test)
  expect_null(result$bartlett_test)
  expect_s3_class(result$bp_test, "htest")
  
  # Check that bp_test has correct structure
  expect_named(result$bp_test, c("statistic", "parameter", "p.value", "method", "data.name"))
  expect_equal(result$bp_test$method, "Breusch-Pagan test for heteroscedasticity")
  
  # Check class of summary_anova (aov is used even for regression in this function)
  expect_s3_class(result$summary_anova, "summary.aov")
  
  # Check normality tests
  expect_s3_class(result$shapiro_test, "htest")
  expect_true(is.list(result$ad_test) || is.character(result$ad_test))
})

test_that("vis_lm_assumptions handles small samples correctly", {
  # Create small sample (n < 7)
  small_sample <- c(1, 2, 3, 4, 5)
  small_groups <- factor(c(1, 1, 2, 2, 2))
  
  result <- vis_lm_assumptions(small_sample, small_groups)
  
  # Should still return structure but with warnings for small n
  expect_named(result, c("summary_anova", "shapiro_test", "ad_test", 
                         "levene_test", "bartlett_test", "bp_test"))
  
  # Anderson-Darling should be character message for n < 7
  expect_type(result$ad_test, "character")
})

test_that("vis_lm_assumptions produces plots", {
  ToothGrowth$dose <- as.factor(ToothGrowth$dose)
  
  # Should not error when plotting
  expect_no_error({
    result <- vis_lm_assumptions(ToothGrowth$len, ToothGrowth$dose)
  })
})

test_that("vis_lm_assumptions handles NA values", {
  # Create data with NAs
  data_with_na <- c(1, 2, 3, NA, 5, 6, 7, 8, 9, 10)
  groups_with_na <- factor(c(1, 1, 1, 1, 1, 2, 2, 2, 2, 2))
  
  # Should handle NAs gracefully
  expect_no_error({
    result <- vis_lm_assumptions(data_with_na, groups_with_na)
  })
})

test_that("vis_lm_assumptions cex parameter works", {
  ToothGrowth$dose <- as.factor(ToothGrowth$dose)
  
  # Should accept different cex values
  expect_no_error({
    result <- vis_lm_assumptions(ToothGrowth$len, ToothGrowth$dose, cex = 0.8)
  })
  
  expect_no_error({
    result <- vis_lm_assumptions(ToothGrowth$len, ToothGrowth$dose, cex = 1.5)
  })
})

test_that("bp_test gives reasonable results", {
  # Test with homoscedastic data
  set.seed(123)
  x <- runif(100)
  y <- 2 + 3*x + rnorm(100, sd = 1)
  model1 <- lm(y ~ x)
  
  result1 <- bp.test(model1)
  
  expect_s3_class(result1, "htest")
  expect_named(result1, c("statistic", "parameter", "p.value", "method", "data.name"))
  expect_equal(result1$method, "Breusch-Pagan test for heteroscedasticity")
  expect_true(result1$p.value > 0.01)  # Should not reject for homoscedastic data
  
  # Test with heteroscedastic data
  set.seed(456)
  x2 <- runif(100)
  y2 <- 2 + 3*x2 + rnorm(100, sd = 0.5 + 2*x2)
  model2 <- lm(y2 ~ x2)
  
  result2 <- bp.test(model2)
  
  expect_s3_class(result2, "htest")
  expect_equal(result2$method, "Breusch-Pagan test for heteroscedasticity")
  expect_true(result2$p.value < 0.05)  # Should reject for heteroscedastic data
})

test_that("vis_lm_assumptions uses plot.lm correctly", {
  # Test that plots are generated without errors
  ToothGrowth$dose <- as.factor(ToothGrowth$dose)
  
  # Capture plot output to check it runs
  expect_no_error({
    pdf(NULL)  # Use null device to suppress actual plotting
    result <- vis_lm_assumptions(ToothGrowth$len, ToothGrowth$dose)
    dev.off()
  })
  
  # For regression
  expect_no_error({
    pdf(NULL)
    result <- vis_lm_assumptions(mtcars$mpg, as.factor(mtcars$cyl), regression = TRUE)
    dev.off()
  })
})

library(visStatistics)

# Examples------
while (!is.null(dev.list())) {
  dev.off()
}


options(warn = 0) # for debugging also warnings
filedir <- tempdir()

# Student's t-test (equal variances, two groups)----
ttest = visstat(sleep$group, sleep$extra)
tvalue <- unname(ttest[[3]]$statistic)
# Show equivalance of student's t-test to aov approach-----
aov_sleep <- aov(extra ~ group, data = sleep)
summary_aov <- summary(aov_sleep)
F_anova <- (summary_aov[[1]][["F value"]][1])
stopifnot(all.equal(tvalue^2, F_anova))
# Show equivalance of student's t-test to lm approach---
# Extract t - statistics
lm_sleep <- lm(extra ~ group, data = sleep)
summary_lm <- summary(lm_sleep)
abst = (summary_lm$coefficients[2, 3])
stopifnot(all.equal(abst, abs(tvalue)))
beta1 = summary_lm$coefficients[2, 1]
# Check group means
mean1 = mean(sleep$extra[sleep$group == 1])
mean2 = mean(sleep$extra[sleep$group == 2])
stopifnot(all.equal(beta1, abs(mean2 - mean1)))



# Welch's t-test (unequal variances, two groups)----
visstat(mtcars$am, mtcars$mpg)


# Wilcoxon-----
grades_gender <- data.frame(
  Sex = as.factor(c(rep("Girl", 20), rep("Boy", 20))),
  Grade = c(
    19.25,
    18.1,
    15.2,
    18.34,
    7.99,
    6.23,
    19.44,
    20.33,
    9.33,
    11.3,
    18.2,
    17.5,
    10.22,
    20.33,
    13.3,
    17.2,
    15.1,
    16.2,
    17.3,
    16.5,
    5.1,
    15.25,
    17.41,
    14.5,
    15,
    14.3,
    7.53,
    15.23,
    6,
    17.33,
    7.25,
    14,
    13.5,
    8,
    19.5,
    13.4,
    17.5,
    17.4,
    16.5,
    15.6
  )
)
visstat(grades_gender$Sex, grades_gender$Grade)

# Fisher's ANOVA (equal variances, >2 groups)
visstat(PlantGrowth$group, PlantGrowth$weight)


# Welch ANOVA 1 -----
set.seed(1)
welch_anova_data <- data.frame(group = factor(rep(c("A", "B", "C"), each = 20)),
                               value = c(
                                 rnorm(20, mean = 50, sd = 5),
                                 rnorm(20, mean = 55, sd = 10),
                                 rnorm(20, mean = 60, sd = 15)
                               ))
welch_annova = visstat(welch_anova_data$group, welch_anova_data$value)

set.seed(123)
# Welch ANOVA  with 10 groups  -----
# 
# Create data with 10 groups
groups_10 <- data.frame(
  response = c(
    rnorm(15, mean = 10, sd = 2),  # Group 1
    rnorm(18, mean = 11, sd = 2),  # Group 2
    rnorm(22, mean = 12, sd = 2),  # Group 3
    rnorm(19, mean = 13, sd = 2),  # Group 4
    rnorm(21, mean = 14, sd = 2),  # Group 5
    rnorm(20, mean = 12, sd = 2),  # Group 6
    rnorm(19, mean = 11, sd = 2),  # Group 7
    rnorm(18, mean = 13, sd = 2),  # Group 8
    rnorm(22, mean = 12, sd = 2),  # Group 9
    rnorm(21, mean = 14, sd = 2)   # Group 10
  ),
  group = factor(rep(paste0("G", 1:10), times = c(15, 18, 22, 19, 21, 20, 19, 18, 22, 21)))
)



result_10 <- visstat(groups_10, "response", "group")


# Kruskal-Wallis test: iris----
iris_data = visstat(iris$Species, iris$Petal.Width)

iris_data_stored = visstat(
  iris$Species,
  iris$Petal.Width,
  graphicsoutput = "pdf",
  plotName = "iris_kruskal",
  plotDirectory = filedir
)

# Linear regression: trees data set  ----
dev.off()
linear_regression_trees <- visstat(trees$Girth, trees$Volume,conf.level = 0.99)

plot(linear_regression_trees, which = 1) # replays assumption plot
plot(linear_regression_trees, which = 2)

# Spearman-correlation----
result_swiss1 <- visstat(swiss$Education,
                         swiss$Fertility, correlation = TRUE)
# Kendall-correlation----
set.seed(42)
n <- 150
# Negative monotone association: higher alcohol -> lower academic performance
xs <- sample(1:5, n, replace = TRUE)
ys <- pmin(5, pmax(1, (6 - xs) + sample(-1:1, n, replace = TRUE)))
likert_levels  <- c("never", "rarely", "sometimes", "often", "always")
likert_levels2 <- c("poor", "fair", "ok", "good", "great")
alcohol     <- ordered(likert_levels[xs],  levels = likert_levels)
performance <- ordered(likert_levels2[ys], levels = likert_levels2)
kendall_result <- visstat(performance, alcohol, correlation = TRUE)


# Chi squared, mosaic plots with HairEyeColor----
# HairEyeColor data set: Pearsons Chi squared, mosaic plot with Pearson's residuals
HairEyeColorMale <- counts_to_cases(as.data.frame(HairEyeColor[, , 1]))
visstat(HairEyeColorMale$Eye, HairEyeColorMale$Hair)
HairEyeColorMaleFisher <- HairEyeColor[, , 1]
# replace cells to smaller values to enforce Cochran's rule
HairEyeColorMaleFisher[HairEyeColorMaleFisher < 10] <- 4
HairEyeColorMaleFisher <- counts_to_cases(as.data.frame(HairEyeColorMaleFisher))

res_chi <- visstat(
  HairEyeColorMaleFisher$Eye,
  HairEyeColorMaleFisher$Hair
) # stores two graphics outputs

# Fisher-test 2x2 contingency tables----
HairEyeColorMaleFisher <- HairEyeColor[, , 1]
# slicing out a 2 x2 contingency table
blackBrownHazelGreen <- HairEyeColorMaleFisher[1:2, 3:4]


blackBrownHazelGreen <- counts_to_cases(as.data.frame(blackBrownHazelGreen))
fisher_stats <- visstat(blackBrownHazelGreen, "Hair", "Eye")

# Fisher-test 2x2: tea-tasting example (Fisher 1935)
# Small cell counts -> Cochran violated -> exact Fisher -> OR and CI in output
tea <- data.frame(
  poured  = as.factor(c(rep("milk_first", 8), rep("tea_first", 8))),
  correct = as.factor(c(rep("yes", 7), rep("no", 1), rep("yes", 1), rep("no", 7)))
)
tea_stats <- visstat(tea$poured, tea$correct)
tea_stats$estimate   # odds ratio
tea_stats$conf.int   # 95% CI


# The visstat-methods -------
summary(iris_data_stored)
print(iris_data_stored)
plot(iris_data_stored) #file paths to plots
plot(iris_data, which = 1) #replay plot 1


# Replay plots  or file paths of stored graphics------

plot(fisher_stats, which = 1) # column  plot 


# Saving the graphical output to a user specified plotDirectory in user specified graphicsouput format ----
linear_regression_trees_paths <- visstat(
  trees$Height,
  trees$Girth,
  graphicsoutput = "svg",
  plotName = "trees",
  plotDirectory = filedir
)


# Show all graphics  of type ".png", ".pdf", ".svg", ".ps" in filedir=tempdir()-----
graphicaltypes <- c(".png", ".pdf", ".svg", ".ps")
for (i in graphicaltypes) {
  plotname <- dir(filedir, pattern = i)
  print(file.path(filedir, plotname))
  # file.remove(file.path(filedir, plotname)) # removes all files of type ".png", ".pdf", ".svg", ".ps" in filedir=tempdir()-
}
dev.off()

#' Convert data frame of counts to dataframe of cases.
#' The data frame must contain a column with frequencies (counts) as generated by
#' as.data.frame from a contingency table
#'
#' @param x a \code{data.frame} of counts generated from a contingency table.
#' @param countcol character string, name of the column of x containing the
#' counts. Default name of the column is  'Freq'.

#' @return data frame of cases of dimension (total number of counts as sum of
#' 'Freq' in x) times 2.
#' @examples
#'
#' counts_to_cases(as.data.frame(HairEyeColor[, , 1]), countcol = "Freq")
#'
#' @export counts_to_cases

#'
counts_to_cases <- function(x, countcol = "Freq") {
  # Get the row indices to pull from x
  idx <- rep.int(seq_len(nrow(x)), x[[countcol]])

  # Drop count column
  x[[countcol]] <- NULL

  # Get the rows from x
  x[idx, ]
}

### Header vis_group_normality -----

#' Visualisation of the normality assumption for Welch ANOVA/t-test
#'
#' \code{vis_group_normality} checks for normality of each group separately using 
#' the Shapiro-Wilk and Anderson-Darling tests. The null hypothesis is that 
#' each group is normally distributed. The function generates histograms 
#' with normal distribution overlays and Q-Q plots to visually assess normality.
#' The layout is always 2 rows × k columns (histograms on top, Q-Q plots on bottom).
#'
#' @param samples Numeric vector; the dependent variable.
#' @param groups Factor or vector; the grouping variable (2 to 8 groups for visual display).
#' @param conf.level Numeric; confidence level (default: 0.95). Used to determine 
#'   alpha = 1 - conf.level for normality test interpretation.
#' @param samplename Character; label for the y-axis (default: "").
#' @param groupname Character; label for the x-axis (default: "").
#' @param cex Numeric; scaling factor for plot text and symbols (default: 1).
#'
#' @return A list containing:
#' \describe{
#'   \item{shapiro_tests}{List of Shapiro-Wilk test results for each group}
#'   \item{ad_tests}{List of Anderson-Darling test results for each group}
#'   \item{n_groups}{Number of groups}
#'   \item{group_names}{Names of the groups}
#' }
#'
#' @details
#' Layout is always 2 rows × k columns:
#' \itemize{
#'   \item Top row: Histograms with normal overlay for each group
#'   \item Bottom row: Q-Q plots for each group
#' }
#' For more than 8 groups, a tabular summary is provided instead of plots.
#'
#' @examples
#' # Two groups (like t-test)
#' vis_group_normality(ToothGrowth$len, ToothGrowth$supp)
#' 
#' # Three groups
#' ToothGrowth$dose <- as.factor(ToothGrowth$dose)
#' vis_group_normality(ToothGrowth$len, ToothGrowth$dose)
#'
#' @export

vis_group_normality <- function(samples, 
                                groups, 
                                conf.level = 0.95, 
                                samplename = "", 
                                groupname = "", 
                                cex = 1) {
  
  # Store original par settings
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  
  # Rename for internal consistency (legacy variable names)
  y <- samples
  g <- groups
  
  # Clean data
  complete_cases <- complete.cases(y, g)
  y <- y[complete_cases]
  g <- as.factor(g[complete_cases])
  
  # Get group information
  group_levels <- levels(g)
  k <- length(group_levels)
  
  # Check minimum requirements
  if (k < 2) {
    stop("At least 2 groups required")
  }
  
  # Check if too many groups for meaningful visualization
  if (k > 8) {
    cat("\n")
    cat("===============================================\n")
    cat("  TOO MANY GROUPS FOR VISUAL DISPLAY (k = ", k, ")\n")
    cat("===============================================\n\n")
    cat("Visual plots are not meaningful with more than 8 groups.\n")
    cat("Providing normality test results in tabular format instead.\n\n")
    
    # Calculate test results for all groups
    shapiro_tests <- list()
    ad_tests <- list()
    
    results_table <- data.frame(
      Group = character(k),
      n = integer(k),
      Shapiro_W = numeric(k),
      Shapiro_p = numeric(k),
      AD_A = numeric(k),
      AD_p = numeric(k),
      stringsAsFactors = FALSE
    )
    
    for (i in seq_along(group_levels)) {
      gname <- group_levels[i]
      gdata <- y[g == gname]
      n <- length(gdata)
      
      # Shapiro-Wilk test
      if (n >= 3 && n <= 5000) {
        sh_test <- shapiro.test(gdata)
        shapiro_tests[[gname]] <- sh_test
        results_table$Shapiro_W[i] <- sh_test$statistic
        results_table$Shapiro_p[i] <- sh_test$p.value
      } else {
        shapiro_tests[[gname]] <- list(
          statistic = NA,
          p.value = NA,
          method = paste0("Shapiro-Wilk test requires n: 3-5000 (n = ", n, ")")
        )
        results_table$Shapiro_W[i] <- NA
        results_table$Shapiro_p[i] <- NA
      }
      
      # Anderson-Darling test
      if (n >= 7) {
        ad_test <- nortest::ad.test(gdata)
        ad_tests[[gname]] <- ad_test
        results_table$AD_A[i] <- ad_test$statistic
        results_table$AD_p[i] <- ad_test$p.value
      } else {
        ad_tests[[gname]] <- list(
          statistic = NA,
          p.value = NA,
          method = paste0("Anderson-Darling test requires n >= 7 (n = ", n, ")")
        )
        results_table$AD_A[i] <- NA
        results_table$AD_p[i] <- NA
      }
      
      results_table$Group[i] <- gname
      results_table$n[i] <- n
    }
    
    # Format and print table
    results_table$Shapiro_W <- round(results_table$Shapiro_W, 4)
    results_table$Shapiro_p <- format.pval(results_table$Shapiro_p, digits = 3, eps = 0.001)
    results_table$AD_A <- round(results_table$AD_A, 4)
    results_table$AD_p <- format.pval(results_table$AD_p, digits = 3, eps = 0.001)
    
    cat("Normality Test Results by Group:\n")
    cat("================================\n\n")
    print(results_table, row.names = FALSE)
    
    cat("\n")
    cat("Interpretation:\n")
    alpha <- 1 - conf.level
    cat("  - p >", alpha, ": Data consistent with normality\n")
    cat("  - p <=", alpha, ": Evidence against normality\n")
    cat("\n")
    
    # Count how many groups fail normality
    n_fail_shapiro <- sum(results_table$Shapiro_p != "NA" & 
                            as.numeric(sub("<", "", results_table$Shapiro_p)) < alpha, 
                          na.rm = TRUE)
    n_fail_ad <- sum(results_table$AD_p != "NA" & 
                       as.numeric(sub("<", "", results_table$AD_p)) < alpha, 
                     na.rm = TRUE)
    
    cat("Summary:\n")
    cat("  ", n_fail_shapiro, "out of", k, "groups fail Shapiro-Wilk test (p <", alpha, ")\n")
    cat("  ", n_fail_ad, "out of", k, "groups fail Anderson-Darling test (p <", alpha, ")\n")
    cat("\n")
    
    if (n_fail_shapiro > 0 || n_fail_ad > 0) {
      cat("Recommendation: Consider using Kruskal-Wallis test (non-parametric)\n")
    } else {
      cat("Recommendation: Normality assumption appears reasonable for Welch ANOVA\n")
    }
    cat("\n")
    
    # Return results invisibly
    result <- list(
      shapiro_tests = shapiro_tests,
      ad_tests = ad_tests,
      n_groups = k,
      group_names = group_levels,
      results_table = results_table,
      note = "Too many groups for visual display"
    )
    
    class(result) <- "vis_group_normality"
    return(invisible(result))
  }
  
  # For k <= 8: Visual plots with 2 rows × k columns layout
  
  # Split data by groups
  group_data <- lapply(group_levels, function(lev) y[g == lev])
  names(group_data) <- group_levels
  
  # Run normality tests for each group
  shapiro_tests <- list()
  ad_tests <- list()
  
  for (i in seq_along(group_levels)) {
    gname <- group_levels[i]
    gdata <- group_data[[i]]
    n <- length(gdata)
    
    # Shapiro-Wilk test
    if (n >= 3 && n <= 5000) {
      shapiro_tests[[gname]] <- shapiro.test(gdata)
    } else if (n < 3) {
      shapiro_tests[[gname]] <- list(
        statistic = NA, 
        p.value = NA,
        method = paste0("Shapiro-Wilk test requires n >= 3 (n = ", n, ")")
      )
    } else {
      shapiro_tests[[gname]] <- list(
        statistic = NA,
        p.value = NA,
        method = paste0("Shapiro-Wilk test allows max n = 5000 (n = ", n, ")")
      )
    }
    
    # Anderson-Darling test
    if (n >= 7) {
      ad_tests[[gname]] <- nortest::ad.test(gdata)
    } else {
      ad_tests[[gname]] <- list(
        statistic = NA,
        p.value = NA,
        method = paste0("Anderson-Darling test requires n >= 7 (n = ", n, ")")
      )
    }
  }
  
  # Simple 2 × k layout: histograms on top row, Q-Q plots on bottom row
  # Adjust cex based on number of groups
  if (k <= 3) {
    plot_cex <- 0.6 * cex
    mar_val <- c(4, 4, 3, 1)
  } else if (k <= 5) {
    plot_cex <- 0.55 * cex
    mar_val <- c(3.5, 3.5, 2.5, 0.5)
  } else {
    # k = 6, 7, 8
    plot_cex <- 0.5 * cex
    mar_val <- c(3, 3, 2, 0.5)
  }
  
  par(mfrow = c(2, k),
      mar = mar_val,
      oma = c(0, 0, 2.5, 0),
      cex = plot_cex,
      font.main = 1,  # 1 = plain, not bold
      font.lab = 1,
      font.axis = 1)
  
  # Row 1: All histograms
  for (i in seq_along(group_levels)) {
    gname <- group_levels[i]
    gdata <- group_data[[i]]
    
    # Skip if no data
    if (length(gdata) == 0) {
      plot.new()
      next
    }
    
    # Calculate statistics
    gmean <- mean(gdata, na.rm = TRUE)
    gsd <- sd(gdata, na.rm = TRUE)
    
    # Get p-values for this group
    p_s <- shapiro_tests[[gname]]$p.value
    p_a <- ad_tests[[gname]]$p.value
    
    # --- Histogram with normal overlay ---
    hist_data <- hist(gdata, plot = FALSE)
    max_hist_density <- max(hist_data$density)
    max_normal_peak <- dnorm(gmean, mean = gmean, sd = gsd)
    y_max <- max(max_hist_density, max_normal_peak) * 1.1
    
    # Create title with p-values
    p_s_text <- if (!is.na(p_s)) signif(p_s, 2) else "N/A"
    p_a_text <- if (!is.na(p_a)) signif(p_a, 2) else "N/A"
    main_text <- paste0(gname, "\nSW p=", p_s_text, ", AD p=", p_a_text)
    
    hist(gdata, 
         freq = FALSE,
         main = main_text,
         xlab = "",
         ylab = if (i == 1) "Density" else "",
         ylim = c(0, y_max),
         col = "lightblue", 
         border = "black")
    
    # Overlay normal distribution
    data_range <- range(gdata)
    normal_range <- c(gmean - 3*gsd, gmean + 3*gsd)
    full_range <- c(min(data_range[1], normal_range[1]), 
                    max(data_range[2], normal_range[2]))
    x_seq <- seq(full_range[1], full_range[2], length = 200)
    normal_curve <- dnorm(x_seq, mean = gmean, sd = gsd)
    lines(x_seq, normal_curve, col = "red", lwd = 2)
  }
  
  # Row 2: All Q-Q plots
  for (i in seq_along(group_levels)) {
    gname <- group_levels[i]
    gdata <- group_data[[i]]
    
    # Skip if no data
    if (length(gdata) == 0) {
      plot.new()
      next
    }
    
    # --- Q-Q plot ---
    qqnorm(gdata, 
           main = paste0(gname, " Q-Q"),
           xlab = "Theoretical",
           ylab = if (i == 1) "Sample" else "")
    qqline(gdata, col = "red", lwd = 2)
  }
  
  # Outer title
  mtext("Normality Diagnostics by Group", outer = TRUE, cex = 1.0, line = 0.5)
  
  # Return results
  result <- list(
    shapiro_tests = shapiro_tests,
    ad_tests = ad_tests,
    n_groups = k,
    group_names = group_levels
  )
  
  class(result) <- "vis_group_normality"
  return(invisible(result))
}

#' Cairo wrapper function with plot capture capability
#'
#' Cairo wrapper function returning NULL if not \code{type} is specified.
#' Enhanced version that can capture plots for later replay.
#'
#' \code{openGraphCairo()} \code{Cairo()} wrapper function. Differences to
#' \code{Cairo:} a) prematurely ends the function call to \code{Cairo()}
#' returning NULL, if no output \code{type}  of types 'png', 'jpeg', 'pdf',
#' 'svg', 'ps' or 'tiff' is provided. b) The \code{file} argument of the
#' underlying Cairo function is generated by
#' \code{file.path(fileDirectory,paste(fileName,'.', type, sep = ''))}.
#' c) Can set up plot capture when \code{capture_env} is provided.
#'
#' @param width see \code{Cairo()}
#' @param height see \code{Cairo()}
#' @param fileName name of file to be created. Does not include both file
#'   extension '.\code{type}' and file \code{filedirectory}. Default file name
#'   'visstat_plot'.
#' @param type Supported output types are 'png', 'jpeg', 'pdf', 'svg', 'ps' and
#'   'tiff'. See \code{Cairo()}
#' @param fileDirectory path of directory, where plot is stored. Default current
#'   working directory.
#' @param pointsize see \code{Cairo()}
#' @param bg see \code{Cairo()}
#' @param canvas see \code{Cairo()}
#' @param units see \code{Cairo()}
#' @param dpi DPI used for the conversion of units to pixels. Default value 150.
#' @name openGraphCairo
#' @return NULL, if no \code{type} is specified. Otherwise see \code{Cairo()}
#' @examples
#'
#' ##  adapted from example in \code{Cairo()}
#' openGraphCairo(fileName = "normal_dist", type = "pdf", fileDirectory = tempdir())
#' plot(rnorm(4000), rnorm(4000), col = "#ff000018", pch = 19, cex = 2)
#' dev.off() # creates a file 'normal_dist.pdf' in the directory specified in fileDirectory
#' # ## remove the plot from fileDirectory
#' file.remove(file.path(tempdir(), "normal_dist.pdf"))
#' @import Cairo
#' @export
openGraphCairo <- function(width = 640,
                           height = 480,
                           fileName = NULL,
                           type = NULL,
                           fileDirectory = getwd(),
                           pointsize = 12,
                           bg = "transparent",
                           canvas = "white",
                           units = "px",
                           dpi = 150) {
  # oldparCairo <- par(no.readonly = TRUE)
  # oldparCairo$new <- FALSE
  # on.exit(par(oldparCairo))
  # 
  
  
  
  
  if (is.null(type)) {
    return()
  }
  
  if (is.null(fileName)) {
    fileName <- "visstat_plot"
  }
  
  
  
  fullfilename <- paste(fileName, ".", type, sep = "")
  Cairofilename <- file.path(fileDirectory, fullfilename)
  
  if (type == "png") {
    CairoPNG(filename = Cairofilename, width = width, height = height, 
             pointsize = pointsize, bg = bg, dpi = dpi)
  } else if (type == "pdf") {
    CairoPDF(file = Cairofilename, width = width, height = height, 
             pointsize = pointsize, bg = bg)
  } else if (type == "jpeg") {
    CairoJPEG(filename = Cairofilename, width = width, height = height, 
              pointsize = pointsize, bg = bg, dpi = dpi)
  } else if (type == "tiff") {
    CairoTIFF(filename = Cairofilename, width = width, height = height, 
              pointsize = pointsize, bg = bg, dpi = dpi)
  } else if (type == "svg") {
    CairoSVG(file = Cairofilename, width = width, height = height, 
             pointsize = pointsize, bg = bg)
  } else if (type == "ps") {
    CairoPS(file = Cairofilename, width = width, height = height, 
            pointsize = pointsize, bg = bg, family = "Helvetica")
  } else {
    warning("Chosen output type not supported. No graphics saved.")
    return()
  }
}



#' Saves Graphical Output with plot capture capability
#'
#' Closes all graphical devices with \code{dev.off()} and saves the output only
#' if both \code{fileName} and \code{type} are provided. Enhanced version that
#' can capture plots before closing devices.
#'
#' @param fileName name of file to be created in directory \code{fileDirectory}
#'   without file extension '.\code{type}'.
#' @param type see \code{Cairo()}.
#' @param fileDirectory path of directory, where graphic is stored. Default
#'   setting current working directory.
#' @param oldfile old file of same name to be overwritten
#' @param capture_env Environment to store captured plots. If NULL, no capture occurs.
#'
#' @return NULL, if no \code{type} or \code{fileName} is provided, file path if graph
#'   is created
#' @examples
#'
#' # very simple KDE (adapted from example in Cairo())
#' openGraphCairo(type = "png", fileDirectory = tempdir())
#' plot(rnorm(4000), rnorm(4000), col = "#ff000018", pch = 19, cex = 2)
#' # save file 'norm.png' in directory specified in fileDirectory
#' saveGraphVisstat("norm", type = "png", fileDirectory = tempdir())
#' file.remove(file.path(tempdir(), "norm.png")) # remove file 'norm.png'
#'
#' @export
saveGraphVisstat <- function(fileName = NULL,
                             type = NULL,
                             fileDirectory = getwd(),
                             oldfile = NULL,
                             capture_env = NULL) {
  
  # logic to capture plots independent of type
  
  
  
  if (!is.null(capture_env)) {
    capture_env$captured_plots[[length(capture_env$captured_plots) + 1]] <- recordPlot()
  }
  
  if (is.null(fileName)) {
    return()
  } else if (is.null(type)) {
    return()
  } else if (is.null(oldfile)) {
    dummy_name <- "visstat_plot"
    oldPlotName <- paste(dummy_name, ".", type, sep = "")
    oldfile <- file.path(fileDirectory, oldPlotName)
  }
  
  while (!is.null(dev.list())) {
    dev.off()
  }
  
  file2 <- gsub("[^[:alnum:]]", "_", fileName)
  file3 <- gsub("_{2,}", "_", file2)
  
  newFileName <- paste0(file3, ".", type)
  Cairofile <- file.path(fileDirectory, newFileName)
  file.copy(oldfile, Cairofile, overwrite = T)
  
  if (file.exists(oldfile)) {
    file.remove(oldfile)
  }
  
  return(invisible(Cairofile))
}

library(testthat)
library(Cairo)

# Helper function for testing optional Cairo backends with detailed reporting
test_optional_backend <- function(type, test_filename, temp_dir) {
  cat("\n--- Testing", type, "backend ---\n")
  
  tryCatch({
    cat("Creating", type, "device...\n")
    openGraphCairo(fileName = test_filename, type = type, fileDirectory = temp_dir)
    
    cat("Plotting to", type, "device...\n")
    plot(1:3, 1:3, main = paste("Test", type))
    
    cat("Closing", type, "device...\n")
    dev.off()
    
    expected_file <- file.path(temp_dir, paste0(test_filename, ".", type))
    cat("Expected file:", expected_file, "\n")
    
    if(file.exists(expected_file)) {
      file_size <- file.size(expected_file)
      cat("SUCCESS:", type, "file created with size", file_size, "bytes\n")
      
      expect_true(file.exists(expected_file), 
                  info = paste("File should exist for type:", type))
      
      # Clean up
      file.remove(expected_file)
      cat("Cleaned up", type, "file\n")
      return(TRUE)
    } else {
      cat("FAILURE: File was not created for type:", type, "\n")
      return(FALSE)
    }
    
  }, error = function(e) {
    cat("ERROR with", type, "backend:", e$message, "\n")
    
    # Close any open devices before continuing
    if(length(dev.list()) > 0) {
      cat("Closing open graphics devices...\n")
      while(length(dev.list()) > 0) {
        dev.off()
      }
    }
    
    # Check if it's a specific Cairo backend error
    if(grepl("Failed to create Cairo backend", e$message)) {
      cat("DIAGNOSIS:", type, "Cairo backend is not available on this system\n")
    } else if(grepl("font", e$message, ignore.case = TRUE)) {
      cat("DIAGNOSIS:", type, "backend failed due to font issues\n")
    } else {
      cat("DIAGNOSIS: Unknown error with", type, "backend\n")
    }
    
    return(FALSE)
  })
}

# Test suite for opening_saving_Cairo.R functions
# Assumes the functions openGraphCairo() and saveGraphVisstat() are loaded

test_that("openGraphCairo handles NULL type correctly", {
  # Should return NULL (invisibly) when type is NULL
  result <- openGraphCairo(type = NULL)
  expect_null(result)
})

test_that("openGraphCairo handles missing fileName correctly", {
  # Should use default fileName when NULL
  temp_dir <- tempdir()
  
  # Test with PNG type
  expect_silent({
    openGraphCairo(fileName = NULL, type = "png", fileDirectory = temp_dir)
    plot(1:10, 1:10)
    dev.off()
  })
  
  # Check if default file was created
  default_file <- file.path(temp_dir, "visstat_plot.png")
  expect_true(file.exists(default_file))
  
  # Clean up
  if(file.exists(default_file)) file.remove(default_file)
})

test_that("openGraphCairo creates correct file paths", {
  temp_dir <- tempdir()
  test_filename <- "test_plot"
  
  # Test PNG creation
  expect_silent({
    openGraphCairo(fileName = test_filename, type = "png", fileDirectory = temp_dir)
    plot(1:5, 1:5)
    dev.off()
  })
  
  expected_file <- file.path(temp_dir, "test_plot.png")
  expect_true(file.exists(expected_file))
  
  # Clean up
  if(file.exists(expected_file)) file.remove(expected_file)
})

test_that("openGraphCairo supports all required file types", {
  temp_dir <- tempdir()
  test_filename <- "format_test"
  
  # Test types that are more likely to work across different systems
  stable_types <- c("png", "pdf", "jpeg")
  
  for(type in stable_types) {
    expect_silent({
      openGraphCairo(fileName = test_filename, type = type, fileDirectory = temp_dir)
      plot(1:3, 1:3, main = paste("Test", type))
      dev.off()
    })
    
    expected_file <- file.path(temp_dir, paste0(test_filename, ".", type))
    expect_true(file.exists(expected_file), 
                info = paste("File should exist for type:", type))
    
    # Clean up
    if(file.exists(expected_file)) file.remove(expected_file)
  }
  
  # Test additional types with detailed reporting
  optional_types <- c("tiff", "svg", "ps")
  
  cat("\n=== Testing Optional Cairo Backends ===\n")
  
  backend_results <- list()
  for(type in optional_types) {
    backend_results[[type]] <- test_optional_backend(type, test_filename, temp_dir)
  }
  
  # Summary report
  cat("\n=== Backend Test Summary ===\n")
  working_backends <- names(backend_results)[sapply(backend_results, isTRUE)]
  failing_backends <- names(backend_results)[sapply(backend_results, function(x) !isTRUE(x))]
  
  if(length(working_backends) > 0) {
    cat("WORKING backends:", paste(working_backends, collapse = ", "), "\n")
  }
  if(length(failing_backends) > 0) {
    cat("FAILING backends:", paste(failing_backends, collapse = ", "), "\n")
  }
  
  # At least document what we found
  expect_true(length(backend_results) == 3, 
              info = "Should test all three optional backends")
})

test_that("openGraphCairo handles unsupported file types", {
  temp_dir <- tempdir()
  
  # Should issue warning and return NULL for unsupported type
  expect_warning(
    result <- openGraphCairo(fileName = "test", type = "unsupported", fileDirectory = temp_dir),
    "Chosen output type not supported"
  )
  expect_null(result)
})

test_that("openGraphCairo uses correct default parameters", {
  temp_dir <- tempdir()
  
  # Test that function works with minimal parameters
  expect_silent({
    openGraphCairo(type = "png", fileDirectory = temp_dir)
    plot(1:5, 1:5)
    dev.off()
  })
  
  # Should create file with default name
  default_file <- file.path(temp_dir, "visstat_plot.png")
  expect_true(file.exists(default_file))
  
  # Clean up
  if(file.exists(default_file)) file.remove(default_file)
})

test_that("saveGraphVisstat handles NULL parameters correctly", {
  # Should return NULL when fileName is NULL
  result1 <- saveGraphVisstat(fileName = NULL, type = "png")
  expect_null(result1)
  
  # Should return NULL when type is NULL
  result2 <- saveGraphVisstat(fileName = "test", type = NULL)
  expect_null(result2)
})

test_that("saveGraphVisstat captures plots when capture_env is provided", {
  temp_dir <- tempdir()
  
  # Create capture environment
  capture_env <- new.env()
  capture_env$captured_plots <- list()
  
  # Create a plot and save with capture
  openGraphCairo(fileName = "capture_test", type = "png", fileDirectory = temp_dir)
  plot(1:5, 1:5, main = "Capture Test")
  
  result <- saveGraphVisstat(
    fileName = "capture_test", 
    type = "png", 
    fileDirectory = temp_dir,
    capture_env = capture_env
  )
  
  # Check that plot was captured
  expect_length(capture_env$captured_plots, 1)
  expect_s3_class(capture_env$captured_plots[[1]], "recordedplot")
  
  # Check that file was created
  expected_file <- file.path(temp_dir, "capture_test.png")
  expect_true(file.exists(expected_file))
  
  # Clean up
  if(file.exists(expected_file)) file.remove(expected_file)
})

test_that("saveGraphVisstat sanitizes file names correctly", {
  temp_dir <- tempdir()
  
  # Create plot first, then save with different name
  openGraphCairo(fileName = "temp_test", type = "png", fileDirectory = temp_dir)
  plot(1:3, 1:3)
  dev.off()
  
  # Now test the sanitization with saveGraphVisstat
  temp_old_file <- file.path(temp_dir, "temp_test.png")
  
  result <- saveGraphVisstat(
    fileName = "test@#$%plot with spaces!!", 
    type = "png", 
    fileDirectory = temp_dir,
    oldfile = temp_old_file
  )
  
  # Check what file actually gets created (the function does sanitization)
  expect_false(is.null(result))
  expect_true(file.exists(result))
  
  # The exact sanitization pattern might differ, so let's check the pattern
  basename_result <- basename(result)
  expect_match(basename_result, "^test.*\\.png$")
  
  # Clean up
  if(file.exists(result)) file.remove(result)
  if(file.exists(temp_old_file)) file.remove(temp_old_file)
})

test_that("saveGraphVisstat handles oldfile parameter correctly", {
  temp_dir <- tempdir()
  old_file <- file.path(temp_dir, "old_test.png")
  
  # Create an old file to copy from
  openGraphCairo(fileName = "old_test", type = "png", fileDirectory = temp_dir)
  plot(1:5, 1:5, main = "Old Plot")
  dev.off()
  
  # Use saveGraphVisstat with explicit oldfile
  result <- saveGraphVisstat(
    fileName = "new_test", 
    type = "png", 
    fileDirectory = temp_dir,
    oldfile = old_file
  )
  
  new_file <- file.path(temp_dir, "new_test.png")
  expect_true(file.exists(new_file))
  expect_false(file.exists(old_file)) # Old file should be removed
  expect_equal(result, new_file)
  
  # Clean up
  if(file.exists(new_file)) file.remove(new_file)
})

test_that("saveGraphVisstat closes all graphics devices", {
  temp_dir <- tempdir()
  
  # Open multiple devices
  openGraphCairo(fileName = "test1", type = "png", fileDirectory = temp_dir)
  plot(1:3, 1:3)
  
  openGraphCairo(fileName = "test2", type = "png", fileDirectory = temp_dir)
  plot(4:6, 4:6)
  
  # Should have active devices
  expect_true(length(dev.list()) > 0)
  
  # saveGraphVisstat should close all devices
  result <- saveGraphVisstat(
    fileName = "final_test", 
    type = "png", 
    fileDirectory = temp_dir
  )
  
  # All devices should be closed
  expect_null(dev.list())
  
  # Clean up any remaining files
  test_files <- c("test1.png", "test2.png", "final_test.png", "visstat_plot.png")
  for(file in test_files) {
    full_path <- file.path(temp_dir, file)
    if(file.exists(full_path)) file.remove(full_path)
  }
})

test_that("integration test: openGraphCairo and saveGraphVisstat work together", {
  temp_dir <- tempdir()
  
  # Complete workflow test - use PNG which is more reliable
  expect_silent({
    openGraphCairo(fileName = "integration", type = "png", fileDirectory = temp_dir)
    plot(1:10, 1:10, main = "Integration Test", col = "blue", pch = 19)
    lines(1:10, (1:10)^1.1, col = "red", lwd = 2)
    dev.off()
  })
  
  # Check that the intermediate file was created
  intermediate_file <- file.path(temp_dir, "integration.png")
  expect_true(file.exists(intermediate_file))
  
  result <- saveGraphVisstat(
    fileName = "integration_final", 
    type = "png", 
    fileDirectory = temp_dir,
    oldfile = intermediate_file
  )
  
  expected_file <- file.path(temp_dir, "integration_final.png")
  expect_true(file.exists(expected_file))
  expect_equal(result, expected_file)
  
  # Verify file has content (size > 0)
  expect_gt(file.size(expected_file), 0)
  
  # Clean up
  if(file.exists(expected_file)) file.remove(expected_file)
  if(file.exists(intermediate_file)) file.remove(intermediate_file)
})

test_that("error handling for invalid directories", {
  invalid_dir <- "/non/existent/path/that/should/not/exist"
  
  # Test if the system actually prevents writing to invalid directories
  # Some systems might create intermediate directories or handle this differently
  result <- tryCatch({
    openGraphCairo(fileName = "test", type = "png", fileDirectory = invalid_dir)
    "no_error"
  }, error = function(e) {
    "error_occurred"
  })
  
  # We expect either an error or a warning, but behavior may vary by system
  expect_true(result == "error_occurred" || result == "no_error")
  
  # If no error occurred, at least verify the function doesn't crash
  if(result == "no_error") {
    # Close any open devices to clean up
    while(length(dev.list()) > 0) {
      dev.off()
    }
  }
})

test_that("capture environment functionality works correctly", {
  temp_dir <- tempdir()
  
  # Test multiple plots with capture
  capture_env <- new.env()
  capture_env$captured_plots <- list()
  
  # First plot
  openGraphCairo(fileName = "multi1", type = "png", fileDirectory = temp_dir)
  plot(1:5, 1:5, main = "Plot 1")
  saveGraphVisstat(
    fileName = "multi1", 
    type = "png", 
    fileDirectory = temp_dir,
    capture_env = capture_env
  )
  
  # Second plot
  openGraphCairo(fileName = "multi2", type = "png", fileDirectory = temp_dir)
  plot(6:10, 6:10, main = "Plot 2")
  saveGraphVisstat(
    fileName = "multi2", 
    type = "png", 
    fileDirectory = temp_dir,
    capture_env = capture_env
  )
  
  # Should have captured both plots
  expect_length(capture_env$captured_plots, 2)
  expect_s3_class(capture_env$captured_plots[[1]], "recordedplot")
  expect_s3_class(capture_env$captured_plots[[2]], "recordedplot")
  
  # Clean up
  cleanup_files <- c("multi1.png", "multi2.png")
  for(file in cleanup_files) {
    full_path <- file.path(temp_dir, file)
    if(file.exists(full_path)) file.remove(full_path)
  }
})

# choose output directory
filedir <- tempdir()


graphicsoutput <- "png"
openGraphCairo(type = graphicsoutput, fileDirectory = filedir)
plot(rnorm(4000), rnorm(4000), col = "#ff000018", pch = 19, cex = 2)
saveGraphVisstat(
  "norm1",
  type = graphicsoutput, fileDirectory = filedir
)



graphicsoutput <- "pdf"
openGraphCairo(type = graphicsoutput, fileDirectory = filedir)
plot(rnorm(4000), rnorm(4000), col = "#ff000018", pch = 19, cex = 2)
saveGraphVisstat(
  "norm2",
  type = graphicsoutput, fileDirectory = filedir
)

graphicsoutput <- "ps"
openGraphCairo(type = graphicsoutput, fileDirectory = filedir)
plot(rnorm(4000), rnorm(4000), col = "#ff000018", pch = 19, cex = 2)
saveGraphVisstat(
  "norm3",
  type = graphicsoutput, fileDirectory = filedir
)

graphicsoutput <- "svg"
openGraphCairo(type = graphicsoutput, fileDirectory = filedir)
plot(rnorm(4000), rnorm(4000), col = "#ff000018", pch = 19, cex = 2)
saveGraphVisstat(
  "norm4",
  type = graphicsoutput, fileDirectory = filedir
)




# remove output  plots in filedir
graphicaltypes <- c(".png", ".pdf", ".svg", ".ps")
for (i in graphicaltypes) {
  plotname <- dir(filedir, pattern = i)
  print(file.path(filedir, plotname))
  file.remove(file.path(filedir, plotname))
}

# Header visstat -----
#'
#' Wrapper for visstat_core Allowing Three Different Input Styles
#'
#' @description \code{visstat()} is a wrapper around \code{\link{visstat_core}} 
#' that provides three alternative input styles: a formula interface, a 
#' standardised vector interface, and a backward-compatible data frame interface.
#' \code{\link{visstat_core}} defines the decision logic for statistical 
#' hypothesis testing and visualisation between two variables of class 
#' \code{"numeric"}, \code{"integer"}, or \code{"factor"}.
#'
#' @param x For the formula interface: a formula of the form \code{y ~ x}, 
#'   where \code{y} is the response variable and \code{x} is the predictor or 
#'   grouping variable (requires \code{data} argument). For the standardised 
#'   form: a vector of class \code{"numeric"}, \code{"integer"}, or 
#'   \code{"factor"} representing the predictor or grouping variable. For the 
#'   backward-compatible form: a \code{data.frame} containing the relevant 
#'   columns.
#' @param y For the formula interface: not used (variables are extracted from 
#'   the formula). For the standardised form: a vector of class \code{"numeric"}, 
#'   \code{"integer"}, or \code{"factor"} representing the response variable. 
#'   For the backward-compatible form: a \code{character} string specifying the 
#'   name of the response variable column in \code{x}.
#' @param ... For the backward-compatible form only: a \code{character} string 
#'   specifying the name of the predictor or grouping variable column in 
#'   \code{x}. Ignored for formula and standardised input styles.
#' @param data A \code{data.frame} containing the variables specified in the 
#'   formula. Required when using the formula interface. Ignored for other 
#'   input styles.
#' @param conf.level Confidence level for statistical inference; default is 
#'   \code{0.95}.
#' @param correlation Logical. If \code{FALSE} (default), performs simple
#'   linear regression analysis with confidence and prediction bands when both
#'   variables are numeric. If \code{TRUE}, performs Spearman correlation analysis
#'   with trend line only (no regression interpretation).
#' @param numbers Logical. Whether to annotate plots with numeric values.
#' @param minpercent Number between 0 and 1 indicating minimal fraction of 
#'   total count data of a category to be displayed in mosaic count plots.
#' @param graphicsoutput Saves plot(s) of type \code{"png"}, \code{"jpg"}, 
#'   \code{"tiff"} or \code{"bmp"} in directory specified in 
#'   \code{plotDirectory}. If \code{NULL}, no plots are saved.
#' @param plotName Graphical output is stored following the naming convention 
#'   \code{"plotName.graphicsoutput"} in \code{plotDirectory}. Without 
#'   specifying this parameter, \code{plotName} is automatically generated 
#'   following the convention \code{"statisticalTestName_varsample_varfactor"}.
#' @param plotDirectory Specifies directory where generated plots are stored. 
#'   Default is current working directory.
#'
#' @details This wrapper supports three input formats:
#' 
#' (1) Formula interface: \code{visstat(y ~ x, data = df)}, where the formula 
#' specifies the response (\code{y}) and predictor (\code{x}) variables, and 
#' \code{data} is a data frame containing these variables.
#'
#' (2) Standardised form: \code{visstat(x, y)}, where both \code{x} and \code{y} 
#' are vectors of class \code{"numeric"}, \code{"integer"}, or \code{"factor"}. 
#' Here \code{x} is the predictor or grouping variable and \code{y} is the 
#' response variable.
#'
#' (3) Backward-compatible form: \code{visstat(dataframe, "name_of_y", "name_of_x")}, 
#' where the first character string refers to the response variable and the 
#' second to the predictor or grouping variable. Both must be column names in 
#' \code{dataframe}.
#'
#' The interpretation of \code{x} and \code{y} depends on the variable classes. 
#' Throughout, data of class \code{numeric} or \code{integer} are referred to 
#' as numeric, while data of class \code{factor} are referred to as categorical:
#'
#' If one variable is numeric and the other a factor, the numeric vector is the 
#' response (\code{y}) and the factor is the grouping variable (\code{x}). This 
#' supports tests of central tendencies (e.g., t-test, Welch's ANOVA, Wilcoxon, 
#' Kruskal-Wallis).
#'
#' If both variables are numeric, a linear model is fitted with \code{y} as the 
#' response and \code{x} as the predictor.
#'
#' If both variables are factors, an association test (Chi-squared or Fisher's 
#' exact) is used. The test result is invariant to variable order, but 
#' visualisations (e.g., axis layout, bar orientation) depend on the roles of 
#' \code{x} and \code{y}.
#'
#' This wrapper standardises the input and calls \code{\link{visstat_core}}, 
#' which selects and executes the appropriate test with visual output and 
#' assumption diagnostics.
#'
#' @return An object of class \code{"visstat"} containing the results of 
#' the automatically selected statistical test. The specific contents depend on 
#' which test was performed. Additionally, the returned object includes two 
#' attributes:
#' \itemize{
#'   \item \code{plot_paths}: Character vector of file paths where plots were 
#'     saved (if \code{graphicsoutput} was specified)
#'   \item \code{captured_plots}: List of captured plot objects for programmatic 
#'     access
#' }
#'
#' In case of insufficient data, returns a list with an \code{error} element and 
#' basic input summary information.
#'
#' @note For best visualization, ensure that the RStudio Plots pane is adequately 
#' sized. If you get "figure margins too large" errors, try expanding the Plots 
#' pane in RStudio, using \code{dev.new(width=10, height=6)} for a larger plot 
#' window, or reducing the \code{cex} parameter.
#'
#' @seealso \code{\link{visstat_core}} defining the decision logic, the 
#' package's vignette \code{vignette("visStatistics")} explaining the decision 
#' logic accompanied by illustrative examples, and the accompanying webpage 
#' \url{https://shhschilling.github.io/visStatistics/}.
#'
#' @examples
#' # Formula interface
#' mtcars$am <- as.factor(mtcars$am)
#' visstat(mpg ~ am, data = mtcars)
#'
#' # Standardised usage
#' visstat(mtcars$am, mtcars$mpg)
#'
#' # Backward-compatible usage (same result as above)
#' visstat(mtcars, "mpg", "am")
#'
#' ## Student's t-test (equal variances, two groups)
#' # When residuals are normally distributed and Levene's test indicates
#' # homoscedasticity, the classic Student's t-test with pooled variance is used
#' df <- droplevels(subset(PlantGrowth, group %in% c("ctrl", "trt1")))
#' visstat(df$group,df$weight)
#'
#' ## Welch's t-test (unequal variances, two groups)
#' # When residuals are normally distributed but Levene's test indicates
#' # heteroscedasticity, Welch's t-test is used
#' visstat(mtcars$am, mtcars$mpg)
#'
#' ## Wilcoxon rank sum test (non-normal, two groups)
#' # When residuals are not normally distributed
#' grades_gender <- data.frame(
#'   Sex = as.factor(c(rep("Girl", 20), rep("Boy", 20))),
#'   Grade = c(
#'     19.3, 18.1, 15.2, 18.3, 7.9, 6.2, 19.4, 20.3, 9.3, 11.3,
#'     18.2, 17.5, 10.2, 20.1, 13.3, 17.2, 15.1, 16.2, 17.3, 16.5,
#'     5.1, 15.3, 17.1, 14.8, 15.4, 14.4, 7.5, 15.5, 6.0, 17.4,
#'     7.3, 14.3, 13.5, 8.0, 19.5, 13.4, 17.9, 17.7, 16.4, 15.6
#'   )
#' )
#' visstat(grades_gender$Sex, grades_gender$Grade)
#'
#' ## Fisher's ANOVA (equal variances, >2 groups)
#' # When residuals are normally distributed and Levene's test indicates
#' # homoscedasticity, classic Fisher's ANOVA with TukeyHSD post-hoc is used.
#' # Different green letters indicate significant differences between groups. 
#' visstat(PlantGrowth$group, PlantGrowth$weight)
#'
#' ## Welch's one-way ANOVA (unequal variances, >2 groups)
#' set.seed(123)
#' values <- c(rnorm(20, 10, 1),rnorm(20, 15, 5),rnorm(20, 12, 2))
#' groups <- factor(rep(c("A", "B", "C"), each = 20))
#' visstat(groups, values)

#' ## Kruskal-Wallis (non-normal, >2 groups)
#' # When residuals are not normally distributed, kruskal.test() is followed by 
#' # pairwise.wilcox.test. 
#' visstat(iris$Species, iris$Petal.Width)
#'
#' ## Simple linear regression (both numeric)
#' visstat(trees$Height, trees$Girth, conf.level = 0.99)
#'
#' ## Pearson's Chi-squared test (both factors, large expected counts)
#' HairEyeColorDataFrame <- counts_to_cases(as.data.frame(HairEyeColor))
#' visstat(HairEyeColorDataFrame$Eye, HairEyeColorDataFrame$Hair)
#'
#' ## Fisher's exact test (both factors, small expected counts)
#' HairEyeColorMaleFisher <- HairEyeColor[, , 1]
#' blackBrownHazelGreen <- HairEyeColorMaleFisher[1:2, 3:4]
#' blackBrownHazelGreen <- counts_to_cases(as.data.frame(blackBrownHazelGreen))
#' visstat(blackBrownHazelGreen$Eye, blackBrownHazelGreen$Hair)
#'
#' ## Save PNG
#' visstat(blackBrownHazelGreen$Hair, blackBrownHazelGreen$Eye,
#'         graphicsoutput = "png", plotDirectory = tempdir())
#'
#' ## Custom plot name
#' visstat(iris$Species, iris$Petal.Width,
#'         graphicsoutput = "pdf", plotName = "kruskal_iris", plotDirectory = tempdir())
#'
#' @export
visstat <- function(x,
                    y,
                    ...,
                    data = NULL,
                    conf.level = 0.95,
                    correlation = FALSE,
                    numbers = TRUE,
                    minpercent = 0.05,
                    graphicsoutput = NULL,
                    plotName = NULL,
                    plotDirectory = getwd()) {
  # store default graphical parameters------
  oldparvisstat <- par(no.readonly = TRUE)
  on.exit(par(oldparvisstat))
  
  check_visstat_input(x, y, ..., data = data)
  
  clean_name <- function(expr) {
    sub(".*\\$", "", deparse(expr))
  }
  
  # Case 0: formula interface
  if (inherits(x, "formula")) {
    if (is.null(data)) {
      stop("Formula input requires a 'data' argument.")
    }
    
    vars <- all.vars(x)
    if (length(vars) != 2) {
      stop("Formula must be of the form 'y ~ x'.")
    }
    
    yvar <- vars[1]
    xvar <- vars[2]
    
    return(visstat_core(
      dataframe = data,
      varsample = yvar,
      varfactor = xvar,
      conf.level = conf.level,
      correlation = correlation,
      numbers = numbers,
      minpercent = minpercent,
      graphicsoutput = graphicsoutput,
      plotName = plotName,
      plotDirectory = plotDirectory
    ))
  }
  
  # Case 1: legacy form: visstat(data, "Girth", "Height")
  if (is.data.frame(x) && is.character(y) && length(y) == 1) {
    mc <- match.call()
    args <- as.list(mc)[-1]
    if (length(args) < 3 || !is.character(eval(args[[3]], parent.frame()))) {
      stop("When using the backward-compatible form, provide two column names as character strings.")
    }
    varsample <- y                                    # first string: response
    varfactor <- eval(args[[3]], parent.frame())      # second string: predictor
    dataframe <- x
    return(visstat_core(
      dataframe = dataframe,
      varsample = varsample,
      varfactor = varfactor,
      conf.level = conf.level,
      correlation = correlation,
      numbers = numbers,
      minpercent = minpercent,
      graphicsoutput = graphicsoutput,
      plotName = plotName,
      plotDirectory = plotDirectory
    ))
  }
  
  # Case 2: new syntax: visstat(trees$Height, trees$Girth)
  factor_expr <- substitute(x)  # first argument = predictor/grouping variable
  sample_expr <- substitute(y)  # second argument = response
  
  factor_val <- eval(factor_expr, parent.frame())
  sample_val <- eval(sample_expr, parent.frame())
  
  factor_name <- clean_name(factor_expr)
  sample_name <- clean_name(sample_expr)
  
  dataframe <- data.frame(factor_val, sample_val)
  names(dataframe) <- c(factor_name, sample_name)
  
  return(invisible(visstat_core(
    dataframe = dataframe,
    varsample = sample_name,   # second argument = response
    varfactor = factor_name,   # first argument = predictor
    conf.level = conf.level,
    correlation = correlation,
    numbers = numbers,
    minpercent = minpercent,
    graphicsoutput = graphicsoutput,
    plotName = plotName,
    plotDirectory = plotDirectory
  )))
}

library(testthat)

# Test suite for visstat.R wrapper function
# Tests both standardised and backward-compatible input forms

# Helper function to create test datasets
create_visstat_test_data <- function() {
  set.seed(42)
  list(
    # Numeric data for regression
    numeric_x = rnorm(30, mean = 10, sd = 2),
    numeric_y = rnorm(30, mean = 15, sd = 3),
    
    # Factor data for group comparisons
    factor_3_levels = factor(rep(c("A", "B", "C"), each = 10)),
    factor_2_levels = factor(rep(c("Group1", "Group2"), each = 15)),
    
    # Categorical data for chi-square tests
    categorical_x = factor(sample(c("Cat1", "Cat2", "Cat3"), 50, replace = TRUE)),
    categorical_y = factor(sample(c("Type1", "Type2"), 50, replace = TRUE)),
    
    # Data frame for backward compatibility tests
    test_df = data.frame(
      height = rnorm(40, 170, 10),
      weight = rnorm(40, 70, 15),
      group = factor(rep(c("A", "B", "C", "D"), each = 10)),
      category = factor(sample(c("X", "Y"), 40, replace = TRUE)),
      stringsAsFactors = FALSE
    )
  )
}

# Helper functions for graphics device management
setup_test_graphics <- function() {
  if (length(dev.list()) > 0) {
    dev.off()
  }
  pdf(NULL)
}

cleanup_test_graphics <- function() {
  if (length(dev.list()) > 0) {
    dev.off()
  }
}

# Tests for input validation and parameter handling
test_that("visstat handles basic parameter validation", {
  data <- create_visstat_test_data()
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Test with valid inputs - should not error
  expect_no_error(
    visstat(data$factor_2_levels, data$numeric_x, conf.level = 0.95)
  )
  
  # Test with invalid confidence level 
  expect_error(
    visstat(data$factor_2_levels, data$numeric_x, conf.level = 1.5),
    "conf.level"
  )
  
  # For this one, it seems the validation happens in the underlying function
  expect_warning(
    visstat(data$factor_2_levels, data$numeric_x, conf.level = -0.1),
    "conf.level"
  )
})

# Tests for standardised syntax: visstat(x, y)
test_that("visstat standardised syntax works with factor and numeric", {
  data <- create_visstat_test_data()
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Factor (x) and numeric (y) - should trigger group comparison tests
  result <- visstat(data$factor_2_levels, data$numeric_x)
  
  expect_type(result, "list")
  expect_s3_class(result, "visstat")
  expect_true(length(result) > 0)
})

test_that("visstat standardised syntax works with numeric regression", {
  data <- create_visstat_test_data()
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Numeric (x) and numeric (y) - should trigger regression
  result <- visstat(data$numeric_x, data$numeric_y)
  
  expect_type(result, "list")
  expect_s3_class(result, "visstat")
  expect_true(length(result) > 0)
})

test_that("visstat standardised syntax works with categorical data", {
  data <- create_visstat_test_data()
  
  # Factor (x) and factor (y) - should trigger chi-square/Fisher test
  simple_cat_x <- factor(rep(c("A", "B"), each = 25))
  simple_cat_y <- factor(rep(c("X", "Y"), 25))
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  result <- visstat(simple_cat_x, simple_cat_y)
  
  expect_type(result, "list")
  expect_s3_class(result, "visstat")
  expect_true(length(result) > 0)
})

# Tests for backward-compatible syntax: visstat(dataframe, "col1", "col2")
test_that("visstat backward-compatible syntax works correctly", {
  data <- create_visstat_test_data()
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Test with data frame and column names
  result <- visstat(data$test_df, "height", "group")
  
  expect_type(result, "list")
  expect_s3_class(result, "visstat")
  expect_true(length(result) > 0)
})

test_that("visstat backward-compatible syntax handles numeric columns", {
  data <- create_visstat_test_data()
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Regression with backward-compatible syntax
  result <- visstat(data$test_df, "weight", "height")
  
  expect_type(result, "list")
  expect_s3_class(result, "visstat")
  expect_true(length(result) > 0)
})

test_that("visstat backward-compatible syntax handles categorical columns", {
  # Create a more balanced dataset to avoid mosaic plot issues
  set.seed(123)
  balanced_df <- data.frame(
    cat1 = factor(rep(c("A", "B"), 30)),
    cat2 = factor(sample(c("X", "Y"), 60, replace = TRUE, prob = c(0.6, 0.4)))
  )
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Suppress warnings about residuals being zero (expected for some balanced datasets)
  result <- suppressWarnings(visstat(balanced_df, "cat1", "cat2"))
  
  expect_type(result, "list")
  expect_s3_class(result, "visstat")
  expect_true(length(result) > 0)
})

# Tests for error handling in backward-compatible syntax
test_that("visstat backward-compatible syntax validates inputs correctly", {
  data <- create_visstat_test_data()
  
  # These tests check input validation, so we don't need full graphics setup
  if (length(dev.list()) > 0) dev.off()
  
  # Missing second column name - should trigger check_visstat_input error
  expect_error(
    visstat(data$test_df, "height"),
    "Invalid input"
  )
  
  # Non-character column name - should trigger check_visstat_input error
  expect_error(
    suppressWarnings(visstat(data$test_df, "height", 123)),
    "Invalid input"
  )
  
  # Invalid column names - should error when trying to access non-existent columns
  expect_error(
    suppressWarnings(visstat(data$test_df, "nonexistent_col", "height"))
  )
})

# Tests for optional parameters
test_that("visstat handles optional parameters correctly", {
  data <- create_visstat_test_data()
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Test with different confidence levels
  result_95 <- visstat(data$factor_2_levels, data$numeric_x, conf.level = 0.95)
  result_99 <- visstat(data$factor_2_levels, data$numeric_x, conf.level = 0.99)
  
  expect_s3_class(result_95, "visstat")
  expect_s3_class(result_99, "visstat")
  expect_false(identical(result_95, result_99))
  
  # Test with numbers parameter
  result_numbers <- visstat(data$categorical_x, data$categorical_y, numbers = TRUE)
  result_no_numbers <- visstat(data$categorical_x, data$categorical_y, numbers = FALSE)
  
  expect_s3_class(result_numbers, "visstat")
  expect_s3_class(result_no_numbers, "visstat")
  
  # Test with minpercent parameter
  result_minperc <- visstat(data$categorical_x, data$categorical_y, minpercent = 0.1)
  expect_s3_class(result_minperc, "visstat")
})

# Tests for graphics output parameters
test_that("visstat handles graphics output parameters", {
  data <- create_visstat_test_data()
  temp_dir <- tempdir()
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Test with graphics output - suppress par warnings
  result <- suppressWarnings(visstat(
    data$factor_2_levels, 
    data$numeric_x,
    graphicsoutput = "png",
    plotDirectory = temp_dir
  ))
  
  expect_s3_class(result, "visstat")
  
  # Test with custom plot name
  result_custom <- suppressWarnings(visstat(
    data$factor_2_levels, 
    data$numeric_x,
    graphicsoutput = "png",
    plotName = "custom_test",
    plotDirectory = temp_dir
  ))
  
  expect_s3_class(result_custom, "visstat")
  
  # Test that plot method works with saved graphics
  expect_no_error(plot(result))
  expect_no_error(plot(result_custom))
})

# Tests for S3 methods integration
test_that("visstat objects work with S3 methods", {
  data <- create_visstat_test_data()
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Create a visstat object
  result <- visstat(data$factor_2_levels, data$numeric_x)
  
  # Test that S3 methods work
  expect_no_error(print(result))
  expect_no_error(summary(result))
  expect_no_error(plot(result))
  
  # Test that print returns invisibly
  print_result <- print(result)
  expect_identical(print_result, result)
  
  # Test that summary returns invisibly
  summary_result <- summary(result)
  expect_identical(summary_result, result)
  
  # Test that plot returns invisibly
  plot_result <- plot(result)
  expect_identical(plot_result, result)
})

test_that("visstat S3 methods produce expected output", {
  data <- create_visstat_test_data()
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Create a visstat object
  result <- visstat(data$factor_2_levels, data$numeric_x)
  
  # Test print output
  print_output <- capture.output(print(result))
  expect_true(any(grepl("Object of class 'visstat'", print_output)))
  expect_true(any(grepl("Available components:", print_output)))
  
  # Test summary output
  summary_output <- capture.output(summary(result))
  expect_true(any(grepl("Summary of visstat object", summary_output)))
  expect_true(any(grepl("--- Named components ---", summary_output)))
  expect_true(any(grepl("--- Contents ---", summary_output)))
})

# Tests for name extraction and handling
test_that("visstat extracts variable names correctly", {
  # Create named vectors to test name extraction
  height_data <- rnorm(20, 170, 10)
  group_data <- factor(rep(c("A", "B"), each = 10))
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # The function should extract names from the expressions
  result <- visstat(group_data, height_data)
  
  expect_s3_class(result, "visstat")
  # Note: Testing exact names is tricky due to expression handling
  # The important thing is that it works without error
})

# Tests for real-world data scenarios
test_that("visstat works with built-in datasets", {
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Test with mtcars (numeric vs factor)
  mtcars$am <- as.factor(mtcars$am)
  result_mtcars <- visstat(mtcars$am, mtcars$mpg)
  expect_s3_class(result_mtcars, "visstat")
  
  # Test with iris (factor vs numeric)
  result_iris <- visstat(iris$Species, iris$Petal.Width)
  expect_s3_class(result_iris, "visstat")
  
  # Test with regression
  result_trees <- visstat(trees$Height, trees$Girth)
  expect_s3_class(result_trees, "visstat")
})

# Tests for edge cases and data types
test_that("visstat handles edge cases correctly", {
  # Test with integer data
  int_data <- as.integer(c(1, 2, 3, 4, 5, 1, 2, 3, 4, 5))
  factor_data <- factor(rep(c("A", "B"), each = 5))
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  result <- visstat(factor_data, int_data)
  expect_s3_class(result, "visstat")
  
  # Test with small datasets - expect warnings but not errors
  small_numeric <- c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
  small_factor <- factor(rep(c("A", "B"), each = 5))
  
  # Should work, might trigger warnings about sample size
  expect_no_error(suppressWarnings(visstat(small_factor, small_numeric)))
})

# Tests for parameter combinations
test_that("visstat handles multiple parameter combinations", {
  data <- create_visstat_test_data()
  temp_dir <- tempdir()
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Complex parameter combination - suppress par warnings
  result <- suppressWarnings(visstat(
    data$factor_3_levels,
    data$numeric_x,
    conf.level = 0.99,
    numbers = FALSE,
    minpercent = 0.02,
    graphicsoutput = "png",
    plotName = "complex_test",
    plotDirectory = temp_dir
  ))
  
  expect_s3_class(result, "visstat")
  expect_type(result, "list")
})

# Tests for graphics parameter restoration
test_that("visstat restores graphics parameters correctly", {
  data <- create_visstat_test_data()
  
  # Store original parameters
  original_par <- par(no.readonly = TRUE)
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Run visstat
  result <- visstat(data$factor_2_levels, data$numeric_x)
  
  # Check that parameters are restored
  current_par <- par(no.readonly = TRUE)
  
  # Key parameters should be restored
  expect_equal(current_par$mfrow, original_par$mfrow)
  expect_equal(current_par$oma, original_par$oma)
})

# Tests for function return behavior
test_that("visstat returns results correctly", {
  data <- create_visstat_test_data()
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Function should return invisibly for new syntax
  result <- visstat(data$factor_2_levels, data$numeric_x)
  
  # Should be a visstat object
  expect_s3_class(result, "visstat")
  expect_type(result, "list")
  
  # Should have some components (don't assume specific structure)
  expect_true(length(result) > 0)
})

# Tests for input type validation
test_that("visstat validates input types appropriately", {
  # Invalid input types should be caught by check_visstat_input
  expect_error(visstat(NULL, NULL), "Invalid input")
  
  # Test with clearly invalid data types
  suppressWarnings(expect_error(visstat(list(1, 2, 3), factor(c("A", "B"))), "Invalid input"))
  
  # Test with mixed invalid types that should definitely fail
  suppressWarnings(expect_error(visstat(function() {}, matrix(1:4, nrow = 2)), "Invalid input"))
})

# Integration tests with different test scenarios
test_that("visstat integrates correctly with different statistical tests", {
  # Test that different combinations trigger appropriate tests
  data <- create_visstat_test_data()
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Two-sample test (factor + numeric)
  result_two_sample <- visstat(data$factor_2_levels, data$numeric_x)
  expect_s3_class(result_two_sample, "visstat")
  
  # ANOVA/Kruskal-Wallis (factor + numeric, multiple groups)
  result_anova <- visstat(data$factor_3_levels, data$numeric_x)
  expect_s3_class(result_anova, "visstat")
  
  # Chi-square/Fisher (factor + factor)
  result_chisq <- visstat(data$categorical_x, data$categorical_y)
  expect_s3_class(result_chisq, "visstat")
  
  # Regression (numeric + numeric)
  result_regression <- visstat(data$numeric_x, data$numeric_y)
  expect_s3_class(result_regression, "visstat")
})

# Tests for consistency between syntax forms
test_that("visstat produces consistent results across syntax forms", {
  data <- create_visstat_test_data()
  
  setup_test_graphics()
  on.exit(cleanup_test_graphics())
  
  # Create a test data frame
  test_df <- data.frame(
    x_var = data$factor_2_levels,
    y_var = data$numeric_x
  )
  
  # Test both syntax forms
  result_standard <- visstat(test_df$x_var, test_df$y_var)
  result_legacy <- visstat(test_df, "y_var", "x_var")
  
  # Both should be visstat objects
  expect_s3_class(result_standard, "visstat")
  expect_s3_class(result_legacy, "visstat")
  
  # Should have similar structure (exact equality might vary due to names)
  expect_equal(length(result_standard), length(result_legacy))
})

# Comprehensive test series using vis_anova_assumptions() to directly test 
# Levene vs Bartlett test results on heteroscedastic data
# Save as: test-visstat-heteroscedastic.R

library(testthat)

# Helper function to create test datasets with different variance patterns
create_heteroscedastic_dataset <- function(scenario_name, 
                                         n_groups = 3, 
                                         n_per_group = 30,
                                         group_names = NULL,
                                         seed = 123) {
  set.seed(seed)
  
  if (is.null(group_names)) {
    group_names <- paste0("Group_", LETTERS[1:n_groups])
  }
  
  scenarios <- list(
    "equal_variances" = list(
      means = rep(0, n_groups),
      sds = rep(1, n_groups),
      description = "Equal variances (homoscedastic)"
    ),
    "mild_heteroscedastic" = list(
      means = rep(0, n_groups),
      sds = c(1, 1.5, 2),
      description = "Mild heteroscedasticity"
    ),
    "moderate_heteroscedastic" = list(
      means = rep(0, n_groups),
      sds = c(0.5, 1, 2.5),
      description = "Moderate heteroscedasticity"
    ),
    "severe_heteroscedastic" = list(
      means = rep(0, n_groups),
      sds = c(0.2, 1, 5),
      description = "Severe heteroscedasticity"
    ),
    "extreme_heteroscedastic" = list(
      means = rep(0, n_groups),
      sds = c(0.1, 1, 10),
      description = "Extreme heteroscedasticity"
    ),
    "two_group_different" = list(
      means = c(0, 0),
      sds = c(1, 4),
      description = "Two groups with different variances"
    ),
    "skewed_heteroscedastic" = list(
      means = c(0, 2, 5),
      sds = c(1, 2, 4),
      description = "Different means AND variances"
    )
  )
  
  if (!scenario_name %in% names(scenarios)) {
    stop("Unknown scenario. Available: ", paste(names(scenarios), collapse = ", "))
  }
  
  scenario <- scenarios[[scenario_name]]
  n_groups_actual <- length(scenario$sds)
  
  data_list <- list()
  for (i in 1:n_groups_actual) {
    data_list[[i]] <- rnorm(n_per_group, mean = scenario$means[i], sd = scenario$sds[i])
  }
  
  y <- unlist(data_list)
  g <- factor(rep(group_names[1:n_groups_actual], each = n_per_group))
  
  list(y = y, g = g, scenario = scenario_name, description = scenario$description,
       true_sds = scenario$sds, true_means = scenario$means)
}

# Helper function to create non-normal heteroscedastic data
create_nonnormal_heteroscedastic_dataset <- function(distributions = c("normal", "exponential", "uniform"),
                                                   n_per_group = 30,
                                                   scale_factors = c(1, 2, 0.5),
                                                   seed = 456) {
  set.seed(seed)
  
  n_groups <- length(distributions)
  data_list <- list()
  group_names <- paste0("Dist_", 1:n_groups)
  
  for (i in 1:n_groups) {
    if (distributions[i] == "normal") {
      data_list[[i]] <- rnorm(n_per_group, mean = 0, sd = scale_factors[i])
    } else if (distributions[i] == "exponential") {
      data_list[[i]] <- rexp(n_per_group, rate = 1/scale_factors[i])
    } else if (distributions[i] == "uniform") {
      data_list[[i]] <- runif(n_per_group, min = 0, max = scale_factors[i] * 2)
    } else if (distributions[i] == "gamma") {
      data_list[[i]] <- rgamma(n_per_group, shape = 2, scale = scale_factors[i])
    }
  }
  
  y <- unlist(data_list)
  g <- factor(rep(group_names, each = n_per_group))
  
  list(y = y, g = g, distributions = distributions, scale_factors = scale_factors)
}

test_that("visstat: homoscedastic vs heteroscedastic comparison", {
  # Create clearly homoscedastic data
  set.seed(1500)
  homo_y <- c(rnorm(50, mean = 0, sd = 1), rnorm(50, mean = 2, sd = 1), rnorm(50, mean = 4, sd = 1))
  homo_g <- factor(rep(1:3, each = 50))
  homo_df <- data.frame(response = homo_y, group = homo_g)
  
  # Create clearly heteroscedastic data  
  set.seed(1501)
  hetero_y <- c(rnorm(50, mean = 0, sd = 0.5), rnorm(50, mean = 0, sd = 2), rnorm(50, mean = 0, sd = 5))
  hetero_g <- factor(rep(1:3, each = 50))
  hetero_df <- data.frame(response = hetero_y, group = hetero_g)
  
  # Test homoscedastic data with visstat - shows assumption plots + analysis
  homo_result <- visstat(homo_df, "response", "group")
  
  # Test heteroscedastic data with visstat - shows assumption plots + analysis
  hetero_result <- visstat(hetero_df, "response", "group")
  
  # Both should run successfully
  expect_s3_class(homo_result, "visstat")
  expect_s3_class(hetero_result, "visstat")
  
  # Both should generate plots (assumption checks + main analysis)
  expect_true(length(attr(homo_result, "captured_plots")) > 0)
  expect_true(length(attr(hetero_result, "captured_plots")) > 0)
  
  # Both should return meaningful results
  expect_true(length(homo_result) > 0)
  expect_true(length(hetero_result) > 0)
})

test_that("visstat detects mild heteroscedasticity", {
  mild_hetero_data <- create_heteroscedastic_dataset("mild_heteroscedastic", seed = 200)
  df <- data.frame(response = mild_hetero_data$y, group = mild_hetero_data$g)
  
  # Shows assumption plots + data analysis
  result <- visstat(df, "response", "group")
  
  expect_s3_class(result, "visstat")
  expect_true(length(result) > 0)
  expect_true(length(attr(result, "captured_plots")) > 0)
})

test_that("visstat detects severe heteroscedasticity", {
  severe_hetero_data <- create_heteroscedastic_dataset("severe_heteroscedastic", 
                                                      n_per_group = 25, seed = 300)
  df <- data.frame(response = severe_hetero_data$y, group = severe_hetero_data$g)
  
  # Shows dramatic variance differences in both assumption plots and main analysis
  result <- visstat(df, "response", "group")
  
  expect_s3_class(result, "visstat")
  expect_true(length(result) > 0)
  expect_true(length(attr(result, "captured_plots")) > 0)
  expect_no_error(print(result))
})

test_that("visstat with extreme heteroscedasticity", {
  extreme_data <- create_heteroscedastic_dataset("extreme_heteroscedastic", 
                                                n_per_group = 40, seed = 500)
  df <- data.frame(response = extreme_data$y, group = extreme_data$g)
  
  # Shows extreme variance differences visually + statistical analysis
  result <- visstat(df, "response", "group")
  
  expect_s3_class(result, "visstat")
  expect_true(length(result) > 0)
  expect_true(length(attr(result, "captured_plots")) > 0)
  expect_no_error(print(result))
})

test_that("visstat: Levene robustness with non-normal data", {
  nonnormal_data <- create_nonnormal_heteroscedastic_dataset(
    distributions = c("exponential", "uniform", "gamma"),
    n_per_group = 35,
    scale_factors = c(1, 3, 0.5),
    seed = 600
  )
  df <- data.frame(response = nonnormal_data$y, group = nonnormal_data$g)
  
  # Shows non-normal distributions + how Levene handles them
  result <- visstat(df, "response", "group")
  
  expect_s3_class(result, "visstat")
  expect_true(length(result) > 0)
  expect_true(length(attr(result, "captured_plots")) > 0)
  expect_no_error(print(result))
})

test_that("visstat: Power comparison across effect sizes", {
  # Small, medium, large variance differences - all with visual output
  small_data <- create_heteroscedastic_dataset("mild_heteroscedastic", n_per_group = 30, seed = 400)
  medium_data <- create_heteroscedastic_dataset("moderate_heteroscedastic", n_per_group = 30, seed = 401) 
  large_data <- create_heteroscedastic_dataset("severe_heteroscedastic", n_per_group = 30, seed = 402)
  
  small_df <- data.frame(response = small_data$y, group = small_data$g)
  medium_df <- data.frame(response = medium_data$y, group = medium_data$g)
  large_df <- data.frame(response = large_data$y, group = large_data$g)
  
  # Visual comparison of different effect sizes
  small_result <- visstat(small_df, "response", "group")
  medium_result <- visstat(medium_df, "response", "group")
  large_result <- visstat(large_df, "response", "group")
  
  expect_s3_class(small_result, "visstat")
  expect_s3_class(medium_result, "visstat")
  expect_s3_class(large_result, "visstat")
  
  expect_true(length(attr(small_result, "captured_plots")) > 0)
  expect_true(length(attr(medium_result, "captured_plots")) > 0)
  expect_true(length(attr(large_result, "captured_plots")) > 0)
})

test_that("visstat: Two-group heteroscedastic comparison", {
  two_group_data <- create_heteroscedastic_dataset("two_group_different", 
                                                  n_per_group = 30, seed = 700)
  df <- data.frame(response = two_group_data$y, group = two_group_data$g)
  
  # Shows two-group variance differences visually + t-test analysis
  result <- visstat(df, "response", "group")
  
  expect_s3_class(result, "visstat")
  expect_true(length(result) > 0)
  expect_true(length(attr(result, "captured_plots")) > 0)
})

test_that("visstat: Sample size effects on heteroscedasticity detection", {
  # Test different sample sizes with same variance pattern
  variance_pattern <- c(1, 2, 4)  # Moderate heteroscedasticity
  
  # Small sample
  set.seed(800)
  small_y <- unlist(lapply(1:3, function(i) rnorm(10, mean = 0, sd = sqrt(variance_pattern[i]))))
  small_g <- factor(rep(1:3, each = 10))
  small_df <- data.frame(response = small_y, group = small_g)
  
  # Large sample  
  set.seed(801)
  large_y <- unlist(lapply(1:3, function(i) rnorm(50, mean = 0, sd = sqrt(variance_pattern[i]))))
  large_g <- factor(rep(1:3, each = 50))
  large_df <- data.frame(response = large_y, group = large_g)
  
  # Visual comparison of sample size effects
  small_result <- visstat(small_df, "response", "group")
  large_result <- visstat(large_df, "response", "group")
  
  expect_s3_class(small_result, "visstat")
  expect_s3_class(large_result, "visstat")
  
  expect_true(length(attr(small_result, "captured_plots")) > 0)
  expect_true(length(attr(large_result, "captured_plots")) > 0)
})

test_that("visstat: Different confidence levels with heteroscedastic data", {
  test_data <- create_heteroscedastic_dataset("moderate_heteroscedastic", seed = 900)
  df <- data.frame(response = test_data$y, group = test_data$g)
  
  # Test different confidence levels - all show same plots but different interpretations
  conf_levels <- c(0.90, 0.95, 0.99)
  
  for (conf_level in conf_levels) {
    result <- visstat(df, "response", "group", conf.level = conf_level)
    
    expect_s3_class(result, "visstat")
    expect_true(length(result) > 0)
    expect_true(length(attr(result, "captured_plots")) > 0)
  }
})

test_that("visstat: Comprehensive Levene vs Bartlett visual demonstration", {
  comparison_data <- create_heteroscedastic_dataset("moderate_heteroscedastic", 
                                                   n_per_group = 50, seed = 1000)
  df <- data.frame(response = comparison_data$y, group = comparison_data$g)
  
  # Complete visual workflow: assumption plots + statistical analysis
  result <- visstat(df, "response", "group")
  
  expect_s3_class(result, "visstat")
  expect_true(length(result) > 0)
  expect_true(length(attr(result, "captured_plots")) > 0)
  expect_no_error(print(result))
  
  # The visual output shows both Levene and Bartlett results in assumption plots
  # Plus the chosen statistical test based on assumption violations
})

test_that("visstat integration with vis_anova_assumptions", {
  test_data <- create_heteroscedastic_dataset("severe_heteroscedastic", seed = 1100)
  
  df <- data.frame(response = test_data$y, group = test_data$g)
  
  result <- visstat(df, "response", "group")
  
  expect_s3_class(result, "visstat")
  expect_true(length(result) > 0)
  
  captured_plots <- attr(result, "captured_plots")
  expect_true(length(captured_plots) > 0)
  
  expect_no_error(print(result))
})

test_that("visstat comprehensive comparison: Levene vs Bartlett integration", {
  comparison_data <- create_heteroscedastic_dataset("moderate_heteroscedastic", 
                                                   n_per_group = 50, seed = 1200)
  
  df <- data.frame(response = comparison_data$y, group = comparison_data$g)
  
  result <- visstat(df, "response", "group")
  
  expect_s3_class(result, "visstat")
  
  expect_true(length(result) > 0)
  
  captured_plots <- attr(result, "captured_plots")
  expect_true(length(captured_plots) > 0)
  
  expect_true(is.list(result))
  
  expect_no_error(print(result))
})

#' Visualisation of the normality distribution of the standardised residuals
#'
#' Checks for normality of the standardised residuals in the general linear model
#' Student's t-test 
#' (t.test,var=EQUAL) 
#' Fisher oneway ANOVA (aov) or simple linear regression.
#' Performs the Shapiro-Wilk test and Anderson-Darling test for normality and,
#' if not a regression, also the Levene-Brown-Forsythe and the Bartlett's test for homogeneity of variances.
#' It produces a histogram with normal overlay, a residuals vs fitted plot,
#' and a normal Q-Q plot.
#'
#' @param samples Numeric vector; the dependent variable.
#' @param fact Factor; the independent variable.
#' @param cex Numeric; scaling factor for plot text and symbols (default: 1).
#' @param regression Logical; if TRUE, skips Bartlett's test (for regression diagnostics). Default is FALSE.
#'
#' @return A list with elements:
#' \describe{
#'   \item{summary_anova}{Summary of the ANOVA model.}
#'   \item{shapiro_test}{Result from \code{shapiro.test()}.}
#'   \item{ad_test}{Result from \code{nortest::ad.test()} or a character message if n < 7.}
#'   \item{levene_test}{Result from \code{levene.test()} (only if \code{regression = FALSE}).}
#'   \item{bartlett_test}{Result from \code{bartlett.test()} (only if \code{regression = FALSE}).}
#'   \item{bp_test}{Result from \code{bp.test()} (only if \code{regression = TRUE}).}
#' }
#'
#' @examples
#' ToothGrowth$dose <- as.factor(ToothGrowth$dose)
#' vis_lm_assumptions(ToothGrowth$len, ToothGrowth$dose)
#'
#' @export

vis_lm_assumptions <- function(samples, fact, cex = 1, regression = FALSE) {
  
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  
  # Clean the input
  samples3 <- na.omit(samples)
  fact <- subset(fact, !is.na(samples))
  samples <- samples3
  
  # Fit model
  anova_model <- aov(samples ~ fact) # needed for correct output structure, do not use lm(samples~fact)
  std_residuals <- rstandard(anova_model)
  
  # Run assumption tests
  # Shapiro-Wilk test (for n >= 3 and n <= 5000)
  if (length(std_residuals) >= 3 && length(std_residuals) <= 5000) {
    shapiro_test <- shapiro.test(std_residuals)
  } else {
    shapiro_test <- list(
      method = "Shapiro-Wilk normality test",
      statistic = NA,
      p.value = NA,
      data.name = "standardized residuals"
    )
  }
  
  # Anderson-Darling test (for n >= 7)
  if (length(std_residuals) >= 7) {
    ad_test <- nortest::ad.test(std_residuals)
  } else {
    ad_test <- "Sample size too small (n < 7) for Anderson-Darling test"
  }
  
  # Variance tests (only for non-regression)
  if (!regression) {
    levene_test <- levene.test(samples, fact)
    bartlett_test <- bartlett.test(samples ~ fact)
     bp_test <- NULL
  } else {
    # For regression: use Breusch-Pagan test for heteroscedasticity
    # (tests if error variance depends on independent variables)
    levene_test <- NULL
    bartlett_test <- NULL
     bp_test <-   bp.test(anova_model)
  }
  
  # Set up plotting area with outer margin for title
  par(mfrow = c(2, 2), oma = c(0, 0, 3, 0), cex = 0.7 * cex)
  
  # Plot 1: Histogram with normal overlay
  x_min <- min(min(std_residuals), -3.2)
  x_max <- max(max(std_residuals), 3.2)
  temp_hist <- hist(std_residuals, plot = FALSE)
  y_max <- max(max(temp_hist$density), 0.45)
  
  hist(std_residuals, freq = FALSE, main = "Histogram and Normal Density", 
       xlab = "Std. Residuals", col = "lightblue", border = "black",
       xlim = c(x_min, x_max), ylim = c(0, y_max))
  x_seq <- seq(x_min, x_max, length.out = 200)
  lines(x_seq, dnorm(x_seq), col = "red", lwd = 1)
  
  # Plot 2: Standardized Residuals vs Fitted (manual - plot.lm doesn't use standardized)
  ylim_res <- extendrange(std_residuals, f = 0.08)
  plot(fitted(anova_model), std_residuals, 
       main = "Std. Res. vs. Fitted",
       xlab = "Fitted values", 
       ylab = "Std. Residuals",
       ylim = ylim_res)
  abline(h = 0, col = "red", lwd = 1)
  
  
  
  # Plot 3: Normal Q-Q (using internal plot.lm method)
  plot(anova_model, which = 2, cex = cex, caption = "", sub.caption = "", main = "Normal Q-Q Plot")
  
  # Plot 4: Depends on regression vs ANOVA
  if (regression) {
    # Residuals vs Leverage with Cook's distance (using internal plot.lm method)
    plot(anova_model, which = 5, cex = cex, caption = "", sub.caption = "", main = "Std. Res. vs. Leverage")
  } else {
    # Scale-Location plot (using internal plot.lm method)
    plot(anova_model, which = 3, cex = cex, caption = "", sub.caption = "", main = "Scale-Location")
  }
  
  # Add overall title with test results
  p_shapiro <- if (!is.na(shapiro_test$p.value)) signif(shapiro_test$p.value, 2) else "N/A"
  p_AD <- if (is.list(ad_test) && !is.null(ad_test$p.value)) signif(ad_test$p.value, 2) else "N/A"
  
  if (regression) {
    # Regression title with Breusch-Pagan test - split into two rows
    p_bp <- signif( bp_test$p.value, 2)
    title_line1 <- paste("Linear model assumptions: Shapiro-Wilk p =", p_shapiro, 
                         "| Anderson-Darling p =", p_AD)
    title_line2 <- paste("Breusch-Pagan p =", p_bp)
    
    mtext(title_line1, side = 3, outer = TRUE, line = 1, cex = 0.7)
    mtext(title_line2, side = 3, outer = TRUE, line = 0, cex = 0.7)
  } else {
    # ANOVA title with Levene and Bartlett - split into two rows
    title_line1 <- paste("Linear model assumptions: Shapiro-Wilk p =", p_shapiro,
                         "| Anderson-Darling p =", if(is.numeric(p_AD)) signif(p_AD, 2) else p_AD)
    title_line2 <- paste("Levene-Brown-Forsythe p =", signif(levene_test$p.value, 2),
                         "| Bartlett p =", signif(bartlett_test$p.value, 2))
    
    mtext(title_line1, side = 3, outer = TRUE, line = 1, cex = 0.7)
    mtext(title_line2, side = 3, outer = TRUE, line = 0, cex = 0.7)
  }
  
  # Return results
  result <- list(
    summary_anova = summary(anova_model),
    shapiro_test = shapiro_test,
    ad_test = ad_test,
    levene_test = if (!regression) levene_test else NULL,
    bartlett_test = if (!regression) bartlett_test else NULL,
    bp_test = if (regression)  bp_test else NULL
  )
  
  class(result) <- "vis_lm_assumptions"
  return(invisible(result))
}

#' @rdname vis_lm_assumptions
#' @param ... Arguments passed to \code{vis_lm_assumptions()}.
#' @export
vis_anova_assumptions <- function(...) {
  .Deprecated("vis_lm_assumptions")
  vis_lm_assumptions(...)
}

#' Levene-Brown-Forsythe Test for Homogeneity of Variance (center = median)
#'
#' @description Performs Levene's test using the
#'  Brown-Forsythe modification (median-centred). 
#'  It tests the null hypothesis that all groups 
#' have equal variances by testing whether the absolute deviations
#'  from group  medians are equal across groups
#'  The function reproduces the default behaviour of the 
#'  leveneTest(y,g,center=median,...) of the car-package. 
#'
#' @param y A numeric response vector.
#' @param g A grouping factor.
#' @param data Optional data frame containing `y` and `g`.
#'
#' @return An object of class \code{"htest"} with components:
#' \item{statistic}{the value of the F-statistic.}
#' \item{parameter}{degrees of freedom: df1=k-1, 
#' df3=N-k, where k is the number of groups and N the total sample size }
#' \item{p.value}{the p-value of the test.}
#' \item{method}{a character string indicating the test performed.}
#' \item{data.name}{a character string giving the name(s) of the data.}
#'
#' @details 
#' For each observation \eqn{y_{ij}} in group \eqn{i}, compute the absolute 
#' deviation from the group median:
#' 
#' \deqn{z_{ij} = |y_{ij} - \tilde{y}_i|}
#' 
#' where \eqn{\tilde{y}_i} is the median of group \eqn{i}.
#' 
#' The test statistic is the F-statistic from a one-way ANOVA on the \eqn{z_{ij}} values:
#' 
#' \deqn{F = \frac{(N-k) \sum_{i=1}^{k} n_i (\bar{z}_i - \bar{z})^2}{(k-1) \sum_{i=1}^{k} \sum_{j=1}^{n_i} (z_{ij} - \bar{z}_i)^2}}
#' 
#' where:
#' \itemize{
#'   \item \eqn{k} = number of groups
#'   \item \eqn{N} = total sample size
#'   \item \eqn{n_i} = sample size of group \eqn{i}
#'   \item \eqn{\bar{z}_i} = mean of absolute deviations in group \eqn{i}
#'   \item \eqn{\bar{z}} = overall mean of all absolute deviations
#' }
#' 
#' Under the null hypothesis of equal variances, the test statistic follows 
#' an F-distribution: \eqn{F \sim F(k-1, N-k)}.
#' 
#' @references
#' Brown, M. B., and Forsythe, A. B. (1974). Robust tests for the equality of
#' variances. Journal of the American Statistical Association, 69(346), 364–367.
#' DOI: 10.1080/01621459.1974.10482955
#' 
#' @examples
#' set.seed(123)
#' y <- c(rnorm(10), rnorm(10, sd = 2), rnorm(10, sd = 0.5))
#' g <- factor(rep(1:3, each = 10))
#' levene.test(y, g)
#' 
#' # Usage with data frame
#' df <- data.frame(response = y, group = g)
#' levene.test(response, group, data = df)
#' 
#' # Example with unequal variances (should reject null hypothesis)
#' set.seed(456)
#' y_unequal <- c(rnorm(15, sd = 1), rnorm(15, sd = 5), rnorm(15, sd = 0.2))
#' g_unequal <- factor(rep(c("A", "B", "C"), each = 15))
#' levene.test(y_unequal, g_unequal)
#'
#' 
#'
#' @export
levene.test <- function(y, g, data = NULL) {
  
  
   # Error handling ----
  if (!is.null(data)) {
    y <- eval(substitute(y), data, parent.frame())
    g <- eval(substitute(g), data, parent.frame())
  }
  
  if (!is.numeric(y)) stop("`y` must be numeric")
  if (!is.factor(g)) g <- factor(g)
  
  # Leven test ------
  
  medians <- tapply(y, g, median, na.rm = TRUE)
  z <- abs(y - medians[g])
  
  fit <- stats::aov(z ~ g)
  aov_summary <- summary(fit)[[1]]
  
  f_value <- aov_summary[["F value"]][1]
  p_value <- aov_summary[["Pr(>F)"]][1]
  df1 <- aov_summary[["Df"]][1]
  df2 <- aov_summary[["Df"]][2]
  
  structure(
    list(
      statistic = c(F = f_value),
      parameter = c(df1 = df1, df2 = df2),
      p.value = p_value,
      method = "Levene-Brown-Forsythe Test (center = median)",
      data.name = "absolute deviations from group medians (for ANOVA on spread differences)"
    ),
    class = "htest"
  )
}

# Test file for levene.test function
# Save as: test-levene-test.R

library(testthat)

# Helper function to create test data
create_test_data <- function() {
  set.seed(123)
  list(
    equal_var = list(
      y = c(rnorm(10, mean = 0, sd = 1), 
            rnorm(10, mean = 2, sd = 1), 
            rnorm(10, mean = 4, sd = 1)),
      g = factor(rep(1:3, each = 10))
    ),
    unequal_var = list(
      y = c(rnorm(15, mean = 0, sd = 1), 
            rnorm(15, mean = 0, sd = 5), 
            rnorm(15, mean = 0, sd = 0.2)),
      g = factor(rep(c("A", "B", "C"), each = 15))
    )
  )
}

test_that("levene.test returns correct structure", {
  test_data <- create_test_data()
  result <- levene.test(test_data$equal_var$y, test_data$equal_var$g)
  
  # Check class
  expect_s3_class(result, "htest")
  
  # Check components exist
  expect_true(all(c("statistic", "parameter", "p.value", "method", "data.name") %in% names(result)))
  
  # Check statistic is named F
  expect_named(result$statistic, "F")
  
  # Check parameters are named df1 and df2
  expect_named(result$parameter, c("df1", "df2"))
  
  # Check method string
  expect_equal(result$method, "Levene-Brown-Forsythe Test (center = median)")
})

test_that("levene.test calculates correct degrees of freedom", {
  test_data <- create_test_data()
  
  # Test with 3 groups of 10 each (N=30, k=3)
  result <- levene.test(test_data$equal_var$y, test_data$equal_var$g)
  expect_equal(result$parameter[["df1"]], 2)  # k-1 = 3-1 = 2
  expect_equal(result$parameter[["df2"]], 27) # N-k = 30-3 = 27
  
  # Test with 3 groups of 15 each (N=45, k=3)
  result2 <- levene.test(test_data$unequal_var$y, test_data$unequal_var$g)
  expect_equal(result2$parameter[["df1"]], 2)  # k-1 = 3-1 = 2
  expect_equal(result2$parameter[["df2"]], 42) # N-k = 45-3 = 42
})

test_that("levene.test works with data frame argument", {
  test_data <- create_test_data()
  df <- data.frame(
    response = test_data$equal_var$y,
    group = test_data$equal_var$g
  )
  
  # Test with data frame
  result_df <- levene.test(response, group, data = df)
  
  # Test without data frame
  result_no_df <- levene.test(test_data$equal_var$y, test_data$equal_var$g)
  
  # Results should be identical
  expect_equal(result_df$statistic, result_no_df$statistic)
  expect_equal(result_df$parameter, result_no_df$parameter)
  expect_equal(result_df$p.value, result_no_df$p.value)
})

test_that("levene.test handles factor conversion", {
  test_data <- create_test_data()
  
  # Test with character grouping variable
  g_char <- as.character(test_data$equal_var$g)
  result_char <- levene.test(test_data$equal_var$y, g_char)
  
  # Test with factor grouping variable
  result_factor <- levene.test(test_data$equal_var$y, test_data$equal_var$g)
  
  # Results should be identical
  expect_equal(result_char$statistic, result_factor$statistic)
  expect_equal(result_char$p.value, result_factor$p.value)
})

test_that("levene.test produces reasonable results for known cases", {
  # Test case where variances are clearly different
  set.seed(456)
  y_diff <- c(rnorm(20, sd = 0.1), rnorm(20, sd = 10))  # Very different variances
  g_diff <- factor(rep(c("low", "high"), each = 20))
  
  result_diff <- levene.test(y_diff, g_diff)
  
  # Should reject null hypothesis (small p-value)
  expect_lt(result_diff$p.value, 0.05)
  
  # Test case where variances are similar
  set.seed(789)
  y_same <- c(rnorm(20, sd = 1), rnorm(20, sd = 1.1))  # Similar variances
  g_same <- factor(rep(c("A", "B"), each = 20))
  
  result_same <- levene.test(y_same, g_same)
  
  # Should not reject null hypothesis (larger p-value)
  expect_gt(result_same$p.value, 0.05)
})

test_that("levene.test handles edge cases", {
  # Test with minimum number of groups (2)
  set.seed(100)
  y_min <- c(rnorm(5), rnorm(5, sd = 2))
  g_min <- factor(rep(1:2, each = 5))
  
  expect_no_error(levene.test(y_min, g_min))
  result_min <- levene.test(y_min, g_min)
  expect_equal(result_min$parameter[["df1"]], 1)  # 2-1 = 1
  expect_equal(result_min$parameter[["df2"]], 8)  # 10-2 = 8
})

test_that("levene.test input validation works", {
  test_data <- create_test_data()
  
  # Test non-numeric y
  expect_error(
    levene.test(as.character(test_data$equal_var$y), test_data$equal_var$g),
    "`y` must be numeric"
  )
  
  # Test mismatched lengths
  expect_error(
    levene.test(test_data$equal_var$y[1:5], test_data$equal_var$g),
    # This should produce an error from the internal functions
    class = "error"
  )
})

test_that("levene.test handles missing values appropriately", {
  test_data <- create_test_data()
  
  # Add some NA values
  y_na <- test_data$equal_var$y
  y_na[c(1, 15, 25)] <- NA
  
  # Function should handle NAs through tapply's na.rm = TRUE
  expect_no_error(levene.test(y_na, test_data$equal_var$g))
})

test_that("levene.test numerical accuracy", {
  # Test against known mathematical properties
  # When all groups have identical values, variance should be 0 and test should be undefined
  y_identical <- rep(c(1, 2, 3), each = 10)
  g_identical <- factor(rep(1:3, each = 10))
  
  # This might produce NaN or extreme values
  result_identical <- levene.test(y_identical, g_identical)
  
  # The test statistic should be 0 when there's no within-group variation
  expect_true(is.numeric(result_identical$statistic))
})

# test_that("levene.test matches car::leveneTest when available", {
#   # Double-check that car is available without triggering installation
#   skip_if_not(requireNamespace("car", quietly = TRUE), "car package not available")
#   
#   test_data <- create_test_data()
#   our_result <- levene.test(test_data$equal_var$y, test_data$equal_var$g)
#   
#   # Compare with car package result
#   car_result <- car::leveneTest(test_data$equal_var$y, test_data$equal_var$g, center = median)
#   
#   # Test statistics should be very close (allowing for small numerical differences)
#   expect_equal(our_result$statistic[["F"]], car_result$`F value`[1], tolerance = 1e-10)
#   expect_equal(our_result$p.value, car_result$`Pr(>F)`[1], tolerance = 1e-10)
#   expect_equal(our_result$parameter[["df1"]], car_result$Df[1])
#   expect_equal(our_result$parameter[["df2"]], car_result$Df[2])
# })

# MIT License----
# Copyright (c) 2020 Sabine Schilling
# Feedback highly welcome: sabineschilling@gmx.ch

# Header visstat_core -----
#'
#' Automated Visualization of Statistical Hypothesis Testing
#'
#' @description \code{visstat_core()} provides automated selection and visualization 
#' of a statistical hypothesis test between a two vectors in
#' a given \code{data.frame} named \code{dataframe} based on the data's type, 
#' distribution, sample size, and the
#' specified \code{conf.level}. \code{visstat_core()} is called by the main
#'  wrapper function \code{visstat()}.
#' \code{varsample} and \code{varfactor} are \code{character}
#' strings corresponding to the column names of the chosen vectors in \code{dataframe}. 
#' These vectors must be of type \code{integer}, \code{numeric} or \code{factor}.
#' The automatically generated output figures
#' illustrate the selected statistical hypothesis test, display the main test
#' statistics, and include assumption checks and post hoc comparisons when
#' applicable. The primary test results are returned as a list object.
#'
#' @details The decision logic for selecting a statistical test is described below.
#' For more details, please refer to the package's \code{vignette("visStatistics")}.
#' Throughout, data of class \code{numeric} or \code{integer} are referred to as
#' numeric, while data of class \code{factor} are referred to as categorical.
#' The significance level \code{alpha} is defined as one minus the confidence
#' level, given by the argument \code{conf.level}. Assumptions of normality and
#' homoscedasticity are considered met when the corresponding test yields a
#' p-value greater than \code{alpha = 1 - conf.level}.
#' The choice of statistical tests performed by \code{visstat_core()} depends on
#' whether the data are numeric or categorical, the number of levels in the
#' categorical variable, the distribution of the data, and the chosen
#' \code{conf.level}. The function prioritises interpretable visual output and
#' tests that remain valid under their assumptions, following the logic below:
#' 
#' (1) When the response is numerical and the predictor is categorical, tests of
#' central tendencies are performed. For the decision logic, please refer to the 
#' packages vignette \code{vignette("visStatistics")}
#' 
#' (2): When both the response and predictor are numerical, a linear model
#' \code{lm()} is fitted, with residual diagnostics and a confidence band plot.
#' 
#' (3): When both variables are categorical, \code{visstat_core()} uses
#' \code{chisq.test()} or \code{fisher.test()} depending on expected counts,
#' following Cochran's rule (Cochran (1954) <doi:10.2307/3001666>).
#' 
#' Implemented main tests: 
#' 
#' \code{t.test()}, \code{wilcox.test()}, \code{aov()},
#' \code{oneway.test()}, \code{lm()}, \code{kruskal.test()},
#' \code{fisher.test()}, \code{chisq.test()}.
#' 
#' Implemented tests for assumptions:
#' \itemize{
#'   \item Normality: \code{shapiro.test()} and \code{ad.test()}
#'   \item Heteroscedasticity: \code{bartlett.test()} and \code{levene.test()} and \code{bp_test()}
#' }
#' 
#' Implemented post hoc tests:
#' \itemize{
#'   \item \code{TukeyHSD()} for \code{aov()} 
#'   \item \code{games.howell} for  \code{oneway.test()}
#'   \item \code{pairwise.wilcox.test()} for \code{kruskal.test()}
#' }
#' @seealso
#' The package's vignette
#' \code{vignette("visStatistics")} for a description of the 
#' decision logic, illustrated with numerous examples. The package is accompanied 
#' by its webpage
#' \url{https://shhschilling.github.io/visStatistics/}. The main function \code{\link{visstat}} for a detailed description of the return value.  
#'
#' @param dataframe \code{data.frame} with at least two columns.
#' @param varsample \code{character} string matching a column name in 
#'   \code{dataframe}. Interpreted as the response if the referenced column is 
#'   of class \code{numeric} or \code{integer} and the column named by 
#'   \code{varfactor} is of class \code{factor}.
#' @param varfactor \code{character} string matching a column name in 
#'   \code{dataframe}. Interpreted as the grouping variable if the referenced 
#'   column is of class \code{factor} and the column named by \code{varsample} 
#'   is of class \code{numeric} or \code{integer}.
#' @param conf.level Confidence level
#' @param correlation Logical. If FALSE (default), performs simple
#' linear regression analysis with confidence and prediction bands.
#' If TRUE, performs Spearman correlation analysis with trend line only
#'  (no regression interpretation).
#' @param numbers a logical indicating whether to show numbers in mosaic count
#'   plots.
#' @param minpercent number between 0 and 1 indicating minimal fraction of total
#'   count data of a category to be displayed	in mosaic count plots.
#' @param graphicsoutput saves plot(s) of type "png",  "jpg", "tiff" or  "bmp"
#'   in directory specified in \code{plotDirectory}. If graphicsoutput=NULL, no
#'   plots are saved.
#' @param plotName graphical output is stored following the naming convention
#'   "plotName.graphicsoutput" in \code{plotDirectory}. Without specifying this
#'   parameter, plotName is automatically generated following the convention
#'   "statisticalTestName_varsample_varfactor".
#' @param plotDirectory specifies directory, where generated plots are stored.
#'   Default is current working directory.
#' @return An object of class \code{"visstat"} containing the results of 
#' the automatically selected statistical test. The specific contents depend on
#'  which test was performed.
#' Additionally, the returned object includes two attributes:
#' \itemize{
#'   \item \code{plot_paths}: Character vector of file paths where plots were 
#'     saved (if \code{graphicsoutput} was specified)
#'   \item \code{captured_plots}: List of captured plot objects for programmatic 
#'     access
#' }
#' @examples
#' # Welch Two Sample t-test (t.test())
#' visstat_core(mtcars, "mpg", "am")
#'
#' ## Wilcoxon rank sum test (wilcox.test())
#' grades_gender <- data.frame(
#'   Sex = as.factor(c(rep("Girl", 20), rep("Boy", 20))),
#'   Grade = c(
#'     19.3, 18.1, 15.2, 18.3, 7.9, 6.2, 19.4,
#'     20.3, 9.3, 11.3, 18.2, 17.5, 10.2, 20.1, 13.3, 17.2, 15.1, 16.2, 17.3,
#'     16.5, 5.1, 15.3, 17.1, 14.8, 15.4, 14.4, 7.5, 15.5, 6.0, 17.4,
#'     7.3, 14.3, 13.5, 8.0, 19.5, 13.4, 17.9, 17.7, 16.4, 15.6
#'   )
#' )
#' visstat_core(grades_gender, "Grade", "Sex")
#'
#' ## Welch's oneway ANOVA not assuming equal variances (oneway.test())
#' anova_npk <- visstat_core(npk, "yield", "block")
#' anova_npk # prints summary of tests
#'
#' ## Kruskal-Wallis rank sum test (kruskal.test())
#' visstat_core(iris, "Petal.Width", "Species")
#' visstat_core(InsectSprays, "count", "spray")
#'
#' ## Simple linear regression  (lm())
#' visstat_core(trees, "Girth", "Height", conf.level = 0.99)
#'
#' ## Pearson's Chi-squared test (chisq.test())
#' ### Transform array to data.frame
#' HairEyeColorDataFrame <- counts_to_cases(as.data.frame(HairEyeColor))
#' visstat_core(HairEyeColorDataFrame, "Hair", "Eye")
#'
#' ## Fisher's exact test (fisher.test())
#' HairEyeColorMaleFisher <- HairEyeColor[, , 1]
#' ### slicing out a 2 x2 contingency table
#' blackBrownHazelGreen <- HairEyeColorMaleFisher[1:2, 3:4]
#' blackBrownHazelGreen <- counts_to_cases(as.data.frame(blackBrownHazelGreen))
#' fisher_stats <- visstat_core(blackBrownHazelGreen, "Hair", "Eye")


#'
#' @import vcd
#' @import Cairo
#' @import graphics
#' @import grDevices
#' @import grid
#' @import multcompView
#' @import stats
#' @import utils
#' @importFrom nortest ad.test
#' @export visstat_core


visstat_core <- function(dataframe,
                         varsample,
                         varfactor,
                         conf.level = 0.95,
                         correlation = FALSE,
                         numbers = TRUE,
                         minpercent = 0.05,
                         graphicsoutput = NULL,
                         plotName = NULL,
                         plotDirectory = getwd()) {
  stopifnot(is.data.frame(dataframe))
  stopifnot(varsample %in% names(dataframe))
  stopifnot(varfactor %in% names(dataframe))
  
  
  
  capture_env <- new.env()
  capture_env$captured_plots <- list() #restart list of caputre plots 
  #capture_env$capture_next_plot <- FALSE
  
  
  
  # store default graphical parameters------
  oldparvisstat_core <- par(no.readonly = TRUE)
  oldparvisstat_core$new <- FALSE # reset the default value
  on.exit(par(oldparvisstat_core))
  
  
  # Collect plot paths from plot_paths <- c(plot_paths, saveGraphVisstat())
  plot_paths <- character(0)
  
  # Set default values---------------------------
  alpha <- 1 - conf.level
  
  ## Get input variables---------------------------------
  input <-
    get_samples_fact_inputfile(dataframe, varsample, varfactor)
  # out of function get_groups_inputfile
  samples <- input$samples
  fact <- input$fact
  name_of_sample <- input$name_of_sample
  name_of_factor <- input$name_of_factor
  matchingCriteria <- input$matchingCriteria
  
  # Detect ordered x ordered case:
  # Only route to Kendall's tau-b when correlation=TRUE; otherwise treat as
  # ordered response + factor predictor (Wilcoxon/Kruskal-Wallis).
  # For ordered response with non-ordered predictor we always convert to
  # numeric ranks and route to the non-parametric pathway.
  ordinal_response <- FALSE
  both_ordered <- is.ordered(samples) && is.ordered(fact)
  use_kendall   <- both_ordered && correlation

  if (both_ordered && !correlation && nlevels(fact) > 4) {
    warning(
      "Ordered predictor with ", nlevels(fact), " levels detected. ",
      "Kruskal-Wallis discards the ordering of the predictor. ",
      "If a monotone association is of interest, consider correlation = TRUE ",
      "for Kendall's tau_b.",
      call. = FALSE
    )
  }

  if (is.ordered(samples) && (!both_ordered || !correlation)) {
    warning("Ordered response detected. Converting to integer level codes for non-parametric analysis.", call. = FALSE)
    samples <- as.numeric(samples)
    ordinal_response <- TRUE  # Flag to force non-parametric
  }
  
  vis_sample_fact <- list()
  
  # dependent on samples, fact, name_of_sample, name_of_factor, conf.level,
  
  typesample <- class(samples)
  typefactor <-
    class(fact) # type of independent variable returned as a character vector
  
  
  # transform independent variable "fact" of class "character" to factor
  if (inherits(fact, "character")) {
    fact <-
      as.factor(fact) # transform  "fact" of class "character" to factor
    typefactor <- class(fact) # store the class of type "factor"
  }
  
  #  Check order 
  
  if ((inherits(fact, "numeric") || inherits(fact, "integer")) && inherits(samples, "factor")) {
    stop("A numeric or integer predictor with a factor response is ignored.")
  }
  
  maxlabels <- length(levels(samples))
  #  Comparison of all  possible combinations of input variables ----
  #  
  #  
  ## A) median or mean-------
  # --- Numeric vs Factor Logic with Original Error Catching ---
  if ((inherits(samples, "integer") | inherits(samples, "numeric")) &&
      inherits(fact, "factor") && nlevels(fact) >= 2) {
    
    # Pre-check: Original error handling for insufficient data
    counts_per_level <- table(fact)
    if (any(counts_per_level < 1) | length(samples) < 3) {
      warning("In each group must be at least one member and total sample size >= 3.", call. = FALSE)
      vis_sample_fact <- list(
        error = "Insufficient data",
        input_summary = list(sample_name = name_of_sample, factor_name = name_of_factor)
      )
      attr(vis_sample_fact, "plot_paths") <- plot_paths
      class(vis_sample_fact) <- "visstat"
      return(vis_sample_fact)
    }
    
    # Check if response was originally ordinal - force non-parametric
    if (ordinal_response) {
      normality_met <- FALSE
    } else {
      # MANDATORY DIAGNOSTIC: Provide visual evidence for the decision pipeline
      openGraphCairo(type = graphicsoutput, fileDirectory = plotDirectory) 
      vis_lm_assumptions(samples, fact, cex = 0.8)
      
      if (is.null(plotName)) {
        filename <- paste("glm_assumptions_", name_of_sample, "_", name_of_factor, sep = "")
      } else {
        filename <- paste("glm_assumptions_", plotName, sep = "")
      }
      plot_paths <- c(plot_paths, saveGraphVisstat(fileName = filename, type = graphicsoutput, 
                                                   fileDirectory = plotDirectory, capture_env = capture_env))
      
      # Decision logic gate
      all_groups_large <- all(counts_per_level > 50)
      
      if (all_groups_large) {
        normality_met <- TRUE 
      } else {
        current_model <- lm(samples ~ fact)
        std_resids <- rstandard(current_model) #this is already part of the output of vis_lm_assumptions
        normality_met <- shapiro.test(std_resids)$p.value >= alpha
      }
    }
    
    # Testing and Visualization steps
    if (!normality_met) {
      # --- NON-PARAMETRIC BRANCH ---
      openGraphCairo(type = graphicsoutput, fileDirectory = plotDirectory) 
      if (nlevels(fact) == 2) {
        vis_sample_fact <- two_sample_wilcoxon_test(samples, fact, conf.level = conf.level, 
                                                    samplename = varsample, factorname = varfactor)
        if (is.null(plotName)) {
          filename <- paste("wilcoxon_", name_of_sample, "_", name_of_factor, sep = "")
        } else {
          filename <- paste(plotName)
        }
      } else {
        vis_sample_fact <- vis_Kruskal_Wallis(samples, fact, conf.level = conf.level, 
                                              samplename = varsample, factorname = varfactor)
        if (is.null(plotName)) {
          filename <- paste("kruskal_", name_of_sample, "_", name_of_factor, sep = "")
        } else {
          filename <- paste(plotName)
        }
      }
      plot_paths <- c(plot_paths, saveGraphVisstat(fileName = filename, type = graphicsoutput, 
                                                   fileDirectory = plotDirectory, capture_env = capture_env))
    } else {
      # --- PARAMETRIC BRANCH ---
      var_p <- levene.test(samples, fact)$p.value
      if (nlevels(fact) == 2) {
        # Group-wise normality diagnostics for Welch t-tests
        # visualization of normality assumption per group
        if (var_p < alpha) {
          openGraphCairo(type = graphicsoutput, fileDirectory = plotDirectory)
          vis_group_normality(samples, fact, conf.level = conf.level, cex = 0.8)
          
          if (is.null(plotName)) {
            filename <- paste("ttest_assumptions_", name_of_sample, "_", name_of_factor, sep = "")
          } else {
            filename <- paste("ttest_assumptions_", plotName, sep = "")
          }
          plot_paths <- c(plot_paths, saveGraphVisstat(fileName = filename, type = graphicsoutput,
                                                       fileDirectory = plotDirectory, capture_env = capture_env))
        }
        # Final t-test execution
        openGraphCairo(type = graphicsoutput, fileDirectory = plotDirectory) 
        vis_sample_fact <- two_sample_t_test(samples, fact, var.equal = (var_p >= alpha), 
                                             conf.level = conf.level, samplename = varsample, 
                                             factorname = varfactor)
        if (is.null(plotName)) {
          filename <- paste("ttest_", name_of_sample, "_", name_of_factor, sep = "")
        } else {
          filename <- paste(plotName)
        }
        plot_paths <- c(plot_paths, saveGraphVisstat(fileName = filename, type = graphicsoutput, 
                                                     fileDirectory = plotDirectory, capture_env = capture_env))
      } else {
        # ANOVA execution (Fisher/Welch and Post-hoc handled internally)
        # 
        if (var_p < alpha) {
          # Unequal variances - will use Welch ANOVA, show normality per group
          openGraphCairo(type = graphicsoutput, fileDirectory = plotDirectory)
          vis_group_normality(samples, fact, conf.level = conf.level, cex = 0.8)
          
          if (is.null(plotName)) {
            filename <- paste("anova_assumptions_", name_of_sample, "_", name_of_factor, sep = "")
          } else {
            filename <- paste("anova_assumptions_", plotName, sep = "")
          }
          plot_paths <- c(plot_paths, saveGraphVisstat(fileName = filename, type = graphicsoutput,
                                                       fileDirectory = plotDirectory, capture_env = capture_env))
        }
        openGraphCairo(type = graphicsoutput, fileDirectory = plotDirectory) 
        vis_sample_fact <- vis_anova(samples, fact, samplename = varsample, 
                                     factorname = varfactor, conf.level = conf.level)
        if (is.null(plotName)) {
          filename <- paste("anova_", name_of_sample, "_", name_of_factor, sep = "")
        } else {
          filename <- paste(plotName)
        }
        plot_paths <- c(plot_paths, saveGraphVisstat(fileName = filename, type = graphicsoutput, 
                                                     fileDirectory = plotDirectory, capture_env = capture_env))
      }
    }
  }
  
  
  ## B) Both variables of class factor -----
  ##
  ## "ordered" is a subclass of "factor", so the factor-x-factor branch
  ## handles two sub-cases:
  ##   B.1) both ordered AND correlation=TRUE -> Kendall's tau-b rank correlation
  ##   B.2) at least one nominal, or both ordered but correlation=FALSE
  ##        -> Chi^2 / Fisher exact test (ordered response already converted above)

  if (inherits(fact, "factor") && inherits(samples, "factor")) {

    if (use_kendall) {
      ## ----- B.1) Both ordered + correlation=TRUE: Kendall's tau-b -----
      ##
      ## Treating ordered levels as nominal would discard the ordering
      ## and lose power against a monotone trend. Kendall's tau-b
      ## handles tied ranks (unavoidable with few levels, e.g. Likert)
      ## more accurately than Spearman's rho (Agresti 2010, ch. 2;
      ## Kendall 1945).
      samples_num <- as.numeric(samples)
      fact_num    <- as.numeric(fact)

      kendall_test <- suppressWarnings(
        cor.test(samples_num, fact_num,
                 method = "kendall", exact = FALSE,
                 conf.level = conf.level)
      )

      # Plot 1: jittered rank-rank scatter
      # Title via mtext() (outer margin) to match the font used by all other
      # test functions; no "(n=...)" — no other test reports sample size there.
      openGraphCairo(type = graphicsoutput, fileDirectory = plotDirectory)
      # Adaptive left margin: las=1 prints y-axis labels horizontally.
      # strwidth() measures actual rendered width in inches (device already open);
      # dividing by par("csi") converts to margin lines. +2 for tick gap + ylab.
      max_ylabel_in <- max(strwidth(as.character(levels(samples)), units = "inches"))
      label_lines   <- ceiling(max_ylabel_in / par("csi"))  # lines for tick text
      ylab_line     <- label_lines + 1                       # ylab 1 line beyond
      left_mar      <- max(5, ylab_line + 1)                 # margin + 1 buffer
      op <- par(oma = c(0, 0, 3, 0), mar = c(5, left_mar, 4, 2) + 0.1)

      # Colour by x-axis group (fact levels), consistent with boxplot/Kruskal.
      # Semi-transparency preserved so overlapping points show as darker shades.
      n_x <- length(levels(fact))
      if (n_x <= 2) {
        base_cols <- colorscheme(1)
      } else if (n_x <= length(colorscheme(3)) + 2) {
        base_cols <- c(colorscheme(1), head(colorscheme(3), n_x - 2))
      } else {
        base_cols <- rainbow(n_x, s = 0.4, alpha = 1)
      }
      point_cols <- adjustcolor(base_cols[fact_num], alpha.f = 0.6)

      # ylab placed via mtext() so it sits at the outer margin edge, clear of
      # the horizontal tick labels (las=1). Using plot(ylab=...) would anchor
      # it at mgp[1]=3 lines — inside the tick text for any label >2 lines wide.
      plot(jitter(fact_num,    amount = 0.15),
           jitter(samples_num, amount = 0.15),
           xlab = name_of_factor,
           ylab = "",
           xaxt = "n", yaxt = "n",
           pch = 19, col = point_cols)
      axis(1, at = seq_along(levels(fact)),    labels = levels(fact))
      axis(2, at = seq_along(levels(samples)), labels = levels(samples), las = 1)
      mtext(name_of_sample, side = 2, line = ylab_line, las = 0)
      mtext(bquote("Kendall's" ~ tau[b] ~ "=" ~
                     .(round(kendall_test$estimate, 3))), line = 2)
      mtext(bquote("p =" ~ .(signif(kendall_test$p.value, 3))),  line = 1)
      par(op)

      if (is.null(plotName)) {
        filename <- paste("kendall_", name_of_sample, "_", name_of_factor, sep = "")
      } else {
        filename <- paste(plotName, "_kendall", sep = "")
      }
      plot_paths <- c(plot_paths, saveGraphVisstat(fileName = filename,
                                                   type = graphicsoutput,
                                                   fileDirectory = plotDirectory,
                                                   capture_env = capture_env))

      # No mosaic for Kendall: shade=FALSE + ordered factors renders all tiles
      # black; and the jitter scatter already captures the rank structure.

      # cor.test returns class "htest" with $method, $p.value, ... so we put
      # it directly under $test for print.visstat / summary.visstat.
      kendall_test$data.name <- paste(name_of_sample, "and", name_of_factor)
      vis_sample_fact <- list(
        test             = kendall_test,
        n                = length(samples_num),
        levels_response  = levels(samples),
        levels_predictor = levels(fact)
      )

    } else {
      ## ----- B.2) At least one nominal: Chi^2 / Fisher -----
    if (check_assumptions_count_data(samples, fact) == FALSE) {
      # vis_sample_fact <-
      #   makeTable(samples, fact, name_of_sample, name_of_factor)
      vis_sample_fact <- tryCatch({
        makeTable(samples, fact, name_of_sample, name_of_factor)
      }, error = function(e) {
        list(error = paste("Failed to create contingency table:", e$message))
      })
      
      
    } else {
      # Chi^2 Test-----
      openGraphCairo(type = graphicsoutput,fileDirectory = plotDirectory) 
      
      vis_chi <-
        vis_chi_squared_test(samples, fact, name_of_sample, name_of_factor)
      if (is.null(plotName)) {
        filename <- paste("chi_squared_or_fisher_",
                          name_of_sample,
                          "_",
                          name_of_factor,
                          sep = "")
      } else {
        filename <- paste(plotName, "_", "chi_squared_or_fisher", sep = "")
      }
      
      plot_paths <- c(plot_paths, saveGraphVisstat(fileName = filename,
                                                   type = graphicsoutput,
                                                   fileDirectory = plotDirectory,capture_env = capture_env))
      # Mosaic plots: only for Pearson chi-square without Yates correction
      # (not Fisher's exact test, not Yates-corrected 2x2 tables)
      is_fisher <- isTRUE(grepl("Fisher", vis_chi$method, ignore.case = TRUE))
      is_yates  <- isTRUE(grepl("Yates",  vis_chi$method, ignore.case = TRUE))
      vis_mosaic_res <- NULL

      if (!is_fisher && !is_yates) {
        # a) complete labeled mosaic graph
        if (maxlabels > 7) {
          numberflag <- FALSE
        } else {
          numberflag <- TRUE
        }

        openGraphCairo(type = graphicsoutput, fileDirectory = plotDirectory)

        vis_mosaic_res <- vis_mosaic(
          samples,
          fact,
          name_of_sample = name_of_sample,
          name_of_factor = name_of_factor,
          minperc = 0,
          numbers = numberflag,
          shade = !grepl("Yates", vis_chi$method, ignore.case = TRUE)
        )

        if (is.null(plotName)) {
          filename <- paste("mosaic_complete_", name_of_sample, "_", name_of_factor, sep = "")
        } else {
          filename <- paste(plotName, "_", "mosaic_complete", sep = "")
        }

        plot_paths <- c(plot_paths, saveGraphVisstat(filename, type = graphicsoutput,
                                                     fileDirectory = plotDirectory,
                                                     capture_env = capture_env))

        # b) reduced mosaic if many levels
        if (maxlabels > 7) {
          openGraphCairo(type = graphicsoutput, fileDirectory = plotDirectory)

          vis_mosaic_res <- vis_mosaic(
            samples,
            fact,
            name_of_sample = name_of_sample,
            name_of_factor = "groups",
            minperc = minpercent,
            numbers = TRUE,
            shade = !grepl("Yates", vis_chi$method, ignore.case = TRUE)
          )
          plot_paths <- c(plot_paths, saveGraphVisstat(
            paste("mosaic_reduced_", name_of_sample, "_", name_of_factor, sep = ""),
            type = graphicsoutput,
            fileDirectory = plotDirectory,
            capture_env = capture_env
          ))
        }
      }

      vis_sample_fact <- c(vis_chi, vis_mosaic_res)
    }
    }  # end B.2 (nominal Chi^2 / Fisher)
  }    # end B (factor x factor)
  # C) both types numerical: Regression-----
  
  # Both samples and fact of type integer or numeric
  # Regression
  #
  #
  if ((inherits(fact, "integer") | inherits(fact, "numeric")) &&
      (inherits(samples, "integer") | inherits(samples, "numeric"))) {
    
    # samples: independent variable, factor: dependent   variable
    # check normality
    # 
    openGraphCairo(type = graphicsoutput,fileDirectory = plotDirectory
    ) 
    
    
    normality_residual_assumption <-
      vis_lm_assumptions(samples, fact,cex = 0.8,regression = TRUE)
    
    
    
    
    if (is.null(plotName)) {
      filename <-
        paste("glm_assumptions_", varsample, "_", varfactor, sep = "")
    } else {
      filename <-  paste("glm_assumptions_",plotName)
    }
    
    plot_paths <- c(
      plot_paths,
      saveGraphVisstat(
        fileName = filename,
        type = graphicsoutput,
        fileDirectory = plotDirectory,
        capture_env = capture_env
      )
    )
    
    openGraphCairo(type = graphicsoutput,fileDirectory = plotDirectory) 
    
    vis_sample_fact <- vis_numeric(
      samples,
      # y: dependent
      fact,
      # x: independent
      name_of_factor = name_of_factor,
      name_of_sample = name_of_sample,
      conf.level = conf.level,
      correlation = correlation
    )
    if (is.null(plotName)) {
      filename <-
        paste("regression_", name_of_sample, "_", name_of_factor, sep = "")
    } else {
      filename <- paste(plotName)
    }
    
    plot_paths <- c(plot_paths, saveGraphVisstat(fileName = filename,
                                                 type = graphicsoutput,
                                                 fileDirectory = plotDirectory,capture_env = capture_env))
  }
  
  
  
  # At the very end:
  attr(vis_sample_fact, "plot_paths") <- plot_paths
  attr(vis_sample_fact, "captured_plots") <- capture_env$captured_plots
  class(vis_sample_fact) <- "visstat"
  
  # FORCE ALL CAIRO OPERATIONS TO COMPLETE
  if (!is.null(graphicsoutput)) {
    while (!is.null(dev.list())) {
      dev.off()
    }
  }
  
  if (!exists("vis_sample_fact") || is.null(vis_sample_fact)) {
    vis_sample_fact <- list(error = "Analysis completed but no results were generated")
  }
  
  return(invisible(vis_sample_fact))
}
# End of visstat_core function -------

library(testthat)

# Helper function to suppress graphics during tests to avoid opening windows
suppress_graphics <- function(code) {
  pdf(NULL)
  on.exit(if (dev.cur() > 1) dev.off())
  force(code)
}

# =============================================================================
# 1. TEST NUMERIC VS 2-LEVEL FACTOR (The main decision pipeline)
# =============================================================================

test_that("visstat_core - Numeric vs 2-level factor: Path and Diagnostics", {
  
  # 1.1: Large samples - parametric path
  set.seed(123)
  n_large <- 110
  large_df <- data.frame(
    val = c(rnorm(n_large, 10, 2), rnorm(n_large, 12, 2)),
    grp = factor(rep(c("G1", "G2"), each = n_large))
  )
  
  res_large <- suppress_graphics(visstat_core(large_df, "val", "grp"))
  
  # Verify it chose the parametric path
  expect_true("t-test-statistics" %in% names(res_large))
  
  # 1.2: Small samples with skewed residuals trigger Wilcoxon
  set.seed(123)
  skew_df <- data.frame(
    val = c(rexp(25, 0.1), rexp(25, 0.5)),
    grp = factor(rep(c("A", "B"), each = 25))
  )
  
  res_skew <- suppress_graphics(visstat_core(skew_df, "val", "grp"))
  
  # Verify it chose non-parametric path
  expect_true("statsWilcoxon" %in% names(res_skew))
})

# =============================================================================
# 2. TEST MULTI-LEVEL FACTOR (ANOVA/Kruskal)
# =============================================================================



# =============================================================================
# 3. TEST ERROR HANDLING (Original Logic Verification)
# =============================================================================

test_that("visstat_core - Error Handling for insufficient data", {
  
  # 3.1: Total samples < 3
  tiny_df <- data.frame(val = c(1, 2), grp = factor(c("A", "B")))
  res_err <- suppress_graphics(visstat_core(tiny_df, "val", "grp"))
  
  expect_equal(res_err$error, "Insufficient data")
  
  # 3.2: Empty factor levels
  empty_df <- data.frame(
    val = rnorm(10),
    grp = factor(rep("A", 10), levels = c("A", "B"))
  )
  res_empty <- suppress_graphics(visstat_core(empty_df, "val", "grp"))
  
  expect_equal(res_empty$error, "Insufficient data")
})

#' Print method for visstat objects
#'
#' Displays a brief summary of the statistical test results and, if
#' available, assumption tests and post hoc comparisons.
#'
#' @param x An object of class \code{"visstat"}, returned by
#'   \code{visstat()}.
#' @param ... Currently unused. Included for S3 method compatibility.
#'
#' @return The object \code{x}, invisibly.
#'
#' @details Quick overview of the statistical analysis results.
#'
#' @examples
#' anova <- visstat(npk$block, npk$yield)
#' print(anova)
#'
#' @seealso \code{\link{summary.visstat}},
#'   \code{\link{plot.visstat}}, \code{\link{visstat}}
#'
#' @export

print.visstat <- function(x, ...) {
  
  cat("Object of class 'visstat'\n")
  
  if (is.list(x)) {
    
    if (!is.null(x$test) && !is.null(x$test$method)) {
      cat("Test used:", x$test$method, "\n")
    }
    
    if (!is.null(x$test) && !is.null(x$test$p.value)) {
      cat("p-value:", signif(x$test$p.value, 3), "\n")
    }
    
    cat("\nAvailable components:\n")
    print(names(x))
    
  } else {
    
    cat("(No named elements in object)\n")
    
  }
  
  invisible(x)
  
}


#' Summary method for visstat objects
#'
#' Displays the full statistical test results and, if available,
#' assumption tests and post hoc comparisons.
#'
#' @param object An object of class \code{"visstat"}, returned by
#'   \code{visstat()}.
#' @param ... Currently unused. Included for S3 method compatibility.
#'
#' @return The object \code{object}, invisibly.
#'
#' @details This method provides a full textual report of the statistical
#' test results returned by \code{visstat()}, and prints the contents of
#' \code{posthoc_summary} if present.
#'
#' @seealso \code{\link{print.visstat}}, \code{\link{visstat}}
#'
#' @examples
#' anova <- visstat(npk$block, npk$yield)
#' summary(anova)
#'
#' @export

summary.visstat <- function(object, ...) {
  
  cat("Summary of visstat object\n\n")
  
  cat("--- Named components ---\n")
  print(names(object))
  
  cat("\n--- Contents ---\n")
  
  for (name in names(object)) {
    
    cat(paste0("\n$", name, ":\n"))
    
    if (name == "captured_plots") {
      
      .print_captured_plots_summary(object[[name]])
      
    } else {
      
      print(object[[name]])
      
    }
    
  }
  
  invisible(object)
  
}


# Helper function for captured plots in print()

.print_captured_plots <- function(plots) {
  
  if (is.list(plots) && length(plots) > 0) {
    
    cat(sprintf("List of %d plot object(s)\n", length(plots)))
    
  } else {
    
    print(plots)
    
  }
  
}


# Helper function for captured plots in summary()

.print_captured_plots_summary <- function(plots) {
  
  if (is.list(plots) && length(plots) > 0) {
    
    cat(sprintf("List of %d plot object(s)\n", length(plots)))
    
  } else if (is.list(plots) && length(plots) == 0) {
    
    cat("Empty plot list\n")
    
  } else {
    
    cat("Plot object(s) (not displayed in summary)\n")
    
  }
  
}


#' Plot method for visstat objects
#'
#' Replays captured plots or reports saved plot file paths from a
#' \code{visstat} object.
#'
#' When called without \code{which}, the method lists all available plots
#' (either as file paths or as indices of captured plots).
#' When called with \code{which}, the selected plot is displayed:
#' for file-based output the stored path is printed, for captured plots
#' the plot is replayed via \code{replayPlot()}.
#'
#' @param x An object of class \code{"visstat"}, returned by
#'   \code{\link{visstat}()}.
#' @param which Integer selecting a single plot to display (1, 2, ...).
#'   If \code{NULL} (the default), all available plots are listed without
#'   being drawn.
#' @param ... Currently unused. Included for S3 method compatibility.
#'
#' @return The object \code{x}, invisibly. Called for its side effect.
#'
#' @seealso \code{\link{print.visstat}},
#'   \code{\link{summary.visstat}}, \code{\link{visstat}}
#'
#' @examples
#' # File-based output: plot() lists stored paths
#' anova_path <- visstat(
#'   npk$block,
#'   npk$yield,
#'   graphicsoutput = "png",
#'   plotDirectory = tempdir()
#' )
#'
#' plot(anova_path)
#'
#' # Interactive output: plot() lists available plots,
#' # plot(obj, which = n) replays a specific one
#' linreg <- visstat(trees$Height, trees$Girth)
#'
#' plot(linreg)
#' plot(linreg, which = 2)
#'
#' @export

plot.visstat <- function(x, which = NULL, ...) {
  
  path <- attr(x, "plot_paths")
  capture <- attr(x, "captured_plots")
  
  # File-based plots ---------------------------------------------------
  
  if (!is.null(path) && length(path) > 0) {
    
    if (!is.null(which)) {
      
      message("Plot [", which, "] stored in ", path[[which]])
      
    } else {
      
      for (i in seq_along(path)) {
        
        message("Plot [", i, "] stored in ", path[[i]])
        
      }
      
    }
    
    return(invisible(x))
    
  }
  
  # Captured plots -----------------------------------------------------
  
  if (!is.null(capture) && length(capture) > 0) {
    
    if (!is.null(which)) {
      
      oldpar <- graphics::par(no.readonly = TRUE)
      
      on.exit(graphics::par(oldpar), add = TRUE)
      
      replayPlot(capture[[which]])
      
    } else {
      
      for (i in seq_along(capture)) {
        
        message(
          "Plot [", i,
          "] captured. Use plot(obj, which = ", i,
          ") to display."
        )
        
      }
      
    }
    
    return(invisible(x))
    
  }
  
  invisible(x)
  
}

library(testthat)

# Test suite for visstat_methods.R S3 methods
# Assumes the methods print.visstat, summary.visstat, and plot.visstat are loaded

# Helper function to create mock visstat objects for testing
create_mock_visstat <- function(type = "basic") {
  if (type == "basic") {
    obj <- list(
      test = list(
        method = "One-way ANOVA",
        p.value = 0.0234,
        statistic = 4.567,
        data.name = "test data"
      ),
      assumptions = list(
        normality = list(method = "Shapiro-Wilk", p.value = 0.123),
        homogeneity = list(method = "Levene's Test", p.value = 0.456)
      )
    )
  } else if (type == "empty") {
    obj <- list()
  } else if (type == "no_test") {
    obj <- list(
      data = data.frame(x = 1:5, y = 6:10),
      notes = "No statistical test performed"
    )
  } else if (type == "with_plots") {
    obj <- list(
      test = list(
        method = "Kruskal-Wallis test",
        p.value = 0.001
      ),
      captured_plots = list("plot1", "plot2")  # Mock plot objects
    )
  }
  
  class(obj) <- "visstat"
  return(obj)
}

# Tests for print.visstat method
test_that("print.visstat displays basic information correctly", {
  obj <- create_mock_visstat("basic")
  
  # Capture output
  output <- capture.output(result <- print(obj))
  
  # Check return value (should return object invisibly)
  expect_identical(result, obj)
  
  # Check output content
  expect_true(any(grepl("Object of class 'visstat'", output)))
  expect_true(any(grepl("Test used: One-way ANOVA", output)))
  expect_true(any(grepl("p-value: 0.0234", output)))
  expect_true(any(grepl("Available components:", output)))
})

test_that("print.visstat handles empty visstat objects", {
  obj <- create_mock_visstat("empty")
  
  output <- capture.output(result <- print(obj))
  
  expect_identical(result, obj)
  expect_true(any(grepl("Object of class 'visstat'", output)))
  expect_true(any(grepl("Available components:", output)))
})

test_that("print.visstat handles objects without test method", {
  obj <- create_mock_visstat("no_test")
  
  output <- capture.output(result <- print(obj))
  
  expect_identical(result, obj)
  expect_true(any(grepl("Object of class 'visstat'", output)))
  expect_true(any(grepl("Available components:", output)))
  # Should not show test method or p-value lines
  expect_false(any(grepl("Test used:", output)))
  expect_false(any(grepl("p-value:", output)))
})

test_that("print.visstat handles non-list objects", {
  obj <- "not a list"
  class(obj) <- "visstat"
  
  output <- capture.output(result <- print(obj))
  
  expect_identical(result, obj)
  expect_true(any(grepl("Object of class 'visstat'", output)))
  expect_true(any(grepl("No named elements in object", output)))
})

# Tests for summary.visstat method
test_that("summary.visstat displays comprehensive information", {
  obj <- create_mock_visstat("basic")
  
  output <- capture.output(result <- summary(obj))
  
  # Check return value
  expect_identical(result, obj)
  
  # Check output structure
  expect_true(any(grepl("Summary of visstat object", output)))
  expect_true(any(grepl("--- Named components ---", output)))
  expect_true(any(grepl("--- Contents ---", output)))
  
  # Check that all components are shown
  expect_true(any(grepl("\\$test:", output)))
  expect_true(any(grepl("\\$assumptions:", output)))
})

test_that("summary.visstat handles captured_plots specially", {
  obj <- create_mock_visstat("with_plots")
  
  output <- capture.output(result <- summary(obj))
  
  expect_identical(result, obj)
  expect_true(any(grepl("\\$captured_plots:", output)))
  expect_true(any(grepl("List of 2 plot object", output)))
})

test_that("summary.visstat handles empty objects", {
  obj <- create_mock_visstat("empty")
  
  output <- capture.output(result <- summary(obj))
  
  expect_identical(result, obj)
  expect_true(any(grepl("Summary of visstat object", output)))
})

# Tests for helper functions
test_that(".print_captured_plots_summary works correctly", {
  # Test with non-empty list
  plots <- list("plot1", "plot2", "plot3")
  output <- capture.output(visStatistics:::.print_captured_plots_summary(plots))
  expect_true(any(grepl("List of 3 plot object", output)))
  
  # Test with empty list
  empty_plots <- list()
  output <- capture.output(visStatistics:::.print_captured_plots_summary(empty_plots))
  expect_true(any(grepl("Empty plot list", output)))
  
  # Test with non-list object
  non_list <- "not a list"
  output <- capture.output(visStatistics:::.print_captured_plots_summary(non_list))
  expect_true(any(grepl("Plot object.*not displayed", output)))
})

# Tests for plot.visstat method
test_that("plot.visstat handles plot_paths attribute", {
  obj <- create_mock_visstat("basic")
  temp_dir <- tempdir()
  plot_paths <- c(
    file.path(temp_dir, "plot1.png"),
    file.path(temp_dir, "plot2.png")
  )
  attr(obj, "plot_paths") <- plot_paths
  
  # Test showing all plots
  output <- capture.output(result <- plot(obj), type = "message")
  
  expect_identical(result, obj)
  expect_true(any(grepl("Plot \\[1\\] stored in", output)))
  expect_true(any(grepl("Plot \\[2\\] stored in", output)))
})

test_that("plot.visstat handles specific plot selection with paths", {
  obj <- create_mock_visstat("basic")
  temp_dir <- tempdir()
  plot_paths <- c(
    file.path(temp_dir, "plot1.png"),
    file.path(temp_dir, "plot2.png")
  )
  attr(obj, "plot_paths") <- plot_paths
  
  # Test showing specific plot
  output <- capture.output(result <- plot(obj, which = 1), type = "message")
  
  expect_identical(result, obj)
  expect_true(any(grepl("Plot \\[1\\] stored in", output)))
  expect_false(any(grepl("Plot \\[2\\] stored in", output)))
})

test_that("plot.visstat handles captured_plots attribute", {
  skip_if_not_installed("grDevices")

  obj <- create_mock_visstat("basic")

  # Create mock recorded plots
  # Note: This is tricky to test without actually creating plots
  # We'll test the structure but skip the actual replayPlot calls

  mock_plots <- list()
  # Create a simple plot and record it
  plot(1:5, 1:5)
  mock_plots[[1]] <- recordPlot()
  plot(1:3, 1:3)
  mock_plots[[2]] <- recordPlot()
  dev.off()

  attr(obj, "captured_plots") <- mock_plots

  # Test that the function runs and displays first plot with helpful messages
  # When multiple plots are available, show guidance to user
  expect_message(result <- plot(obj), "captured. Use plot")
  expect_identical(result, obj)
})

test_that("plot.visstat handles specific captured plot selection", {
  skip_if_not_installed("grDevices")
  
  obj <- create_mock_visstat("basic")
  
  # Create mock recorded plots
  mock_plots <- list()
  plot(1:5, 1:5)
  mock_plots[[1]] <- recordPlot()
  dev.off()
  
  attr(obj, "captured_plots") <- mock_plots
  
  # Test specific plot selection
  expect_silent(result <- plot(obj, which = 1))
  expect_identical(result, obj)
})

test_that("plot.visstat handles objects with no plots", {
  obj <- create_mock_visstat("basic")
  
  # No plot_paths or captured_plots attributes
  expect_silent(result <- plot(obj))
  expect_identical(result, obj)
})

test_that("plot.visstat handles empty plot_paths", {
  obj <- create_mock_visstat("basic")
  attr(obj, "plot_paths") <- character(0)
  
  expect_silent(result <- plot(obj))
  expect_identical(result, obj)
})

test_that("plot.visstat handles empty captured_plots", {
  obj <- create_mock_visstat("basic")
  attr(obj, "captured_plots") <- list()
  
  expect_silent(result <- plot(obj))
  expect_identical(result, obj)
})

# Integration tests
test_that("S3 methods work with realistic visstat objects", {
  # Create a more realistic visstat object
  realistic_obj <- list(
    test = list(
      method = "Kruskal-Wallis rank sum test",
      statistic = 5.678,
      parameter = 2,
      p.value = 0.0234,
      data.name = "y by x"
    ),
    assumptions = list(
      normality = list(
        method = "Shapiro-Wilk normality test",
        p.value = 0.001,
        significant = TRUE
      )
    ),
    posthoc = list(
      method = "Dunn's test",
      results = data.frame(
        comparison = c("A-B", "A-C", "B-C"),
        p.value = c(0.01, 0.05, 0.8)
      )
    ),
    data_summary = list(
      groups = c("A", "B", "C"),
      n = c(10, 12, 15)
    )
  )
  class(realistic_obj) <- "visstat"
  
  # Test all methods work (these methods are supposed to produce output)
  expect_output(print_result <- print(realistic_obj), "Object of class 'visstat'")
  expect_output(summary_result <- summary(realistic_obj), "Summary of visstat object")
  expect_silent(plot_result <- plot(realistic_obj))  # plot should be silent when no plots
  
  # Check return values
  expect_identical(print_result, realistic_obj)
  expect_identical(summary_result, realistic_obj)
  expect_identical(plot_result, realistic_obj)
})

test_that("Methods handle edge cases gracefully", {
  # Test with empty list (can't set class on NULL)
  empty_obj <- list()
  class(empty_obj) <- "visstat"
  
  # These should not error (though behavior may vary)
  expect_no_error(print(empty_obj))
  expect_no_error(summary(empty_obj))
  expect_no_error(plot(empty_obj))
  
  # Test with minimal object
  minimal_obj <- list(test = list(p.value = 0.05))
  class(minimal_obj) <- "visstat"
  
  expect_no_error(print(minimal_obj))
  expect_no_error(summary(minimal_obj))
  expect_no_error(plot(minimal_obj))
  
  # Test with object that has no test component
  no_test_obj <- list(data = "some data")
  class(no_test_obj) <- "visstat"
  
  expect_no_error(print(no_test_obj))
  expect_no_error(summary(no_test_obj))
  expect_no_error(plot(no_test_obj))
})

test_that("print.visstat p-value formatting works correctly", {
  # Test different p-value formats
  test_cases <- list(
    list(p.value = 0.001234, expected = "0.00123"),
    list(p.value = 0.05, expected = "0.05"),
    list(p.value = 0.999, expected = "0.999"),
    list(p.value = 1e-10, expected = "1e-10")
  )
  
  for(case in test_cases) {
    obj <- list(test = list(method = "Test", p.value = case$p.value))
    class(obj) <- "visstat"
    
    output <- capture.output(print(obj))
    p_line <- output[grepl("p-value:", output)]
    expect_true(length(p_line) > 0, info = paste("Failed for p-value:", case$p.value))
  }
})

test_that("Method inheritance works correctly", {
  obj <- create_mock_visstat("basic")
  
  # Test that S3 dispatch works
  expect_true(is.function(print.visstat))
  expect_true(is.function(summary.visstat))
  expect_true(is.function(plot.visstat))
  
  # Test that methods are called for visstat objects
  expect_identical(class(obj), "visstat")
  
  # These should call the visstat methods, not defaults
  expect_no_error(print(obj))
  expect_no_error(summary(obj))
  expect_no_error(plot(obj))
})

#' Test normality using pooled standardized residuals
#'
#' Standardizes values within each group and applies a single normality 
#' test to the pooled standardized values. This avoids multiple testing 
#' issues when deciding between parametric and non-parametric methods.
#'
#' @param y Numeric vector of response values
#' @param g Factor or vector defining groups (must have at least 2 levels)
#' @param test Character, either "shapiro" or "ad" for Anderson-Darling
#' @param min_n Minimum sample size per group required (default 3)
#' @return A list with test results (statistic, p.value, method, data.name)

pooled_normality_test <- function(y, g, 
                                  test = c("shapiro", "ad"),
                                  min_n = 3L) {
  test <- match.arg(test)
  g <- as.factor(g)
  
  # Ensure y and g are the same length and handle NAs upfront
  complete_cases <- !is.na(y) & !is.na(g)
  y <- y[complete_cases]
  g <- g[complete_cases]
  
  if (nlevels(g) < 2L) {
    stop("At least 2 groups required for normality test")
  }
  
  # Within-group standardization: z_ij = (y_ij - mean_i) / sd_i
  # ave() keeps the original order and is more concise
  z <- ave(y, g, FUN = function(x) {
    n <- length(x)
    if (n < min_n) return(rep(NA_real_, n))
    # Scale center = TRUE, scale = TRUE performs (x - mean)/sd
    as.vector(scale(x))
  })
  
  # Remove NAs from groups that didn't meet min_n
  z <- z[!is.na(z)]
  
  # Sample size checks
  n_total <- length(z)
  if (n_total < min_n) {
    warning(sprintf("Insufficient data: only %d values available", n_total))
    return(list(statistic = NA, p.value = NA, method = "Insufficient data", data.name = "z"))
  }
  
  # Automated fallback for Shapiro-Wilk limit
  if (test == "shapiro" && n_total > 5000L) {
    warning("Sample size > 5000; falling back to Anderson-Darling")
    test <- "ad"
  }
  
  # Execute tests
  res <- switch(test,
                shapiro = shapiro.test(z),
                ad = {
                  if (!requireNamespace("nortest", quietly = TRUE)) {
                    stop("Package 'nortest' required for Anderson-Darling test")
                  }
                  nortest::ad.test(z)
                }
  )
  
  res$data.name <- "pooled standardized residuals"
  return(res)
}

check_visstat_input <- function(x, y=NULL, ...,data=NULL) {
  dots <- list(...)
  dot_names <- names(dots)
  # Extract unnamed arguments: if no names at all, all are unnamed
  # If names exist, unnamed ones have empty string as name
  if (is.null(dot_names)) {
    unnamed <- dots
  } else {
    unnamed <- dots[dot_names == ""]
  }
  total_positional <- 2 + length(unnamed)
  
  
  
  allowed_classes <- c("numeric", "integer", "factor", "character")
  
  
  # Case 0: Formula interface - visstat(y ~ x, data = dataframe)
  if (inherits(x, "formula")) {
    if (is.null(data)) {
      stop("Formula input requires a 'data' argument.")
    }
    
    vars <- all.vars(x)
    if (length(vars) != 2) {
      stop("Formula must be of the form 'y ~ x'.")
    }
    
    yvar <- vars[1]
    xvar <- vars[2]
    
    # Check if columns exist in dataframe
    missing_cols <- vars[!vars %in% names(data)]
    if (length(missing_cols) > 0) {
      stop(paste("Column(s) not found in dataframe:", paste(missing_cols, collapse = ", ")))
    }
    
    # Check if columns have allowed classes
    for (col in vars) {
      if (!inherits(data[[col]], allowed_classes)) {
        stop(paste("Column", col, "must be of class:", paste(allowed_classes, collapse = ", ")))
      }
    }
    
    return(invisible(TRUE))  # valid formula input
  }
  
  # Case 1: Formula interface - visstat(y ~ x, data = dataframe)
  # Note: Formula validation is handled in the main visstat() function
  # before calling check_visstat_input, so we don't need to check it here
  
  # Case 2: Standard form - visstat(x_vector, y_vector)
  if (total_positional == 2 &&
      inherits(x, allowed_classes) &&
      inherits(y, allowed_classes)) {
    return(invisible(TRUE))  # valid vector input
  }
  
  # Case 3: Legacy form - visstat(dataframe, "y_column", "x_column")
  if (total_positional == 3 &&
      is.data.frame(x) &&
      is.character(y) &&
      length(y) == 1 &&
      length(unnamed) > 0 &&
      is.character(unnamed[[1]]) &&
      length(unnamed[[1]]) == 1) {
    
    # Check if columns exist in the dataframe
    columns <- c(y, unnamed[[1]])
    missing_cols <- columns[!columns %in% names(x)]
    if (length(missing_cols) > 0) {
      stop(paste("Column(s) not found in dataframe:", paste(missing_cols, collapse = ", ")))
    }
    
    # Check if columns have allowed classes
    for (col in columns) {
      if (!inherits(x[[col]], allowed_classes)) {
        stop(paste("Column", col, "must be of class:", paste(allowed_classes, collapse = ", ")))
      }
    }
    
    return(invisible(TRUE))  # valid data.frame input
  }
  
  # If none of the valid cases match, provide helpful error message
  stop("Invalid input. Use one of the following forms:
       1. visstat(y ~ x, data = dataframe)  [Formula handled before this function]
       2. visstat(x_vector, y_vector)
       3. visstat(dataframe, 'y_column', 'x_column')
       
       All variables must be of class: ", paste(allowed_classes, collapse = ", "))
}

# Internal helper function in visstat----
#' Selects columns defined by characters varsample and varfactor from
#' a data.frame
#'
#' Selects columns defined by characters \code{varsample} and \code{varfactor}
#' from \code{dataframe}, returns selected columns with their names.
#' @param dataframe \code{data.frame} or \code{list} containing at least two
#' columns with column headings of data type \code{character}.Data must be
#' column wise ordered.
#' @param varsample column name of dependent variable in dataframe,
#' datatype \code{character}
#' @param varfactor column name of independent variable in dataframe, datatype
#' \code{character}
#' @return selected columns, \code{sample}, \code{factor}, \code{name_of_sample}
#' (character string equaling varsample), \code{name_of_factor}
#' (character string equaling varsample)
#'
#' @examples
#' get_samples_fact_inputfile(trees, "Girth", "Height")
#' @noRd

get_samples_fact_inputfile <- function(dataframe, varsample, varfactor) {
  # json input------
  if (is.null(dim(dataframe)))
  # FALSE for csv
    {
      fulldata <- dataframe
      data <- dataframe$data
      data <- as.data.frame(data)

      if ("matching" %in% names(dataframe) & varfactor == "match") {
        matched_selected_group0 <- which(data$group0 == 1 & data$match == 1)
        matched_selected_group1 <- which(data$group1 == 1 &
          data$match == 1)
        fact <- c(
          rep(fulldata$group0name, length(matched_selected_group0)),
          rep(fulldata$group1name, length(matched_selected_group1))
        )
        fact <- as.factor(fact)
        fullsample <- data[, varsample]

        samples <- fullsample[c(matched_selected_group0, matched_selected_group1)]
        name_of_factor <- paste(fulldata$group0name, "and", fulldata$group1name)
        # name_of_factor="groups"
        name_of_sample <- varsample
        # does not work on multiple matching criterias:
        # matchingCriteria=paste(tolower(paste(as.character(dataframe$matching),collapse =" ")),sep="")
        matchingCriteria <- tolower(paste(apply(dataframe$matching, 1, function(x) {
          paste(x, collapse = " ")
        }), collapse = ", "))
        name_of_sample <- paste(name_of_sample, "with match:", matchingCriteria)
        # json file with no matching criterion
      } else if ("matching" %in% names(dataframe) &
        varfactor != "match") {
        samples <- data[, varsample]
        fact <- data[, varfactor]
        name_of_sample <- varsample
        name_of_factor <- varfactor
        matchingCriteria <- ""
      } else {
        stop("code runs only on json files generated by Web server")
      }
      # csv input------
    } else {
    # Select samples and fact from data.frame dataframe-----
    samples <- dataframe[, varsample]
    fact <- dataframe[, varfactor]
    name_of_sample <- varsample
    name_of_factor <- varfactor
    matching_criteria <- ""
  }


  # samples are the two groups selected by the user in groups at the web surface
  mylist <- list(
    "samples" = samples,
    "fact" = fact,
    "name_of_sample" = name_of_sample,
    "name_of_factor" = name_of_factor,
    "matching_criteria" = matching_criteria
  )
  return(mylist)
}

---
title: 'visStatistics: The right test, visualised'
author:
- name: Sabine Schilling
  affiliation: Institute of Tourism and Mobility, Lucerne University of Applied Sciences
    and Arts
  email: sabineschilling@gmx.ch
date: "`r Sys.Date()`"
output:
 rmarkdown::html_vignette:
 # pdf_document:
    toc: true
    number_sections: true
    toc_depth: 4
    fig_caption: true
link-citations: true
bibliography: REFERENCES.bib
#csl: ims.csl
vignette: >
  %\VignetteIndexEntry{visStatistics: The right test, visualised}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
editor_options:
  markdown:
    wrap: sentence
---

```{r setup, include=FALSE}
library(visStatistics)
  knitr::opts_chunk$set(
    fig.width = 7.5,      # Smaller device = smaller text
    fig.height = 5,
    out.width = '100%')
```

# Abstract

`visStatistics` automatically selects and visualises appropriate statistical tests between two column vectors of class `"numeric"`, `"integer"`, or `"factor"`.
No manual test selection is required: the function determines the appropriate method from the data types and distributional properties alone.
The choice of test depends on the `class`, distributional assumptions, and sample size of the vectors.

The main function `visstat()` visualises the selected test with appropriate graphs (box plots, bar charts, regression lines with confidence bands, mosaic plots for Pearson's $\chi^2$ test, residual plots), annotated with the main test results, including visualisations of the assumption checks and post-hoc analyses.

This scripted workflow is well suited for browser-based applications, where users interact only through a web interface, while server-side R applications handle the data processing.

Other typical use cases include quick visualisations, and guided statistical test selection, for example in statistical consulting or educational settings.

# Introduction

Most routine data analyses reduce to a comparatively small set of inferential frameworks, including group comparisons, contingency-table analyses, and regression models [@Hayat:2017, @Sato:2017].
`visStatistics` selects out of these most commonly used tests “right test” by using a reproducible decision framework based on the data’s distributional assumptions and sample size.
It visualises its decision by, where appropriate, assumption-diagnostic plot and a descriptive plot with the main test statistics annotated, and returns an R object whose `print()` and `summary()` methods expose the complete test results.
The scripted workflow is well suited for browser-based applications where sensitive data (such as highly confidential medical records) is stored securely on a server and can not be directly accessed by users.
This approach was already successfully applied to develop a medical scoring tool [@Bijlenga:2017].
When selecting tests for group comparisons, packages with overlapping scope such as `automatedtests` [@Zeevat:2025] and `compareGroups` [@Subirana:2014] base their test selection on group‑wise normality assessments of the response variable. But group-wise testing inflates the [family-wise type I error rate](#post-hoc-analysis): the probability of at least one false rejection of normality grows with the number of compared groups, leading to unnecessary switching to non-parametric methods even when the linear model assumptions hold globally.
In contrast, `visStatistics` assesses the normality of the overall model residuals, adhering to the general linear model framework [@Searle:1971].

# Getting started

#### 1. Install the latest development version from GITHUB {.unnumbered}

```{r install-github, eval = FALSE}
install_github("shhschilling/visStatistics")
```

#### 2. Load the package {.unnumbered}

```{r load, eval = FALSE}
library(visStatistics)
```

#### 3. Minimal function call {.unnumbered}

The function `visstat()` accepts input in three ways:

``` r
# Standardised form (recommended):
visstat(x, y)

# Formula interface:
visstat(y ~ x, data = dataframe)

# Backward-compatible form:
visstat(dataframe, "namey", "namex")
```

`x` and `y` must be vectors of class `"numeric"`, `"integer"`, or `"factor"`.

In the formula interface, `y ~ x` specifies the relationship, where `y` is the response variable and `x` is the predictor, with both being column names in `dataframe`.

In the backward-compatible form, `"namex"` and `"namey"` must be character strings naming columns in `dataframe`, which must themselves be of class `"numeric"`, `"integer"`, or `"factor"`.
Note that in the backward-compatible form the second argument belongs to the response, the third to the predictor.
This is equivalent to writing:

``` r
visstat(dataframe[["namex"]], dataframe[["namey"]])
```

##### Exemplary function call {.unnumbered}

```{r npk, fig.show='hide'}
#Standardised form
visstat(npk$block,npk$yield)
# Using formula interface
 visstat(yield~block,data=npk)
#Backward-compatible form
 visstat(npk,"yield","block")
```

# Decision logic

Throughout the remainder, data of class `"numeric"` or `"integer"` are referred to by their common `mode` `numeric`, while data of class `"factor"` are referred to as categorical.
The significance level $\alpha$, used throughout for hypothesis testing, is defined as `1 - conf.level`, where `conf.level` is a user-controllable argument (defaulting to `0.95`).

The choice of statistical tests performed by the function `visstat()` depends on whether the data are numeric or categorical, the number of levels in the categorical variable, the distribution of the data, as well as the user-defined 'conf.level'.
The common mathematical framework underlying Student's t-test, Fisher's ANOVA and simple linear regression is described in the [Appendix A](#glm).

The function prioritizes interpretable visual output and tests that remain valid under the following decision logic.
The following graph gives an overview of all implemented tests based on their `class`:

```{r fig-overview, echo=FALSE, results='asis'}
if (knitr::opts_knit$get("rmarkdown.pandoc.to") == "html") {
  cat('
<div style="border: 1px solid #666; padding: 10px; display: inline-block; text-align: center;">
  <img src="figures/overview.png" width="100%"
       alt="Overview of implemented tests .">
  <p style="font-style: italic; font-size: 90%; margin-top: 0.5em;">
    Overview of the implemented statistical tests based on the class of the variables.
  </p>
</div>
')
} else {
  cat('
\\begin{center}
\\fbox{%
  \\begin{minipage}{0.95\\linewidth}
    \\centering
    \\includegraphics[width=\\linewidth]{figures/overview.png}\\\\
    \\vspace{0.5em}
    \\textit{Decision tree used to select the appropriate statistical test for a categorical predictor and numeric response, based on the number of factor levels, normality, and homoscedasticity.}
  \\end{minipage}
}
\\end{center}
')
}
```

A graphical summary of the decision logic used for numerical responses and categorical predictors resulting in comparisons of central tendencies is given in the figure below.

```{r fig-decision-switch, echo=FALSE, results='asis'}
if (knitr::opts_knit$get("rmarkdown.pandoc.to") == "html") {
  cat('
<div style="border: 1px solid #666; padding: 10px; display: inline-block; text-align: center;">
  <img src="figures/decision_tree.png" width="100%"
       alt="Decision tree used to select the appropriate statistical test.">
  <p style="font-style: italic; font-size: 90%; margin-top: 0.5em;">
    Decision tree used to select the appropriate statistical test for a categorical
    predictor and numeric response, based on the sample sizes, number of factor levels, normality,
    and homoscedasticity.
  </p>
</div>
')
} else {
  cat('
\\begin{center}
\\fbox{%
  \\begin{minipage}{0.95\\linewidth}
    \\centering
    \\includegraphics[width=\\linewidth]{figures/decision_tree.png}\\\\
    \\vspace{0.5em}
    \\textit{Decision tree used to select the appropriate statistical test for a categorical predictor and numeric response, based on the sample sizes, number of factor levels, normality, and homoscedasticity.}
  \\end{minipage}
}
\\end{center}
')
}
```

## Numeric response and categorical predictor: Comparing central tendencies

When the response `y` is numeric and the predictor `x` is categorical, a statistical hypothesis test comparing central tendencies is selected.
To check whether the assumptions of a general linear model (see [Appendix A](#glm)) are fulfilled, a linear model `lm(y ~ x)` is first fitted.

- Normality testing of the residuals: The Shapiro--Wilk test [@Shapiro:1965] (`shapiro.test()`) is performed on internally studentised residuals computed by `rstandard()`, which scales the raw residuals with an estimate of their standard deviation that accounts for the leverage of each observation [@Cook:1982].
  The Shapiro--Wilk (SW) test is the most powerful for detecting non-normality across most distributions, especially with smaller sample sizes [@Razali:2011; @Ghasemi:2012].

- Non parametric-tests: Only when SW the test rejects normality ($p_{SW} \le \alpha$), non-parametric tests are selected: the Wilcoxon rank-sum test (`wilcox.test()`) for two groups, or the Kruskal--Wallis test (`kruskal.test()`) followed by the Holm adjusted pairwise Wilcoxon tests (`pairwise.wilcox.test()`) for more than two groups.

- Parametric tests: When there is insufficient evidence against normality ($p_{SW} > \alpha$) or when the sample sizes are large (more than 50 observations per group), parametric tests are selected.
  In large samples the central limit theorem ensures approximate normality of the sampling distribution of the mean.
  [@Rasch:2011; @Lumley:2002; @Kwak:2017]

  - Homoscedasticity: The Levene–Brown–Forsythe (L) test (implemented as `levene.test()`) [@Brown:1974] tests for homogeneous variances.
    - When homogeneity is not rejected ($p_L > \alpha$): For two groups, Student's t-test (`t.test()` with `var.equal = TRUE`) is applied; for more than two groups, Fisher's one-way ANOVA (`aov()`) with Tukey's Honestly Significant Differences (HSD) (`TukeyHSD()`) [@Hochberg:1987] is applied.

    - When homogeneity is rejected ($p_L \le \alpha$): For two groups, Welch's t-test (`t.test()` with `var.equal = FALSE`) is applied; for more than two groups, Welch's heteroscedastic one-way ANOVA (`oneway.test()`) with Games-Howell post-hoc test (`games.howell()`) [@Games:1976] is applied.
      Welch's methods outperform their classical counterparts when variances differ [@Moser:1992; @Fagerland:2009; @Delacre:2017].

- Regardless of sample size, assumption diagnostics are always displayed.
  Throughout this vignette, *linear model* refers to the classical Gaussian linear model framework [@Searle:1971] underlying t-tests, ANOVA, and linear regression.
  When normality of residuals is met but homoscedasticity is rejected, `visstat()` selects Welch methods and additionally displays group-wise normality diagnostics via `vis_group_normality()`, since Welch's variance estimates are group-specific and the pooled `lm()` residuals are no longer the appropriate diagnostic under heteroscedasticity.
  These diagnostic plots enable users to visually assess whether assumptions are met and manually override the automated p-value-based test selection.
  The decision logic is based entirely on p-values and distributional assumptions; it does not assess practical relevance.
  Users should always report and interpret an appropriate effect size measure (e.g. Cohen's $d$ for t-tests, $\omega^2$ for ANOVA, rank-biserial $r$ for Wilcoxon and Kruskal--Wallis tests) alongside the test result.

## Both variables numeric: Simple linear regression (default) or Spearman rank correlation (`correlation = TRUE`)

### Simple linear regression (`lm()`)

By default, `visstat()` always fits a simple linear regression model (`lm()`) for two numeric variables, regardless of whether the GLM assumptions of normality and homoscedasticity are met.
Assumption diagnostics are always shown, enabling the user to assess whether the model is appropriate.
Correlation analysis is never triggered automatically; it requires the explicit flag `correlation = TRUE`.
Note that **only one** predictor variable is allowed, as the function is designed for two-dimensional visualisation.

In the linear regression branch, homoscedasticity is assessed using the Breusch–Pagan test `bp.test()` (see [Appendix B](#variance)), which evaluates whether the variance of the raw residuals depends on the predictor variable.

### Spearman rank correlation (`correlation = TRUE`)

When `correlation = TRUE` is set, `visstat()` uses Spearman's $\rho$ (`cor.test(..., method = "spearman")`) to measure the monotone association between the two numeric variables.
Switching to correlation requires an explicit user choice, because the decision between modelling a directional relationship (regression) and measuring a monotone association (correlation) cannot be derived from the data type alone.
Spearman correlation operates on the ranks of the data rather than the original values, making it robust to outliers and non-normal distributions while detecting monotonic relationships.
Because Spearman's $\rho$ is Pearson's $r$ applied to the ranks, it yields nearly identical results to Pearson correlation when the data are bivariate normal (the assumption of Pearson's inference) but remains valid without any distributional assumptions.
A separate Pearson branch is therefore not implemented.
When regression assumptions are violated, Spearman rank correlation offers a robust, assumption-free alternative — at the cost of causal interpretability.
For alternatives not implemented in this package, see [Limitations](#limitations).

## Both variables of class `factor`

### Comparing proportions (Chi-squared or Fisher's exact test)

When both variables are categorical (and not ordered), no direction is assumed; the order of variables in the function call does not affect the test statistic, but it does influence the graphical output.
For consistency, we continue referring to the variables as *predictor* and *response*.

`visstat()` tests the null hypothesis that the variables are independent using either Pearson’s $\chi^2$ test (`chisq.test()`) or Fisher’s exact test (`fisher.test()`), depending on expected cell counts.
The choice of test is based on Cochran's rule [@Cochran:1954], which advises that the $\chi^2$ approximation is reliable only if no expected cell count is less than 1 and no more than 20 percent of cells have expected counts below 5.

## Response of class `ordered`, predictor of class `factor`

When the response variable is an ordered factor (e.g., Likert scale ratings) and the predictor is categorical, `visstat()` converts the ordered response to integer level codes and redirects the analysis to the non-parametric `wilcox.test()` (predictor with two levels) or `kruskal.test()` (predictor with more than two levels) respectively, as the numeric distances between ordered categories are not necessarily equal and the data cannot be assumed to follow a normal distribution.

## Both variables of class `factor` and `ordered`: Wilcoxon/Kruskal-Wallis (default) or Kendall's $\tau_b$ (`correlation = TRUE`)

When `correlation = FALSE` (default) and both variables are ordered factors, the response is converted to integer level codes and the analysis follows the standard non-parametric path (Wilcoxon or Kruskal--Wallis).
When the research question concerns a monotone trend between both ordered variables rather than group differences, the user should consider switching to `correlation = TRUE`.

### Kendall rank correlation (`correlation = TRUE`)

When `correlation = TRUE` is set, `visstat()` converts both ordered factors to integer level codes, which are used directly as ranks, and tests for a monotone association via Kendall's $\tau_b$ rank correlation [@Kendall:1945; @Agresti:2010].
Kendall's $\tau_b$ is preferred over Spearman's $\rho$ for ordinal data with few levels, where ties are common: $\tau_b$ corrects for ties explicitly, while Spearman's $\rho$ uses only an approximate adjustment [@Agresti:2010, @Xu:2013].
The visualisation is a jittered rank--rank scatter that makes the monotone trend visible, annotated with $\tau_b$ and the $p$-value.

# Assumption diagnostics: `vis_lm_assumptions()` {#assumption-diagnostics-vis_lm_assumptions}

All tests within the linear model framework (see [Appendix A](#glm)) share the same set of assumptions:

- Linearity: The expected value of the response is a linear function of the predictors; assessed by checking for systematic patterns in residual plots.

- Error terms are independent, normally distributed with expectation value 0 and have constant error variance $\sigma^2$

The function `vis_lm_assumptions()` provides a unified visual and statistical framework for validating the linear model assumptions.
It fits a linear model using `aov()` and provides four standard diagnostic plots:

1.  **Histogram and Normal Density**: Displays the distribution of standardised residuals with a red normal density curve overlay to visually assess normality.

2.  **Std. Residuals vs. Fitted**: It shows the standardised residuals against fitted values with a zero-line to detect non-linearity or heteroscedasticity.

3.  **Normal Q-Q Plot**: A theoretical quantile-quantile plot using the standardised residuals to identify deviations from the Gaussian distribution.

4.  If `correlation = FALSE` (regression mode), the **Standardised Residuals vs. Leverage** plot; if `correlation = TRUE` (correlation mode), a **Scale-Location** plot.

## Test for normality and homoscedasticity

The diagnostic plots are enhanced by p-values of tests for normality and homoscedasticity, shown in the title of the plot.

**Normality Assessment**: The function evaluates residual normality using both the Shapiro--Wilk test (`shapiro.test()`) and the Anderson--Darling test (`ad.test()`) [@Gross:2015].
Anderson--Darling is particularly sensitive to tail deviations [@Razali:2011; @Yap:2011].

**Homoscedasticity Assessment**: Variance equality is tested using the Levene-Brown-Forsythe test (`levene.test()`) [@Brown:1974] and Bartlett's test (`bartlett.test()`) in the ANOVA path.

These tests compare the spread of data across different factor levels.

For simple linear regression, the Breusch-Pagan test (`bp.test()`) is implemented.

These three tests have standalone implementations in the package to avoid dependencies on external packages (see [Appendix B](#variance)).

# Implemented tests with examples

For all implemented tests, this section provides mathematical background and references.
We report the definition of the test statistic, its distribution under the null hypothesis, and any relevant assumptions.

## Numeric response and categorical predictor: Comparing central tendencies {#comparing-central-tendencies}

### Categorical predictor with two levels: Welch's t-test and Wilcoxon rank-sum

#### Student's t-test (`t.test(var.equal = TRUE)`)

When `var.equal = TRUE`, R's `t.test()` reports its `method` as `"Two Sample t-test"`; this is the same procedure that is commonly known as Student's t-test, and `visstat()` displays the R wording verbatim in the plot title.

The test statistic for Student's t-test is given by:

$$t = \frac{\bar{x}_1 - \bar{x}_2}{s_p \sqrt{\frac{1}{n_1} + \frac{1}{n_2}}},$$

where $\bar{x}_1$ and $\bar{x}_2$ are the sample means, $n_1$ and $n_2$ are the sample sizes, and $s_p$ is the pooled standard deviation:

$$s_p = \sqrt{\frac{(n_1 - 1)s_1^2 + (n_2 - 1)s_2^2}{n_1 + n_2 - 2}},$$

where $s_1^2$ and $s_2^2$ are the sample variances in the two groups.
The test statistic follows a t-distribution with $\nu = n_1 + n_2 - 2$ degrees of freedom.

#### Welch's t-test (`t.test()`)

Welch's t-test relaxes the homoscedasticity assumption while maintaining the requirements for independent observations and normally distributed residuals.
It evaluates the null hypothesis that the means of two groups are equal without assuming equal variances.

The test statistic is given by [@Welch:1947; @Satterthwaite:1946]

$$t = \frac{\bar{x}_1 - \bar{x}_2}{\sqrt{\frac{s_1^2}{n_1} + \frac{s_2^2}{n_2}}},$$

where $\bar{x}_1$ and $\bar{x}_2$ are the sample means, $s_1^2$ and $s_2^2$ the sample variances, and $n_1$, $n_2$ the sample sizes in the two groups.
The statistic follows a *t*-distribution with degrees of freedom approximated by the Welch-Satterthwaite equation:

$$\nu \approx \frac{
\left( \frac{s_1^2}{n_1} + \frac{s_2^2}{n_2} \right)^2
}{
\frac{(s_1^2 / n_1)^2}{n_1 - 1} + \frac{(s_2^2 / n_2)^2}{n_2 - 1}
}.$$

The resulting p-value is computed from the *t*-distribution with $\nu$ degrees of freedom.

#### Wilcoxon rank-sum test (`wilcox.test()`)

The two-sample Wilcoxon rank-sum test (also known as the Mann-Whitney test) is a non-parametric alternative that does not require the response variable to be approximately normally distributed within each group.
It tests for a difference in location between two independent distributions [@Mann:1947].
If the two groups have distributions that are sufficiently similar in shape and scale, the Wilcoxon rank-sum test can be interpreted as testing whether the medians of the two populations are equal [@Hollander:2014].

The two-level factor variable `x` defines two groups, with sample sizes $n_1$ and $n_2$.
All $N=n_1 + n_2$ observations are pooled and assigned ranks from $1$ to $N$.
Let $W_1$ denote the sum of the ranks assigned to the group $x_1$ corresponding to the first level of `x` containing $n_1$ observations:

$$W_{1}= \sum_{i=1}^{n_1} R(x_{1,i})$$,

where $R(x_{1,i})$ is the rank of observation $x_{1,i}$ in the pooled sample.

The test statistic $W$ returned by `wilcox.test()` is then computed as

$$W =U_{1}=W_{1} - \frac{n_1(n_1 + 1)}{2}.$$ It corresponds to the Mann-Whitney [@Mann:1947] $U$ statistic of the first group.

If both groups contain fewer than 50 observations and the data contain no ties, the *p*-value is computed exactly.
Otherwise, a normal approximation with continuity correction is used.

The function returns a list containing the results of the applied test and the summary statistics used to construct the plot.

### Examples

#### Welch's t-test {.unnumbered}

The *Motor Trend Car Road Tests* dataset (`mtcars`) contains 32 observations, where `mpg` denotes miles per (US) gallon, and `am` represents the transmission type (`0` = automatic, `1` = manual).

```{r}
mtcars$am <- as.factor(mtcars$am)
t_test_statistics <- visstat(mtcars$am, mtcars$mpg)
```

#### Wilcoxon rank sum test {.unnumbered}

The Wilcoxon rank sum test is exemplified on differences between the central tendencies of grades of "boys" and "girls" in a class:

```{r}
grades_gender <- data.frame(
  sex = as.factor(c(rep("girl", 21), rep("boy", 23))),
  grade = c(
    19.3, 18.1, 15.2, 18.3, 7.9, 6.2, 19.4,
    20.3, 9.3, 11.3, 18.2, 17.5, 10.2, 20.1, 13.3, 17.2, 15.1, 16.2, 17.0,
    16.5, 5.1, 15.3, 17.1, 14.8, 15.4, 14.4, 7.5, 15.5, 6.0, 17.4,
    7.3, 14.3, 13.5, 8.0, 19.5, 13.4, 17.9, 17.7, 16.4, 15.6, 17.3, 19.9, 4.4, 2.1
  )
)

wilcoxon_statistics <- visstat(grades_gender$sex, grades_gender$grade)
```

#### Wilcoxon with ordinal response {.unnumbered}

```{r ordinal, fig.show='hide'}
set.seed(123)

# Create predictor: Customer segment (2 groups)
segment <- factor(rep(c("Budget", "Premium"), each = 50))

# Create response: Likert scale ratings (1-5)
satisfaction_numeric <- c(
  sample(1:5, 50, replace = TRUE, prob = c(0.15, 0.25, 0.30, 0.20, 0.10)),  # Budget
  sample(1:5, 50, replace = TRUE, prob = c(0.05, 0.10, 0.20, 0.35, 0.30))   # Premium
)

# Create dataframe with ORDERED response
survey_data <- data.frame(
  segment = segment,
  satisfaction = ordered(satisfaction_numeric)  # Declare as ordered
)

# triggers warnings and use Wilcoxon test
wilcox_ordered <- visstat(survey_data, "satisfaction", "segment")
```

## Categorical predictor with more than two levels

### Fisher's one-way ANOVA (`aov()`)

Fisher's one-way ANOVA (`aov()`) tests the null hypothesis that the means of $k$ groups are equal.

As a [linear model](#glm), it assumes independent observations, normally distributed residuals, and **homogeneous** variances across groups.
The test statistic is the ratio of the variance explained by differences among group means (between-group variance) to the unexplained variance within groups [@Fisher:1990]

$$F  = \frac{MS_{between}}{MS_{within}}=
\frac{SS_{between}/(k-1)}{SS_{within}/(N-k)} = \frac{\frac{\sum_{i=1}^{k} n_i (\bar{x}_i - \bar{x})^2}{k - 1}}
{\frac{\sum_{i=1}^{k}\sum_{j=1}^{n_i}(x_{ij}-\bar{x}_i)^2}{N - k}}$$

where:

- $MS_{between}$ and $MS_{within}$ are the mean square between groups and mean square within groups, respectively.

- $SS_{between}$ = Sum of Squares between groups (variance due to group differences)

- $SS_{within}$ = Sum of Squares within groups (error variance)

- $k$ = number of groups

- $N$ = total sample size

$\bar{x}_i$ is the mean of group $i$, $\bar{x}$ is the overall mean, $x_{ij}$ is the observation $j$ in group $i$, $n_i$ is the sample size in group $i$, $k$ is the number of groups, and $N$ is the total number of observations.

Under the null hypothesis, this statistic follows an F-distribution with two parameters for degrees of freedom: $(k - 1)$ and $(N - k)$: $F \sim F(k-1, N-k)$ The resulting p-value is computed from this distribution.

### Welch’s heteroscedastic one-way ANOVA (`oneway.test()`)

When only the assumptions of independent observations and normally distributed residuals are met, but *homogeneous variances* across groups *cannot be assumed*, Welch's heteroscedastic one-way ANOVA (`oneway.test()`) [@Welch:1951] provides an alternative to `aov()`.
It compares group means using weights based on sample sizes and variances.
The degrees of freedom are adjusted using a Satterthwaite-type approximation [@Satterthwaite:1946], resulting in an F-statistic with non-integer degrees of freedom.
The Welch F-statistic is calculated as [@Welch:1951]:

$$F_W = \frac{\sum_{i=1}^{k} w_i (\bar{y}_i - \bar{y}_w)^2 / (k-1)}{1 + \frac{2(k-2)}{k^2-1} \sum_{i=1}^{k} \frac{(1-w_i/w)^2}{n_i-1}}$$

where $w_i = n_i/s_i^2$ are the weights (inverse variances), $w = \sum_{i=1}^{k} w_i$, $\bar{y}_w = \sum_{i=1}^{k} w_i \bar{y}_i / w$ is the weighted grand mean, $k$ is the number of groups, $n_i$ is the sample size of group $i$, and $s_i^2$ is the variance of group $i$.

Numerical relationships within the parametric tests defined by the decision logic above (including the identity $t^2 = F$ in the equal-variance two-group case) are summarised in [Appendix A](#glm).

### Kruskal–Wallis test (`kruskal.test()`)

When the assumption of normality is not met, the Kruskal–Wallis test provides a non-parametric alternative.
It compares group distributions based on ranked values and tests the null hypothesis that the groups come from the same population — specifically, that the distributions have the same location [@Kruskal:1952].
If the group distributions are sufficiently similar in shape and scale, then the Kruskal–Wallis test can be interpreted as testing for equality of medians across groups [@Hollander:2014].

The test statistic is defined as:

$$H = \frac{12}{N(N+1)} \sum_{i=1}^{k} n_i \left(\bar{R}_i - \bar{R} \right)^2,$$

where $n_i$ is the sample size in group $i$, $k$ is the number of groups, $\bar{R}_i$ is the average rank of group $i$, $N$ is the total sample size, and $\bar{R} = \frac{N+1}{2}$ is the average of all ranks.
Under the null hypothesis, $H$ approximately follows a $\chi^2$ distribution with $k - 1$ degrees of freedom.

### Post-hoc analysis

ANOVA, Welch ANOVA, and Kruskal--Wallis are omnibus tests: a significant result tells us that *some* group differs, but not which.
To identify the differing pairs we test all

$$M = \frac{n \cdot (n - 1)}{2}$$

pairwise comparisons among the $n$ factor levels, defining a *family of tests* [@Abdi:2007].
Without correction, the family-wise error rate---the probability of at least one false rejection across the family---grows quickly with $n$.
Because the three omnibus tests rest on different assumptions, `visstat()` pairs each with a different post-hoc procedure and returns the corresponding adjusted p-values for every pairwise comparison.

#### Following `aov()`: `TukeyHSD()` {.unnumbered}

When the residuals are normal and variances are homogeneous, `visstat()` follows `aov()` with Tukey's Honestly Significant Differences procedure.
TukeyHSD controls the family-wise error rate via the studentised range distribution, which exploits the *common* residual variance shared across pairs [@Hochberg:1987].
`visstat()` returns the HSD-adjusted p-values for every pairwise mean comparison.

#### Following `oneway.test()`: `games.howell()` {.unnumbered}

When the residuals are normal but variances are heterogeneous, the common-variance assumption of TukeyHSD breaks down.
`visstat()` therefore pairs Welch’s heteroscedastic `oneway.test()` with the Games–Howell procedure, which is appropriate under unequal variances and unequal sample sizes. `games.howell()` uses *separate* variance estimates for each pair and Welch-adjusted degrees of freedom [@Games:1976].
The returned object contains the Games--Howell-adjusted p-values for every pairwise comparison.

#### Following `kruskal.test()`: `pairwise.wilcox.test()` {.unnumbered}

When normality is rejected, `visstat()` follows `kruskal.test()` with `pairwise.wilcox.test()`, which compares each pair of factor levels via the Wilcoxon rank-sum test on ranks rather than means.
The resulting p-values are adjusted for multiplicity using Holm's step-down method [@Holm:1979]: sorted ascending and tested against thresholds that loosen with rank.

### Graphical output

The graphical output for all tests based on a numeric response and a categorical predictor with more than two levels consists of two panels: the first focuses on the residual analysis, the second on the actual test chosen by the decision logic.

The residual panel addresses the assumption of normality, both graphically and through formal tests.
It displays a scatter plot of the standardised residuals versus the predicted values, as well as a normal Q--Q plot comparing the sample quantiles to the theoretical quantiles.
If the residuals are normally distributed, no more than $5\%$ of the standardised residuals should exceed approximately $|2|$; in the Q--Q plot the data points should approximately follow the red straight line.

The p-values of the formal tests for normality (Shapiro--Wilk and Anderson--Darling) as well as the tests for homoscedasticity (Bartlett's and Levene Brown–Forsythe) are given in the title.

`visstat()` then illustrates, in the subsequent graph, either the `kruskal.test()`, the `oneway.test()`, or `aov()` result (see also Section "Decision logic").
In all three branches the result is shown as box plots with the number of observations per level above each box; the title gives the name of the test that was run and its p-value (and, for the parametric branches, the corresponding $F$ statistic).
When the largest group contains no more than 50 observations, individual data points are overlaid as a jittered strip chart, making the raw distribution visible.

In every branch, pairs of groups whose adjusted post-hoc p-value falls below $\alpha$ are marked with *different* green letters below the box plots; pairs sharing a letter are not significantly different.
The letters are produced by `multcompLetters()` from the `multcompView` package [@Graves:2026].

Besides the graphical output, `visstat()` returns a list containing the relevant test statistics along with the corresponding post-hoc-adjusted $p$-values for all pairwise comparisons.

### Examples

#### Fisher's one-way ANOVA {.unnumbered}

The `npk` dataset reports the yield of peas (in pounds per block) from an agricultural experiment conducted on six blocks.
In this experiment, the application of three different fertilisers -- nitrogen (N), phosphate (P), and potassium (K) -- was varied systematically.
Each block received either none, one, two, or all three of the fertilisers.

```{r}
anova_npk <- visstat(npk$block,npk$yield,conf.level=0.95)
```

Normality of residuals is supported by graphical diagnostics (histogram, scatter plot of standardised residuals, Q-Q plot) and formal tests (Shapiro--Wilk and Anderson- Darling, both with $p > \alpha$).
Homogeneity of variances is not supported at the given confidence level by `bartlett.test()`, but by `levene.test()` ($p > \alpha$).
The decision logic is solely based on `levene.test()` and triggers `aov()`.
Post-hoc analysis with `TukeyHSD()`.
Note that the omnibus F-test reports p = 0.086 — not significant at $\alpha=0.05$.
Therefore no pairwise differences are expected in the post-hoc analysis and all groups share the same green letter "a".

#### Kruskal--Wallis rank sum test {.unnumbered}

The `iris` dataset contains petal width measurements (in cm) for three different iris species.

```{r,  results='hide'}
visstat(iris$Species, iris$Petal.Width)
```

In this example, scatter plots of the standardised residuals and the Q-Q plot suggest that the residuals are not normally distributed.
This is confirmed by very small p-values from both the Shapiro--Wilk and Anderson-Darling tests.

Since the Shapiro--Wilk p-value is below $\alpha$, `visstat()` switches to the non-parametric `kruskal.test()`.
Post-hoc analysis using `pairwise.wilcox.test()` shows significant differences in petal width between all three species, as indicated by distinct group labels (all green letters differ).

#### Kruskal--Wallis with ordinal response {.unnumbered}

When the response is of class `ordered` and the predictor is a categorical factor with more than two levels, `visstat()` issues a warning, converts the ordered response to integer level codes and routes the analysis to `kruskal.test()` followed by `pairwise.wilcox.test()`.

```{r ordinal-kruskal, fig.show='hide'}
set.seed(123)

# Predictor: customer segment (3 groups)
segment <- factor(rep(c("Budget", "Standard", "Premium"), each = 50))

# Response: Likert scale ratings (1-5), with a deliberate trend across segments
comfort_numeric <- c(
  sample(1:5, 50, replace = TRUE, prob = c(0.30, 0.30, 0.20, 0.15, 0.05)),  # Budget
  sample(1:5, 50, replace = TRUE, prob = c(0.10, 0.20, 0.40, 0.20, 0.10)),  # Standard
  sample(1:5, 50, replace = TRUE, prob = c(0.05, 0.10, 0.20, 0.35, 0.30))   # Premium
)

# Dataframe with ORDERED response
survey_data_3 <- data.frame(
  segment = segment,
  comfort = ordered(comfort_numeric)  # Declare as ordered
)

# triggers warning and uses Kruskal-Wallis test
kruskal_ordered <- visstat(survey_data_3, "comfort", "segment")
```

## Both variables numeric

### Simple linear regression (`lm()`)

The regression plot displays the point estimate of the regression line

$$y = b_0 + b_1 \cdot x,$$

where $y$ is the response variable, $x$ is the predictor variable, $b_0$ is the intercept, and $b_1$ is the slope of the regression line.

#### Residual analysis

`visstat()` checks the normality of the standardised residuals from `lm()` both with diagnostic plots and using the Shapiro--Wilk and Anderson-Darling tests.
(via `vis_lm_assumptions()`)

Note, that regardless of the result of the residual analysis, `visstat()` proceeds to perform the regression.
The title of the graphical output indicates the chosen confidence level (`conf.level`), the estimated regression parameters with their confidence intervals and p-values, and $R^2$.
The plot displays the raw data, the fitted regression line, and both the confidence and prediction bands corresponding to the specified `conf.level`.

`visstat()` returns a list containing the regression test statistics, the p-values from the normality tests of the standardised residuals, and the pointwise estimates of the confidence and prediction bands.

### Examples

#### `swiss` dataset {.unnumbered}

The `swiss` dataset records standardised fertility and socioeconomic indicators for 47 French-speaking Swiss provinces in 1888.
We examine how the share of draftees achieving the highest army examination score (`Examination`) predicts the fertility measure (`Fertility`).

```{r}
linreg_swiss <- visstat(swiss$Examination, swiss$Fertility, conf.level = 0.99)
```

Both normality tests pass ($p > \alpha$) and the Breusch--Pagan test confirms homoscedasticity, validating the linear model.


### Spearman rank correlation (`cor.test(..., method = "spearman")`)

For two numeric variables with `correlation = TRUE`, `visstat()` calls `cor.test(x, y, method = "spearman")` to measure the monotonic association between $x$ and $y$ using ranks, evaluating linear dependence on the ranked data rather than on the original scale:

$$\rho = r(\operatorname{rank}(x), \operatorname{rank}(y))$$ where $r(u, v)$ denotes Pearson’s correlation coefficient: $$r(u,v)
=
\frac{\sum_{i=1}^{n}(u_i-\bar u)(v_i-\bar v)}
{\sqrt{\sum_{i=1}^{n}(u_i-\bar u)^2}\,
 \sqrt{\sum_{i=1}^{n}(v_i-\bar v)^2}}.$$

For inference, `cor.test(..., method = "spearman")` computes an exact p-value for small samples without ties by evaluating all $n!$ rank permutations.
For larger samples or when ties are present, it uses an approximation to the null distribution of the test statistic.
No distributional assumption on the original data is required.

#### Example {.unnumbered}

The `airquality` dataset contains daily air quality measurements in New York, May to September 1973 [@Chambers:2018]

A default call to `visstat()` fits a simple linear regression model, but triggers the following warning:

```{r, fig.show='hide'}
result_ozone0 <- visstat(airquality$Wind, airquality$Ozone)
```

The warning reports violated normality and homoscedasticity and recommends two paths: rerun with `correlation = TRUE` within `visstat()`, or switch to a model outside it.

Staying within `visstat()`, `correlation = TRUE` gives an assumption-free result at the cost of causality:

```{r}
result_ozone1 <- visstat(airquality$Wind, airquality$Ozone, correlation = TRUE)
```


To preserve causality, we follow the second path and fit a Gamma generalised linear model with log link. The Gamma family is suited here because Ozone is strictly positive and continuous, and its variance grows with the mean — the structure detected by the Breusch--Pagan test. The log link (not the canonical inverse link) is preferred in practice as it guarantees positive fitted values.

```{r}
# Remove zeros to satisfy Gamma requirements
airquality_clean <- subset(airquality, Ozone > 0)
# Gamma model with log mapping
model_gamma <- glm(Ozone ~ Wind, data = airquality_clean, family = Gamma(link = "log"))
summary(model_gamma)
```

```{r, echo=FALSE}
# Plotting the data with Gamma model overlay
plot(airquality_clean$Wind, airquality_clean$Ozone, log = "y", 
     pch = 19, col = "darkgrey", xlab = "Wind (mph)", ylab = "Ozone (ppb) [log scale]",
     main = "Gamma GLM Fit (Log Link)")
# Generate predictions for the overlay
wind_seq <- seq(min(airquality_clean$Wind), max(airquality_clean$Wind), length.out = 100)
preds <- predict(model_gamma, newdata = data.frame(Wind = wind_seq), type = "response")
lines(wind_seq, preds, col = "red", lwd = 2)

legend("topright", legend = c("Data", "Gamma GLM (log link)"), 
       col = c("darkgrey", "red"), pch = c(19, NA), lty = c(NA, 1), lwd = c(NA, 2))
```

For a well-fitting Gamma generalised linear model, standardised deviance residuals are asymptotically standard normal; we use Shapiro--Wilk and Anderson--Darling as approximate checks.

```{r}
# Extract standardised  residuals
std_dev_res <- rstandard(model_gamma, type = "deviance")
# Validate using the Shapiro-Wilk normality test
shapiro.test(std_dev_res)
# Validate using the Anderson-Darling normality test
nortest::ad.test(std_dev_res)
```

## Both variables categorical: Comparing proportions

Observed frequencies are arranged in a contingency table, where rows index the levels $i$ of the response variable and columns index the levels $j$ of the predictor variable.
`visstat()` tests the null hypothesis that the two variables are independent.

### Pearson’s $\chi^2$-test (`chisq.test()`)

Let $O_{ij}$ and $E_{ij}$ denote the observed and expected frequencies in row $i$ and column $j$ of an $R \times C$ contingency table.
The Pearson residual for each cell is defined as

$$r_{ij} = \frac{O_{ij} - E_{ij}}{\sqrt{E_{ij}}}, \quad i = 1, \ldots, R,\quad j = 1, \ldots, C.$$

The test statistic of Pearson’s $\chi^2$-test [@Pearson:1900] is the sum of squared Pearson residuals:

$$\chi^2 = \sum_{i=1}^{R} \sum_{j=1}^{C} r_{ij}^2 =
\sum_{i=1}^{R} \sum_{j=1}^{C} \frac{(O_{ij} - E_{ij})^2}{E_{ij}}.$$

The test statistic is compared to the chi-squared distribution with $(R - 1)(C - 1)$ degrees of freedom.
The resulting p-value corresponds to the upper tail probability — that is, the probability of observing a value greater than or equal to the test statistic under the null hypothesis.

For general $R \times C$ tables, `visstat()` supplements the bar chart with a mosaic plot [@Meyer:2006; @Meyer:2024], in which the area of each tile is proportional to the observed cell frequency and tiles are coloured by their Pearson residual $r_{ij}$: blue for positive residuals (observed exceeds expected) and red for negative ones (observed falls short of expected).
This makes the cells driving the association immediately visible.

### Pearson’s $\chi^2$ test with Yates’ continuity correction

Pearson $\chi^2$ statistic in $2 \times 2$ contingency tables (resulting in only one degree of freedom) tends to overestimate the significance level of the test.

To correct for this, Yates proposed subtracting 0.5 from each absolute difference between observed and expected counts [@Yates:1934], resulting in a smaller test statistic: $$\chi^2_{\text{Yates}} = \sum_{i=1}^{2} \sum_{j=1}^{2}
\frac{(|O_{ij} - E_{ij}| - 0.5)^2}{E_{ij}}.$$

This reduced test statistic yields a larger p-value, thereby lowering the risk of a Type I error.

Yates' continuity correction is applied by default to $2 \times 2$ contingency tables with one degree of freedom by the underlying routine `chisq.test()`.

#### Fisher's exact test (`fisher.test()`)

The $\chi^2$ approximation is considered reliable only if no expected cell count is less than 1 and no more than 20% of cells have expected counts below 5 [@Cochran:1954]. If this condition is not met, Fisher's exact test [@Fisher:1970] (`fisher.test()`) is applied instead. 

The test calculates an exact $p$-value by conditioning on the observed margins of an $R \times C$ contingency table. To maintain consistency with the `y ~ x` (response ~ predictor) framework used throughout `visStatistics`, the **rows ($i=1, \dots, R$) represent the levels of the response variable $y$** and the **columns ($j=1, \dots, C$) represent the levels of the predictor $x$**. The row totals are defined as $n_{i\cdot} = \sum_{j=1}^C n_{ij}$ and the column totals as $n_{\cdot j} = \sum_{i=1}^R n_{ij}$, for $i = 1, \dots, R$ and $j = 1, \dots, C$.

In the $2 \times 2$ case ($R=2, C=2$), the table is structured as follows:

$$\begin{array}{c|cc|c}
& x_1 & x_2 & \text{Row sums} \\
\hline
y_1 & n_{11} & n_{12} & n_{1\cdot} \\
y_2 & n_{21} & n_{22} & n_{2\cdot} \\
\hline
\text{Column sums} & n_{\cdot 1} & n_{\cdot 2} & n
\end{array}$$

The exact probability of observing this table under the null hypothesis of independence, given the fixed margins, is given by the hypergeometric probability mass function (PMF):

$$\mathbb{P}(N \mid n_{1\cdot}, n_{2\cdot}, n_{\cdot 1}, n_{\cdot 2}) = \frac{\binom{n_{1\cdot}}{n_{11}} \binom{n_{2\cdot}}{n_{21}}}{\binom{n}{n_{\cdot 1}}}$$

where $n = n_{1\cdot} + n_{2\cdot} = n_{\cdot 1} + n_{\cdot 2}$ is the total sample size. The $p$-value is computed by summing the probabilities of all tables with the same margins whose probabilities under the null are less than or equal to that of the observed table.

For general $R \times C$ tables, `fisher.test()` generalises this approach using the multivariate hypergeometric distribution.

For $2 \times 2$ tables, `fisher.test()` additionally returns the conditional maximum likelihood estimate of the odds ratio $\widehat{\text{OR}}$. Given the orientation where rows are the response $y$, the odds of observing response level $y_1$ (relative to $y_2$) in predictor group $x_1$ are $n_{11}/n_{21}$, and the corresponding odds in group $x_2$ are $n_{12}/n_{22}$. The odds ratio is thus:

$$\widehat{\text{OR}} = \frac{n_{11} / n_{21}}{n_{12} / n_{22}} = \frac{n_{11} \cdot n_{22}}{n_{12} \cdot n_{21}}$$

This estimate and its confidence interval are accessible as `$estimate` and `$conf.int` in the returned `visstat` object.
### Test choice and graphical output

If the expected frequencies are sufficiently large - specifically, if at least 80% of the cells have expected counts greater than 5 and no expected count is smaller than 1, the function uses Pearson's ${\chi}^2$-test (`chisq.test()`).

Otherwise, it switches to Fisher's exact test (`fisher.test()`) [@Cochran:1954].

For 2-by-2 contingency tables, Yates' continuity correction [@Yates:1934] is always applied to Pearson's ${\chi}^2$-test.

The graphical output depends on the selected test:

- **Pearson's** $\chi^2$ test (general $R \times C$ tables): a grouped column plot showing row percentages with the $p$-value in the title, followed by a mosaic plot with colour-coded Pearson residuals.
- **Pearson's** $\chi^2$ test with Yates' continuity correction ($2 \times 2$ tables): a grouped column plot showing row percentages with the $p$-value in the title. <!-- No mosaic plot is produced, because the Yates-corrected statistic is not decomposable into cell-level Pearson residuals. -->
- **Fisher's exact test**: a grouped column plot showing absolute counts with $N$ labels above each bar and the $p$-value in the title. <!-- No mosaic plot is produced, as Fisher's exact test does not yield Pearson residuals. -->

### Transforming a contingency table to a data frame

The following examples for tests of categorical predictor and response are all based on the `HairEyeColor` contingency table.

Contingency tables must be converted to the required column-based `data.frame` using the helper function `counts_to_cases()`.
The function transforms the contingency table `HairEyeColor` into `data.frame` named `HairEyeColourDataFrame`.

```{r}
HairEyeColourDataFrame <- counts_to_cases(as.data.frame(HairEyeColor))
```

### Examples

In all examples of this section, we will test the null hypothesis that hair colour ("Hair") and eye colour ("Eye") are independent of each other.

#### Pearson's ${\chi}^2$-test (`chisq.test()`)

```{r}
hair_eye_colour_df <- counts_to_cases(as.data.frame(HairEyeColor))
visstat(hair_eye_colour_df$Eye, hair_eye_colour_df$Hair)
```

The graphical output shows that the null hypothesis of Pearson's $\chi^2$ test -- namely, that hair colour and eye colour are independent -- must be rejected at the default significance level $\alpha=0.05$ ($p = 2.33 \cdot 10^{-25} <
\alpha$).
The mosaic plot indicates that the strongest deviations are due to over-representation of individuals with black hair and brown eyes, and of those with blond hair and blue eyes.
In contrast, individuals with blond hair and brown eyes are the most under-represented.

#### Pearson's ${\chi}^2$-test with Yates' continuity correction

In the following example, we restrict the data to participants with either black or brown hair and either brown or blue eyes, resulting in a 2-by-2 contingency table.

```{r}
hair_black_brown_eyes_brown_blue <- HairEyeColor[1:2, 1:2, ]
# Transform to data frame
hair_black_brown_eyes_brown_blue_df <- counts_to_cases(as.data.frame(hair_black_brown_eyes_brown_blue))
# Chi-squared test with Yates' continuity correction

visstat(hair_black_brown_eyes_brown_blue_df$Eye, hair_black_brown_eyes_brown_blue_df$Hair)
```

Also in this reduced dataset we reject the null hypothesis of independence of the hair colours "brown" and "black" from the eye colours "brown" and "blue".
As a $2 \times 2$ table, Yates' continuity correction is applied and no mosaic plot is produced.
Note that the Yates-corrected $p$-value is slightly higher than the uncorrected Pearson $p$-value, reflecting the more conservative correction.

#### Fisher's exact test (`fisher.test()`)

Again, we extract a 2-by-2 contingency table from the full dataset, this time keeping only male participants with black or brown hair and hazel or green eyes.

Pearson's ${\chi}^2$ test applied to this table would yield an expected frequency less than 5 in one of the four cells (25% of all cells), which violates the requirement that at least 80% of the expected frequencies must be 5 or greater [@Cochran:1954].

Therefore, `visstat()` automatically selects Fisher's exact test instead.

```{r fisher-data-prep}
hair_eye_colour_male <- HairEyeColor[, , 1]
# Slice out a 2 by 2 contingency table
black_brown_hazel_green_male <- hair_eye_colour_male[1:2, 3:4]
# Transform to data frame
black_brown_hazel_green_male <- counts_to_cases(as.data.frame(black_brown_hazel_green_male))
# Fisher test
fisher_stats <- visstat(black_brown_hazel_green_male$Eye, black_brown_hazel_green_male$Hair)
```

For this $2 \times 2$ table, the odds ratio and its 95% confidence interval are available in the returned object:

```{r fisher-or}
fisher_stats$estimate   # odds ratio
fisher_stats$conf.int   # 95% CI
```

## Response of class `ordered`, predictor of class `factor`

When the response is an ordered factor, `visstat()` converts it internally to integer level codes and redirects to the non-parametric path (see examples in Section [Comparing central tendencies](#comparing-central-tendencies)).

## Both variables of class `factor` and `ordered`: Wilcoxon/Kruskal-Wallis (default) or Kendall's $\tau_b$ (`correlation = TRUE`)

When both the response and the predictor are ordered factors and `correlation = TRUE` is set, `visstat()` tests the null hypothesis of no monotone association via Kendall's $\tau_b$ rank correlation [@Kendall:1945; @Agresti:2010].
Without `correlation = TRUE`, both-ordered inputs follow the standard non-parametric path (Wilcoxon or Kruskal--Wallis).

### Kendall's $\tau_b$ (`cor.test(..., method = "kendall")`)

For two ordinal variables with $n$ joint observations, let $C$ denote the number of concordant pairs (those whose ranks agree in both variables) and $D$ the number of discordant pairs.
Kendall's $\tau_b$ is defined as

$$\tau_b \;=\; \frac{C - D}{\sqrt{(n_0 - n_1)(n_0 - n_2)}}$$

where $n_0 = n(n-1)/2$, $n_1 = \sum_i t_i(t_i-1)/2$ summed over groups of tied ranks in the response, and $n_2$ is the analogous quantity for the predictor.
The denominator correction makes $\tau_b$ attain $\pm 1$ even with ties, which Spearman's $\rho$ does not [@Kendall:1945].
With few ordered levels (e.g., five-point Likert items), ties are unavoidable; this is the principal reason to prefer $\tau_b$ over Spearman's $\rho$ in this setting [@Agresti:2010].

`visstat()` calls `cor.test(as.numeric(y), as.numeric(x), method = "kendall", exact = FALSE)` and reports $\tau_b$, the test statistic $z$, and the two-sided $p$-value.

### Graphical output

One plot is produced: a jittered rank--rank scatter that visualises the monotone trend, with points colour-coded by the predictor level and annotated with $\tau_b$ and the $p$-value.

### Example

We construct a hypothetical survey of 150 secondary-school students in which alcohol consumption frequency and academic performance are each recorded on a five-point ordinal scale.
A negative monotone association is expected: students who consume alcohol more frequently tend to achieve lower academic performance.

```{r kendall-example}
set.seed(42)
n <- 150
# Latent scores with deliberate negative monotone association:
# higher alcohol consumption (xs) -> lower academic performance (ys)
xs <- sample(1:5, n, replace = TRUE)
ys <- pmin(5, pmax(1, (6 - xs) + sample(-1:1, n, replace = TRUE)))
likert_levels  <- c("never", "rarely", "sometimes", "often", "always")
likert_levels2 <- c("poor", "fair", "ok", "good", "great")
alcohol     <- ordered(likert_levels[xs],  levels = likert_levels)
performance <- ordered(likert_levels2[ys], levels = likert_levels2)
kendall_result <- visstat(performance, alcohol, correlation = TRUE)
```

# Saving the graphical output

All generated graphics can be saved in any file format supported by `Cairo()` [@Urbanek:2025], including "png", "jpeg", "pdf", "svg", "ps", and "tiff" in the user specified `plotDirectory`.

If the optional argument `plotName` is not given, the naming of the output follows the pattern `"testname_namey_namex."`, where `"testname"` specifies the selected test and `"namey"` and `"namex"` are character strings naming the selected data vectors `y` and `x`, respectively.
The suffix corresponding to the chosen `graphicsoutput` (e.g., `"pdf"`, `"png"`) is then concatenated to form the complete output file name.

In the following example, we store the graphics in `png` format in the `plotDirectory` `tempdir()` with the default naming convention:

```{r fisher-save-graphics}
# Graphical output written to plotDirectory: In this example
# a single bar chart showing absolute counts.
# Output file: chi_squared_or_fisher_Hair_Eye.png
save_fisher = visstat(black_brown_hazel_green_male$Eye, black_brown_hazel_green_male$Hair,
        graphicsoutput = "png", plotDirectory = tempdir())
```

The full file path of the generated graphics are stored as the attribute `"plot_paths"` on the returned object of class `"visstat"`.

```{r show-path2}
paths <- attr(save_fisher, "plot_paths")
print(paths)
```

Remove the graphical output from `plotDirectory`:

```{r, eval=TRUE}
file.remove(paths)
```

When assumptions plots (residual and Q-Q plot) are generated, the corresponding plot has the prefix `"assumption_"`.

# The `visstat` methods

Objects returned by `visstat()` are of class `"visstat"` and support the S3 methods `print()`, `summary()`, and `plot()`.

## `print()` and `summary()`

`print()` displays the test used and the p-value; `summary()` provides the full contents of the returned object, including assumption tests and post-hoc comparisons.

```{r visstat-methods, fig.show='hide', results='hide'}
iris_kruskal <- visstat(iris$Species, iris$Petal.Width)
```

```{r visstat-methods-print-summary}
print(iris_kruskal)
summary(iris_kruskal)
```

## `plot()`

When `visstat()` is called with `graphicsoutput` specified, `plot()` lists the file paths of the stored graphics:

```{r visstat-methods-plot-paths}
iris_kruskal_stored <- visstat(iris$Species, iris$Petal.Width,
                               graphicsoutput = "pdf",
                               plotName = "iris_kruskal",
                               plotDirectory = tempdir())
plot(iris_kruskal_stored)
```

When `visstat()` is called without `graphicsoutput` (the default interactive mode), the generated plots are captured internally.
Calling `plot()` without `which` lists the available plots; calling `plot()` with `which` replays the selected plot in the interactive R session:

```{r visstat-methods-plot-replay}
plot(iris_kruskal)
```

``` r
# Interactive only (not executed during vignette build):
plot(iris_kruskal, which = 2)
```

```{r visstat-methods-cleanup, echo = FALSE}
file.remove(attr(iris_kruskal_stored, "plot_paths"))
```

# Limitations {#limitations}

## Default settings

The main purpose of this package is a decision-logic based automatic selection visualisation of the "right" statistical test.
Therefore, except for the user-adjustable `conf.level` parameter, all statistical tests are applied using their default settings from the corresponding base R functions.
As a consequence, paired tests are currently not supported and `visstat()` does not allow to study interactions terms between the different levels of an independent variable in an analysis of variance.
Focusing on the graphical representation of tests, only simple linear regression is implemented, as multiple linear regressions cannot be visualised.
When regression assumptions are violated, the package offers Spearman rank correlation (`correlation = TRUE`) as the sole alternative to linear regression.
Alternative methods such as data transformation, generalised linear models or robust regression are not implemented: each requires user judgment — about the transformation family, the link function, or the estimator — that cannot be automated without substantially expanding the decision tree and increasing the risk of Type I error inflation.

## Test decisions based solely on p-values of statistical tests

No single test maintains optimal Type I error rates and statistical power across all distributions [@Olejnik:1987], and p-values obtained from these tests may be unreliable if their assumptions are violated.

Assessing assumptions solely through p-values can lead to both type I errors (false positives) and type II errors (false negatives).
In large samples, even minor, random deviations from the null hypothesis can result in statistically significant p-values, leading to type I errors.
Conversely, in small samples, substantial violations of the assumption may not reach statistical significance, resulting in type II errors [@Kozak:2018].

Moreover, assumption tests provide no information on the nature of deviations from the expected distribution [@Shatz:2024].
Thus the assessment of normality or homoscedasticity should never rely solely on p-values but should be complemented by visual inspection of the diagnostic plots generated by `visstat()`.

## Testing normality with the Shapiro–Wilk normality test

Normality tests behave poorly at both ends of the sample-size range: with small samples they fail to detect non-normality, and with large samples they flag negligible departures from normality as significant [@Ghasemi:2012; @Fagerland:2012; @Franc:2025].

This package uses $n > 50$ per group as a rule of thumb threshold to omit normality testing, assuming parametric tests of central tendencies to become robust to non-normality at larger sample sizes based on the central limit theorem.
The threshold is necessarily arbitrary; simulation studies show that the convergence rate depends on the skewness and kurtosis of the underlying distribution, with moderately skewed distributions requiring roughly 40--50 observations for adequate convergence of the sampling distribution of the mean [@Fagerland:2012].
Simulation studies suggest that Shapiro–Wilk has the highest power among normality tests in small to moderate ($n = 10$ to 100) sample sizes [@Razali:2011].

This package uses therefore the Shapiro-Wilk test to select between parametric (ANOVA/Welch's t-test) and non-parametric (Kruskal-Wallis/Wilcoxon) methods for groups smaller than 50.

## Visualisation based solely on base R

The package depends only on base R graphics with no `ggplot2` [@Wickham:2016] dependency keeping the transitive dependency footprint minimal.
For more polished, annotated plots of chosen statistical test, we refer to packages as `ggstatsplot` [@Patil:2021] or `ggpubr` [@Kassambara:2026].

## Combining tests inflates the overall Type I error rate

Combining tests for normality and homoscedasticity using simple majority voting inflates the overall Type I error rate.

Therefore, automated test selection based solely on p-values cannot replace the visual inspection of sample distributions provided by `visstat()`.
Based on the provided diagnostic plots, it may be necessary to override the automated choice of test in individual cases.

## Limited number of implemented tests

While one of R's greatest strengths is the sheer volume of statistical methods available, our automated test selection pipeline deliberately restricts its scope to the most frequently used tests, particularly those relevant in a medical context [@Strasak:2007,@Sato:2017,@Chicco:2025].
Incorporating a wider array of methods (such as regression models with a categorical response) would require additional preliminary assumption checks, which in turn exacerbates the risk of overall Type I error inflation.
Furthermore, expanding the pipeline would result in a highly complex decision tree, rendering the underlying statistical logic increasingly opaque to the user.



## Bootstrapping as modern alternative to hypothesis testing

Bootstrapping methods [@Wilcox:2021] make minimal distributional assumptions and can provide confidence intervals for nearly any statistic.
However, bootstrapping is computationally intensive, often requiring thousands of resamples, and may perform poorly with very small sample sizes.

The computational intensity of bootstrap runs counter to the purpose of the `visStatistics` package, which is designed to offer a rapid overview of the data, laying the groundwork for deeper analysis in subsequent steps.

# Appendix A: The general linear model {#glm}

The general linear model [@Searle:1971] provides a unified mathematical framework underlying Student's t-test, Fisher's ANOVA, and simple linear regression.

Let $n$ denote the number of observations and $k-1$ the number of predictors.
The model for observation $i,\;i = 1, \ldots, n$ is:

$$Y_i = \beta_0 + \beta_1 x_{i1} + \cdots + \beta_{k-1} x_{ik-1} + \varepsilon_i, \quad \varepsilon_i \sim N(0, \sigma^2)$$

where $Y_i$ is the response for observation $i$, $x_{ij}$ is the value of predictor $j$ for observation $i$, $\beta_0, \beta_1, \ldots, \beta_{k-1}$ are the $k$ parameters, and $\varepsilon_i$ are independently distributed error terms with constant variance $\sigma^2$.

## Student's t-test as a linear model

Student's t-test tests the null hypothesis that the means of two (unpaired) groups are equal.
We create one indicator binary variable $x_{i1}$, where $x_{i1} = 0$ for observations in group 1 and $x_{i1} = 1$ for observations in group 2.
Taking expectations for each group:

- Group 1 ($x_{i1} = 0$): $E[Y_i | x_{i1} = 0] = \beta_0 = \mu_1$
- Group 2 ($x_{i1} = 1$): $E[Y_i | x_{i1} = 1] = \beta_0 + \beta_1 = \mu_2$

Therefore, $\beta_1 = \mu_2 - \mu_1$ represents the difference between group means.
Testing $H_0: \beta_1 = 0$ is mathematically equivalent to testing $H_0: \mu_1 = \mu_2$ in Student's t-test.

## Fisher's ANOVA as a linear model

Fisher's one-way ANOVA tests the null hypothesis that the means of $k$ groups are equal.

It can be formulated within the linear model framework using $k-1$ indicator variables $x_{i1}, x_{i2}, \ldots, x_{ik-1}$ for each observation $i$ to represent group membership.
The indicator variable coding is defined as follows:

- Group 1 (reference): $x_{i1} = x_{i2} = \cdots = x_{ik-1} = 0$ for all observations i in group 1
- Group 2: $x_{i1} = 1$ and $x_{i2} = \cdots = x_{ik-1} = 0$ for all observations i in group 2\
- Group 3: $x_{i2} = 1$ and $x_{i1} = x_{i3} = \cdots = x_{ik-1} = 0$ for all observations i in group 3
- ...
- Group k: $x_{ik-1} = 1$ and $x_{i1} = \cdots = x_{i,k-2} = 0$ for all observations i in group k

Taking expectations for each group:

- Group 1: $E[Y_i | x_{i1} = x_{i2}=\cdots = x_{ik-1} = 0] = \beta_0 = \mu_1$
- Group 2: $E[Y_i | x_{i1} = 1] = \beta_0 + \beta_1 = \mu_2$
- Group 3: $E[Y_i | x_{i2} = 1] = \beta_0 + \beta_2 = \mu_3$
- ...
- Group k: $E[Y_i | x_{ik-1} = 1] = \beta_0 + \beta_{k-1} = \mu_{k}$

Testing $H_0: \beta_1 = \beta_2 = \cdots = \beta_{k-1} = 0$ is mathematically equivalent to testing $H_0: \mu_1 = \mu_2 = \cdots = \mu_{k}$ in Fisher's ANOVA.

### Relations between tests

The two-sample case sits inside both the t-test and the ANOVA frameworks, so several pairs of tests are numerically related.

**Welch's t-test → Student's t-test.** When the assumption of equal variances holds ($s_1^2 = s_2^2 = s^2$) and sample sizes are equal ($n_1 = n_2 = n$), Welch's t-test reduces to Student's t-test with $2n - 2$ degrees of freedom.
This exact equivalence does not extend to the multi-group case:

**Welch's one-way ANOVA → Fisher's one-way ANOVA.** Even under equal variances ($s_1^2 = \cdots = s_k^2$) and equal sample sizes ($n_1 = \cdots = n_k$), the Welch test statistic is not algebraically identical to the classical ANOVA $F$-statistic.
Nevertheless, under these conditions the Welch statistic converges to the classical $F$-statistic, and any numerical differences become negligible in practice [@Welch:1951].

**t-tests as special cases of ANOVA.** t-tests are special cases of ANOVA for the comparison of two groups.
The squared $t$-statistic equals the corresponding $F$-statistic:

$t^2 = F$.

For the comparison of two groups, Student's t-test, `t.test(var.equal = TRUE)` and `aov()` yield identical p-values, as do Welch's t-test `t.test(var.equal = FALSE)` and `oneway.test()`.
In this case `visstat()` reports the t-statistic, as it provides sign information (indicating which group has the larger mean).

## Simple linear regression as a linear model

Simple linear regression corresponds to one continuous predictor $x_{i1}$ for observation $i$.
Testing $H_0: \beta_1 = 0$ examines whether there is a linear relationship between the predictor and the response.

# Appendix B: Tests for homoscedasticity {#variance}

## The Levene–Brown–Forsythe test `levene.test()`

The Levene–Brown–Forsythe test improves upon Levene's original test [@Levene:1960] by using the median instead of the mean to centre the data.

This makes it more robust to skewed data or data with outliers providing more reliable results in many practical situations [@Allingham:2012].

`levene.test()` mimics the default behaviour of `leveneTest()` in the `car` package [@Fox:2019].

The Levene–Brown–Forsythe test evaluates the null hypothesis that all groups have equal variances by testing whether the absolute deviations from group medians are equal across groups.

For each observation $y_{ij}$ in group $i$, it computes the absolute deviation from the group median:

$$z_{ij} = |y_{ij} - \tilde{y_i}|,$$

where $\tilde{y_i}$ is the median of group $i$.

The test statistic is the F-statistic from a one-way ANOVA on the $z_{ij}$ values:

$$F = \frac{\frac{\sum_{i=1}^{k} n_i (\bar{z}_i - \bar{z})^2}{k-1}}{\frac{\sum_{i=1}^{k} \sum_{j=1}^{n_i} (z_{ij} - \bar{z}_i)^2}{N-k}} = \frac{(N-k) \sum_{i=1}^{k} n_i (\bar{z}_i - \bar{z})^2}{(k-1) \sum_{i=1}^{k} \sum_{j=1}^{n_i} (z_{ij} - \bar{z}_i)^2}$$

where $k$ is the number of groups, $N$ is the total sample size, $n_i$ is the sample size of group $i$, $\bar{z}_i$ is the mean of absolute deviations from the median in group $i$, and $\bar{z}$ is the overall mean of all absolute deviations.

Under the null hypothesis of equal variances, the test statistic follows an F-distribution: $F \sim F(k-1, N-k)$.

## Bartlett's test `bartlett.test()`

Additionally, homoscedasticity is assessed via Bartlett's test [@Bartlett:1937] (`bartlett.test()`), which has more power than the Brown–Forsythe version of Levene’s test [@Brown:1974] when the normality assumption is met [@Allingham:2012].

Bartlett’s test evaluates whether sample variances are equal across $k$ normally distributed groups.

The test statistic is

$$K^2 = \frac{(N - k) \ln s_p^2 - \sum_{i=1}^k (n_i - 1) \ln s_i^2}{
   1 + \frac{1}{3(k - 1)} \left( \sum_{i=1}^k \frac{1}{n_i - 1}
- \frac{1}{N - k} \right)},$$ where $s_i^2$ is the sample variance of group $i$, and $s_p^2$ is the pooled variance:

$$s_p^2 = \frac{1}{N - k} \sum_{i=1}^k (n_i - 1) s_i^2.$$

Under the null hypothesis that all group variances are equal and the data are normally distributed, the test statistic approximately follows a $\chi^2$-distribution with $k - 1$ degrees of freedom [@Bartlett:1937].

## Breusch-Pagan test `bp.test()`

For linear regression, group-based tests are not applicable.
Instead, the `visStatistics` package function `bp.test()` implements the Breusch-Pagan test.

The Breusch-Pagan test assesses whether the variance of the residuals from a regression model depends on the values of the independent variables [@Breusch:1979].

The diagnostic plots together with the p-values of the tests for normality and homoscedasticity enable the user to assess whether the linear model assumptions are met and manually override the automated test selection.

# Bibliography

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bp.test.R
\name{bp.test}
\alias{bp.test}
\title{Breusch-Pagan Test for Heteroscedasticity}
\usage{
bp.test(model)
}
\arguments{
\item{model}{A fitted linear model object (from \code{lm()}).}
}
\value{
An object of class \code{"htest"} with components:
\item{statistic}{the value of the chi-squared test statistic.}
\item{parameter}{degrees of freedom.}
\item{p.value}{the p-value of the test.}
\item{method}{a character string indicating the test performed.}
\item{data.name}{a character string giving the name of the model.}
}
\description{
Performs the Breusch-Pagan test for heteroscedasticity in 
linear regression models. Tests the null hypothesis that the error variance 
is constant (homoscedasticity) against the alternative that the error 
variance depends on the fitted values.
}
\details{
The Breusch-Pagan test regresses the squared standardized residuals on 
the fitted values. The test statistic is:

\deqn{BP = n \cdot R^2}

where:
\itemize{
  \item \eqn{n} = sample size
  \item \eqn{R^2} = coefficient of determination from auxiliary regression of 
        \eqn{e_i^2} on \eqn{\hat{y}_i}
}

Under the null hypothesis of homoscedasticity, the test statistic follows 
a chi-squared distribution: \eqn{BP \sim \chi^2(p-1)} where \eqn{p} is the 
number of parameters in the model (including intercept).

Large values of the test statistic (small p-values) provide evidence against 
homoscedasticity.
}
\examples{
# Example with homoscedastic errors
set.seed(123)
x <- runif(100)
y <- 2 + 3*x + rnorm(100, sd = 1)
model1 <- lm(y ~ x)
bp.test(model1)  # Should not reject (p > 0.05)

# Example with heteroscedastic errors (variance increases with x)
set.seed(456)
x <- runif(100)
y <- 2 + 3 *x + rnorm(100, sd = 0.5 + 2*x)
model2 <- lm(y ~ x)
bp.test(model2)  # Should reject (p < 0.05)

}
\references{
Breusch, T. S., and Pagan, A. R. (1979). A simple test for heteroscedasticity 
and random coefficient variation. Econometrica, 47(5), 1287-1294.
DOI: 10.2307/1911963
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_lm_assumptions.R
\name{vis_lm_assumptions}
\alias{vis_lm_assumptions}
\alias{vis_anova_assumptions}
\title{Visualisation of the normality distribution of the standardised residuals}
\usage{
vis_lm_assumptions(samples, fact, cex = 1, regression = FALSE)

vis_anova_assumptions(...)
}
\arguments{
\item{samples}{Numeric vector; the dependent variable.}

\item{fact}{Factor; the independent variable.}

\item{cex}{Numeric; scaling factor for plot text and symbols (default: 1).}

\item{regression}{Logical; if TRUE, skips Bartlett's test (for regression diagnostics). Default is FALSE.}

\item{...}{Arguments passed to \code{vis_lm_assumptions()}.}
}
\value{
A list with elements:
\describe{
  \item{summary_anova}{Summary of the ANOVA model.}
  \item{shapiro_test}{Result from \code{shapiro.test()}.}
  \item{ad_test}{Result from \code{nortest::ad.test()} or a character message if n < 7.}
  \item{levene_test}{Result from \code{levene.test()} (only if \code{regression = FALSE}).}
  \item{bartlett_test}{Result from \code{bartlett.test()} (only if \code{regression = FALSE}).}
  \item{bp_test}{Result from \code{bp.test()} (only if \code{regression = TRUE}).}
}
}
\description{
Checks for normality of the standardised residuals in the general linear model
Student's t-test 
(t.test,var=EQUAL) 
Fisher oneway ANOVA (aov) or simple linear regression.
Performs the Shapiro-Wilk test and Anderson-Darling test for normality and,
if not a regression, also the Levene-Brown-Forsythe and the Bartlett's test for homogeneity of variances.
It produces a histogram with normal overlay, a residuals vs fitted plot,
and a normal Q-Q plot.
}
\examples{
ToothGrowth$dose <- as.factor(ToothGrowth$dose)
vis_lm_assumptions(ToothGrowth$len, ToothGrowth$dose)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visstat_methods.R
\name{plot.visstat}
\alias{plot.visstat}
\title{Plot method for visstat objects}
\usage{
\method{plot}{visstat}(x, which = NULL, ...)
}
\arguments{
\item{x}{An object of class \code{"visstat"}, returned by
\code{\link{visstat}()}.}

\item{which}{Integer selecting a single plot to display (1, 2, ...).
If \code{NULL} (the default), all available plots are listed without
being drawn.}

\item{...}{Currently unused. Included for S3 method compatibility.}
}
\value{
The object \code{x}, invisibly. Called for its side effect.
}
\description{
Replays captured plots or reports saved plot file paths from a
\code{visstat} object.
}
\details{
When called without \code{which}, the method lists all available plots
(either as file paths or as indices of captured plots).
When called with \code{which}, the selected plot is displayed:
for file-based output the stored path is printed, for captured plots
the plot is replayed via \code{replayPlot()}.
}
\examples{
# File-based output: plot() lists stored paths
anova_path <- visstat(
  npk$block,
  npk$yield,
  graphicsoutput = "png",
  plotDirectory = tempdir()
)

plot(anova_path)

# Interactive output: plot() lists available plots,
# plot(obj, which = n) replays a specific one
linreg <- visstat(trees$Height, trees$Girth)

plot(linreg)
plot(linreg, which = 2)

}
\seealso{
\code{\link{print.visstat}},
  \code{\link{summary.visstat}}, \code{\link{visstat}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/counts_to_cases.R
\name{counts_to_cases}
\alias{counts_to_cases}
\title{Convert data frame of counts to dataframe of cases.
The data frame must contain a column with frequencies (counts) as generated by
as.data.frame from a contingency table}
\usage{
counts_to_cases(x, countcol = "Freq")
}
\arguments{
\item{x}{a \code{data.frame} of counts generated from a contingency table.}

\item{countcol}{character string, name of the column of x containing the
counts. Default name of the column is  'Freq'.}
}
\value{
data frame of cases of dimension (total number of counts as sum of
'Freq' in x) times 2.
}
\description{
Convert data frame of counts to dataframe of cases.
The data frame must contain a column with frequencies (counts) as generated by
as.data.frame from a contingency table
}
\examples{

counts_to_cases(as.data.frame(HairEyeColor[, , 1]), countcol = "Freq")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/levene.test.R
\name{levene.test}
\alias{levene.test}
\title{Levene-Brown-Forsythe Test for Homogeneity of Variance (center = median)}
\usage{
levene.test(y, g, data = NULL)
}
\arguments{
\item{y}{A numeric response vector.}

\item{g}{A grouping factor.}

\item{data}{Optional data frame containing `y` and `g`.}
}
\value{
An object of class \code{"htest"} with components:
\item{statistic}{the value of the F-statistic.}
\item{parameter}{degrees of freedom: df1=k-1, 
df3=N-k, where k is the number of groups and N the total sample size }
\item{p.value}{the p-value of the test.}
\item{method}{a character string indicating the test performed.}
\item{data.name}{a character string giving the name(s) of the data.}
}
\description{
Performs Levene's test using the
 Brown-Forsythe modification (median-centred). 
 It tests the null hypothesis that all groups 
have equal variances by testing whether the absolute deviations
 from group  medians are equal across groups
 The function reproduces the default behaviour of the 
 leveneTest(y,g,center=median,...) of the car-package.
}
\details{
For each observation \eqn{y_{ij}} in group \eqn{i}, compute the absolute 
deviation from the group median:

\deqn{z_{ij} = |y_{ij} - \tilde{y}_i|}

where \eqn{\tilde{y}_i} is the median of group \eqn{i}.

The test statistic is the F-statistic from a one-way ANOVA on the \eqn{z_{ij}} values:

\deqn{F = \frac{(N-k) \sum_{i=1}^{k} n_i (\bar{z}_i - \bar{z})^2}{(k-1) \sum_{i=1}^{k} \sum_{j=1}^{n_i} (z_{ij} - \bar{z}_i)^2}}

where:
\itemize{
  \item \eqn{k} = number of groups
  \item \eqn{N} = total sample size
  \item \eqn{n_i} = sample size of group \eqn{i}
  \item \eqn{\bar{z}_i} = mean of absolute deviations in group \eqn{i}
  \item \eqn{\bar{z}} = overall mean of all absolute deviations
}

Under the null hypothesis of equal variances, the test statistic follows 
an F-distribution: \eqn{F \sim F(k-1, N-k)}.
}
\examples{
set.seed(123)
y <- c(rnorm(10), rnorm(10, sd = 2), rnorm(10, sd = 0.5))
g <- factor(rep(1:3, each = 10))
levene.test(y, g)

# Usage with data frame
df <- data.frame(response = y, group = g)
levene.test(response, group, data = df)

# Example with unequal variances (should reject null hypothesis)
set.seed(456)
y_unequal <- c(rnorm(15, sd = 1), rnorm(15, sd = 5), rnorm(15, sd = 0.2))
g_unequal <- factor(rep(c("A", "B", "C"), each = 15))
levene.test(y_unequal, g_unequal)



}
\references{
Brown, M. B., and Forsythe, A. B. (1974). Robust tests for the equality of
variances. Journal of the American Statistical Association, 69(346), 364–367.
DOI: 10.1080/01621459.1974.10482955
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/opening_saving_Cairo.R
\name{openGraphCairo}
\alias{openGraphCairo}
\title{Cairo wrapper function with plot capture capability}
\usage{
openGraphCairo(
  width = 640,
  height = 480,
  fileName = NULL,
  type = NULL,
  fileDirectory = getwd(),
  pointsize = 12,
  bg = "transparent",
  canvas = "white",
  units = "px",
  dpi = 150
)
}
\arguments{
\item{width}{see \code{Cairo()}}

\item{height}{see \code{Cairo()}}

\item{fileName}{name of file to be created. Does not include both file
extension '.\code{type}' and file \code{filedirectory}. Default file name
'visstat_plot'.}

\item{type}{Supported output types are 'png', 'jpeg', 'pdf', 'svg', 'ps' and
'tiff'. See \code{Cairo()}}

\item{fileDirectory}{path of directory, where plot is stored. Default current
working directory.}

\item{pointsize}{see \code{Cairo()}}

\item{bg}{see \code{Cairo()}}

\item{canvas}{see \code{Cairo()}}

\item{units}{see \code{Cairo()}}

\item{dpi}{DPI used for the conversion of units to pixels. Default value 150.}
}
\value{
NULL, if no \code{type} is specified. Otherwise see \code{Cairo()}
}
\description{
Cairo wrapper function returning NULL if not \code{type} is specified.
Enhanced version that can capture plots for later replay.
}
\details{
\code{openGraphCairo()} \code{Cairo()} wrapper function. Differences to
\code{Cairo:} a) prematurely ends the function call to \code{Cairo()}
returning NULL, if no output \code{type}  of types 'png', 'jpeg', 'pdf',
'svg', 'ps' or 'tiff' is provided. b) The \code{file} argument of the
underlying Cairo function is generated by
\code{file.path(fileDirectory,paste(fileName,'.', type, sep = ''))}.
c) Can set up plot capture when \code{capture_env} is provided.
}
\examples{

##  adapted from example in \code{Cairo()}
openGraphCairo(fileName = "normal_dist", type = "pdf", fileDirectory = tempdir())
plot(rnorm(4000), rnorm(4000), col = "#ff000018", pch = 19, cex = 2)
dev.off() # creates a file 'normal_dist.pdf' in the directory specified in fileDirectory
# ## remove the plot from fileDirectory
file.remove(file.path(tempdir(), "normal_dist.pdf"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/opening_saving_Cairo.R
\name{saveGraphVisstat}
\alias{saveGraphVisstat}
\title{Saves Graphical Output with plot capture capability}
\usage{
saveGraphVisstat(
  fileName = NULL,
  type = NULL,
  fileDirectory = getwd(),
  oldfile = NULL,
  capture_env = NULL
)
}
\arguments{
\item{fileName}{name of file to be created in directory \code{fileDirectory}
without file extension '.\code{type}'.}

\item{type}{see \code{Cairo()}.}

\item{fileDirectory}{path of directory, where graphic is stored. Default
setting current working directory.}

\item{oldfile}{old file of same name to be overwritten}

\item{capture_env}{Environment to store captured plots. If NULL, no capture occurs.}
}
\value{
NULL, if no \code{type} or \code{fileName} is provided, file path if graph
  is created
}
\description{
Closes all graphical devices with \code{dev.off()} and saves the output only
if both \code{fileName} and \code{type} are provided. Enhanced version that
can capture plots before closing devices.
}
\examples{

# very simple KDE (adapted from example in Cairo())
openGraphCairo(type = "png", fileDirectory = tempdir())
plot(rnorm(4000), rnorm(4000), col = "#ff000018", pch = 19, cex = 2)
# save file 'norm.png' in directory specified in fileDirectory
saveGraphVisstat("norm", type = "png", fileDirectory = tempdir())
file.remove(file.path(tempdir(), "norm.png")) # remove file 'norm.png'

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visstat.R
\name{visstat}
\alias{visstat}
\title{Wrapper for visstat_core Allowing Three Different Input Styles}
\usage{
visstat(
  x,
  y,
  ...,
  data = NULL,
  conf.level = 0.95,
  correlation = FALSE,
  numbers = TRUE,
  minpercent = 0.05,
  graphicsoutput = NULL,
  plotName = NULL,
  plotDirectory = getwd()
)
}
\arguments{
\item{x}{For the formula interface: a formula of the form \code{y ~ x}, 
where \code{y} is the response variable and \code{x} is the predictor or 
grouping variable (requires \code{data} argument). For the standardised 
form: a vector of class \code{"numeric"}, \code{"integer"}, or 
\code{"factor"} representing the predictor or grouping variable. For the 
backward-compatible form: a \code{data.frame} containing the relevant 
columns.}

\item{y}{For the formula interface: not used (variables are extracted from 
the formula). For the standardised form: a vector of class \code{"numeric"}, 
\code{"integer"}, or \code{"factor"} representing the response variable. 
For the backward-compatible form: a \code{character} string specifying the 
name of the response variable column in \code{x}.}

\item{...}{For the backward-compatible form only: a \code{character} string 
specifying the name of the predictor or grouping variable column in 
\code{x}. Ignored for formula and standardised input styles.}

\item{data}{A \code{data.frame} containing the variables specified in the 
formula. Required when using the formula interface. Ignored for other 
input styles.}

\item{conf.level}{Confidence level for statistical inference; default is 
\code{0.95}.}

\item{correlation}{Logical. If \code{FALSE} (default), performs simple
linear regression analysis with confidence and prediction bands when both
variables are numeric. If \code{TRUE}, performs Spearman correlation analysis
with trend line only (no regression interpretation).}

\item{numbers}{Logical. Whether to annotate plots with numeric values.}

\item{minpercent}{Number between 0 and 1 indicating minimal fraction of 
total count data of a category to be displayed in mosaic count plots.}

\item{graphicsoutput}{Saves plot(s) of type \code{"png"}, \code{"jpg"}, 
\code{"tiff"} or \code{"bmp"} in directory specified in 
\code{plotDirectory}. If \code{NULL}, no plots are saved.}

\item{plotName}{Graphical output is stored following the naming convention 
\code{"plotName.graphicsoutput"} in \code{plotDirectory}. Without 
specifying this parameter, \code{plotName} is automatically generated 
following the convention \code{"statisticalTestName_varsample_varfactor"}.}

\item{plotDirectory}{Specifies directory where generated plots are stored. 
Default is current working directory.}
}
\value{
An object of class \code{"visstat"} containing the results of 
the automatically selected statistical test. The specific contents depend on 
which test was performed. Additionally, the returned object includes two 
attributes:
\itemize{
  \item \code{plot_paths}: Character vector of file paths where plots were 
    saved (if \code{graphicsoutput} was specified)
  \item \code{captured_plots}: List of captured plot objects for programmatic 
    access
}

In case of insufficient data, returns a list with an \code{error} element and 
basic input summary information.
}
\description{
\code{visstat()} is a wrapper around \code{\link{visstat_core}} 
that provides three alternative input styles: a formula interface, a 
standardised vector interface, and a backward-compatible data frame interface.
\code{\link{visstat_core}} defines the decision logic for statistical 
hypothesis testing and visualisation between two variables of class 
\code{"numeric"}, \code{"integer"}, or \code{"factor"}.
}
\details{
This wrapper supports three input formats:

(1) Formula interface: \code{visstat(y ~ x, data = df)}, where the formula 
specifies the response (\code{y}) and predictor (\code{x}) variables, and 
\code{data} is a data frame containing these variables.

(2) Standardised form: \code{visstat(x, y)}, where both \code{x} and \code{y} 
are vectors of class \code{"numeric"}, \code{"integer"}, or \code{"factor"}. 
Here \code{x} is the predictor or grouping variable and \code{y} is the 
response variable.

(3) Backward-compatible form: \code{visstat(dataframe, "name_of_y", "name_of_x")}, 
where the first character string refers to the response variable and the 
second to the predictor or grouping variable. Both must be column names in 
\code{dataframe}.

The interpretation of \code{x} and \code{y} depends on the variable classes. 
Throughout, data of class \code{numeric} or \code{integer} are referred to 
as numeric, while data of class \code{factor} are referred to as categorical:

If one variable is numeric and the other a factor, the numeric vector is the 
response (\code{y}) and the factor is the grouping variable (\code{x}). This 
supports tests of central tendencies (e.g., t-test, Welch's ANOVA, Wilcoxon, 
Kruskal-Wallis).

If both variables are numeric, a linear model is fitted with \code{y} as the 
response and \code{x} as the predictor.

If both variables are factors, an association test (Chi-squared or Fisher's 
exact) is used. The test result is invariant to variable order, but 
visualisations (e.g., axis layout, bar orientation) depend on the roles of 
\code{x} and \code{y}.

This wrapper standardises the input and calls \code{\link{visstat_core}}, 
which selects and executes the appropriate test with visual output and 
assumption diagnostics.
}
\note{
For best visualization, ensure that the RStudio Plots pane is adequately 
sized. If you get "figure margins too large" errors, try expanding the Plots 
pane in RStudio, using \code{dev.new(width=10, height=6)} for a larger plot 
window, or reducing the \code{cex} parameter.
}
\examples{
# Formula interface
mtcars$am <- as.factor(mtcars$am)
visstat(mpg ~ am, data = mtcars)

# Standardised usage
visstat(mtcars$am, mtcars$mpg)

# Backward-compatible usage (same result as above)
visstat(mtcars, "mpg", "am")

## Student's t-test (equal variances, two groups)
# When residuals are normally distributed and Levene's test indicates
# homoscedasticity, the classic Student's t-test with pooled variance is used
df <- droplevels(subset(PlantGrowth, group \%in\% c("ctrl", "trt1")))
visstat(df$group,df$weight)

## Welch's t-test (unequal variances, two groups)
# When residuals are normally distributed but Levene's test indicates
# heteroscedasticity, Welch's t-test is used
visstat(mtcars$am, mtcars$mpg)

## Wilcoxon rank sum test (non-normal, two groups)
# When residuals are not normally distributed
grades_gender <- data.frame(
  Sex = as.factor(c(rep("Girl", 20), rep("Boy", 20))),
  Grade = c(
    19.3, 18.1, 15.2, 18.3, 7.9, 6.2, 19.4, 20.3, 9.3, 11.3,
    18.2, 17.5, 10.2, 20.1, 13.3, 17.2, 15.1, 16.2, 17.3, 16.5,
    5.1, 15.3, 17.1, 14.8, 15.4, 14.4, 7.5, 15.5, 6.0, 17.4,
    7.3, 14.3, 13.5, 8.0, 19.5, 13.4, 17.9, 17.7, 16.4, 15.6
  )
)
visstat(grades_gender$Sex, grades_gender$Grade)

## Fisher's ANOVA (equal variances, >2 groups)
# When residuals are normally distributed and Levene's test indicates
# homoscedasticity, classic Fisher's ANOVA with TukeyHSD post-hoc is used.
# Different green letters indicate significant differences between groups. 
visstat(PlantGrowth$group, PlantGrowth$weight)

## Welch's one-way ANOVA (unequal variances, >2 groups)
set.seed(123)
values <- c(rnorm(20, 10, 1),rnorm(20, 15, 5),rnorm(20, 12, 2))
groups <- factor(rep(c("A", "B", "C"), each = 20))
visstat(groups, values)
## Kruskal-Wallis (non-normal, >2 groups)
# When residuals are not normally distributed, kruskal.test() is followed by 
# pairwise.wilcox.test. 
visstat(iris$Species, iris$Petal.Width)

## Simple linear regression (both numeric)
visstat(trees$Height, trees$Girth, conf.level = 0.99)

## Pearson's Chi-squared test (both factors, large expected counts)
HairEyeColorDataFrame <- counts_to_cases(as.data.frame(HairEyeColor))
visstat(HairEyeColorDataFrame$Eye, HairEyeColorDataFrame$Hair)

## Fisher's exact test (both factors, small expected counts)
HairEyeColorMaleFisher <- HairEyeColor[, , 1]
blackBrownHazelGreen <- HairEyeColorMaleFisher[1:2, 3:4]
blackBrownHazelGreen <- counts_to_cases(as.data.frame(blackBrownHazelGreen))
visstat(blackBrownHazelGreen$Eye, blackBrownHazelGreen$Hair)

## Save PNG
visstat(blackBrownHazelGreen$Hair, blackBrownHazelGreen$Eye,
        graphicsoutput = "png", plotDirectory = tempdir())

## Custom plot name
visstat(iris$Species, iris$Petal.Width,
        graphicsoutput = "pdf", plotName = "kruskal_iris", plotDirectory = tempdir())

}
\seealso{
\code{\link{visstat_core}} defining the decision logic, the 
package's vignette \code{vignette("visStatistics")} explaining the decision 
logic accompanied by illustrative examples, and the accompanying webpage 
\url{https://shhschilling.github.io/visStatistics/}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/games.howell.R
\name{games.howell}
\alias{games.howell}
\title{Games-Howell Post-Hoc Test}
\usage{
games.howell(samples, groups, conf.level = 0.95)
}
\arguments{
\item{samples}{Numeric vector; the dependent variable.}

\item{groups}{Factor or vector; the grouping variable.}

\item{conf.level}{Numeric; confidence level for confidence intervals (default: 0.95).}
}
\value{
A data frame with columns:
\describe{
  \item{group1}{First group in comparison}
  \item{group2}{Second group in comparison}
  \item{mean_diff}{Difference in means (group1 - group2)}
  \item{se}{Standard error of the difference}
  \item{t}{t-statistic}
  \item{df}{Degrees of freedom (Welch-Satterthwaite)}
  \item{p_value}{Unadjusted p-value}
  \item{p_adj}{Holm-adjusted p-value for multiple comparisons}
  \item{ci_lower}{Lower bound of confidence interval}
  \item{ci_upper}{Upper bound of confidence interval}
  \item{significant}{Logical; TRUE if p_adj < (1 - conf.level)}
}
}
\description{
Performs pairwise comparisons using the Games-Howell test, which does not
assume equal variances or equal sample sizes. This is the appropriate
post-hoc test to use after a significant Welch's ANOVA.
}
\details{
The Games-Howell test uses the Welch-Satterthwaite approximation for
degrees of freedom and does not pool variances. P-values are adjusted
using the Holm method to control family-wise error rate.
}
\examples{
# Convert dose to factor
ToothGrowth$dose <- as.factor(ToothGrowth$dose)

# Perform Games-Howell test
result <- games.howell(ToothGrowth$len, ToothGrowth$dose)
print(result)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_anova.R
\name{vis_anova}
\alias{vis_anova}
\title{ANOVA or Welch's ANOVA with appropriate post-hoc tests}
\usage{
vis_anova(
  samples,
  fact,
  conf.level = conf.level,
  samplename = "",
  factorname = "",
  cex = 1
)
}
\arguments{
\item{samples}{Numeric vector; the dependent variable.}

\item{fact}{Factor; the grouping variable.}

\item{conf.level}{Numeric; confidence level for tests and intervals (default: 0.95).}

\item{samplename}{Character; label for y-axis (default: "").}

\item{factorname}{Character; label for x-axis (default: "").}

\item{cex}{Numeric; character expansion factor for plot elements (default: 1).}
}
\value{
A list with components:
\describe{
  \item{summary statistics of ANOVA}{Summary of Fisher's ANOVA or Welch's oneway test}
  \item{post-hoc analysis}{TukeyHSD object or Games-Howell results in compatible format}
  \item{conf.level}{The confidence level used}
}
}
\description{
Internal function that performs ANOVA or Welch's one-way test and 
corresponding post-hoc comparisons. Uses TukeyHSD for equal variances 
(Fisher's ANOVA) and Games-Howell for unequal variances (Welch's ANOVA).
}
\details{
The function first tests for homogeneity of variance using Levene's test.
If variances are equal (p > alpha), Fisher's one-way ANOVA with Tukey's HSD
post-hoc is performed. If variances are unequal (p <= alpha), Welch's
one-way ANOVA with Games-Howell post-hoc is performed.

The function produces a box plot with jittered points and group means
(red diamonds for the parametric branches), annotated with a compact
letter display showing which groups differ significantly.
}
\examples{
# Example with equal variances (uses Fisher's ANOVA + TukeyHSD)
data(PlantGrowth)
result1 <- vis_anova(PlantGrowth$weight, PlantGrowth$group, 
                     samplename = "Weight", factorname = "Group")

# Example with unequal variances (uses Welch's ANOVA + Games-Howell)
# Create data with heterogeneous variances
set.seed(123)
group_a <- rnorm(20, mean = 10, sd = 1)
group_b <- rnorm(20, mean = 15, sd = 5)  # Much larger variance
group_c <- rnorm(20, mean = 12, sd = 2)
values <- c(group_a, group_b, group_c)
groups <- factor(rep(c("A", "B", "C"), each = 20))
result2 <- vis_anova(values, groups, 
                     samplename = "Value", factorname = "Group")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visstat_methods.R
\name{print.visstat}
\alias{print.visstat}
\title{Print method for visstat objects}
\usage{
\method{print}{visstat}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"visstat"}, returned by
\code{visstat()}.}

\item{...}{Currently unused. Included for S3 method compatibility.}
}
\value{
The object \code{x}, invisibly.
}
\description{
Displays a brief summary of the statistical test results and, if
available, assumption tests and post hoc comparisons.
}
\details{
Quick overview of the statistical analysis results.
}
\examples{
anova <- visstat(npk$block, npk$yield)
print(anova)

}
\seealso{
\code{\link{summary.visstat}},
  \code{\link{plot.visstat}}, \code{\link{visstat}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/games.howell.R
\name{gh_letters}
\alias{gh_letters}
\title{Get compact letter display from Games-Howell results}
\usage{
gh_letters(x, alpha = 0.05)
}
\arguments{
\item{x}{A games.howell object from \code{games.howell()}}

\item{alpha}{Significance level (default: 0.05)}
}
\value{
A named vector with group names and their letter codes
}
\description{
Converts Games-Howell test results into compact letter display
using multcompView. Groups sharing a letter are not significantly different.
}
\examples{
# Convert dose to factor
ToothGrowth$dose <- as.factor(ToothGrowth$dose)
result <- games.howell(ToothGrowth$len, ToothGrowth$dose)
letters <- gh_letters(result)
print(letters)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visstat_core.R
\name{visstat_core}
\alias{visstat_core}
\title{Automated Visualization of Statistical Hypothesis Testing}
\usage{
visstat_core(
  dataframe,
  varsample,
  varfactor,
  conf.level = 0.95,
  correlation = FALSE,
  numbers = TRUE,
  minpercent = 0.05,
  graphicsoutput = NULL,
  plotName = NULL,
  plotDirectory = getwd()
)
}
\arguments{
\item{dataframe}{\code{data.frame} with at least two columns.}

\item{varsample}{\code{character} string matching a column name in 
\code{dataframe}. Interpreted as the response if the referenced column is 
of class \code{numeric} or \code{integer} and the column named by 
\code{varfactor} is of class \code{factor}.}

\item{varfactor}{\code{character} string matching a column name in 
\code{dataframe}. Interpreted as the grouping variable if the referenced 
column is of class \code{factor} and the column named by \code{varsample} 
is of class \code{numeric} or \code{integer}.}

\item{conf.level}{Confidence level}

\item{correlation}{Logical. If FALSE (default), performs simple
linear regression analysis with confidence and prediction bands.
If TRUE, performs Spearman correlation analysis with trend line only
 (no regression interpretation).}

\item{numbers}{a logical indicating whether to show numbers in mosaic count
plots.}

\item{minpercent}{number between 0 and 1 indicating minimal fraction of total
count data of a category to be displayed    in mosaic count plots.}

\item{graphicsoutput}{saves plot(s) of type "png",  "jpg", "tiff" or  "bmp"
in directory specified in \code{plotDirectory}. If graphicsoutput=NULL, no
plots are saved.}

\item{plotName}{graphical output is stored following the naming convention
"plotName.graphicsoutput" in \code{plotDirectory}. Without specifying this
parameter, plotName is automatically generated following the convention
"statisticalTestName_varsample_varfactor".}

\item{plotDirectory}{specifies directory, where generated plots are stored.
Default is current working directory.}
}
\value{
An object of class \code{"visstat"} containing the results of 
the automatically selected statistical test. The specific contents depend on
 which test was performed.
Additionally, the returned object includes two attributes:
\itemize{
  \item \code{plot_paths}: Character vector of file paths where plots were 
    saved (if \code{graphicsoutput} was specified)
  \item \code{captured_plots}: List of captured plot objects for programmatic 
    access
}
}
\description{
\code{visstat_core()} provides automated selection and visualization 
of a statistical hypothesis test between a two vectors in
a given \code{data.frame} named \code{dataframe} based on the data's type, 
distribution, sample size, and the
specified \code{conf.level}. \code{visstat_core()} is called by the main
 wrapper function \code{visstat()}.
\code{varsample} and \code{varfactor} are \code{character}
strings corresponding to the column names of the chosen vectors in \code{dataframe}. 
These vectors must be of type \code{integer}, \code{numeric} or \code{factor}.
The automatically generated output figures
illustrate the selected statistical hypothesis test, display the main test
statistics, and include assumption checks and post hoc comparisons when
applicable. The primary test results are returned as a list object.
}
\details{
The decision logic for selecting a statistical test is described below.
For more details, please refer to the package's \code{vignette("visStatistics")}.
Throughout, data of class \code{numeric} or \code{integer} are referred to as
numeric, while data of class \code{factor} are referred to as categorical.
The significance level \code{alpha} is defined as one minus the confidence
level, given by the argument \code{conf.level}. Assumptions of normality and
homoscedasticity are considered met when the corresponding test yields a
p-value greater than \code{alpha = 1 - conf.level}.
The choice of statistical tests performed by \code{visstat_core()} depends on
whether the data are numeric or categorical, the number of levels in the
categorical variable, the distribution of the data, and the chosen
\code{conf.level}. The function prioritises interpretable visual output and
tests that remain valid under their assumptions, following the logic below:

(1) When the response is numerical and the predictor is categorical, tests of
central tendencies are performed. For the decision logic, please refer to the 
packages vignette \code{vignette("visStatistics")}

(2): When both the response and predictor are numerical, a linear model
\code{lm()} is fitted, with residual diagnostics and a confidence band plot.

(3): When both variables are categorical, \code{visstat_core()} uses
\code{chisq.test()} or \code{fisher.test()} depending on expected counts,
following Cochran's rule (Cochran (1954) <doi:10.2307/3001666>).

Implemented main tests: 

\code{t.test()}, \code{wilcox.test()}, \code{aov()},
\code{oneway.test()}, \code{lm()}, \code{kruskal.test()},
\code{fisher.test()}, \code{chisq.test()}.

Implemented tests for assumptions:
\itemize{
  \item Normality: \code{shapiro.test()} and \code{ad.test()}
  \item Heteroscedasticity: \code{bartlett.test()} and \code{levene.test()} and \code{bp_test()}
}

Implemented post hoc tests:
\itemize{
  \item \code{TukeyHSD()} for \code{aov()} 
  \item \code{games.howell} for  \code{oneway.test()}
  \item \code{pairwise.wilcox.test()} for \code{kruskal.test()}
}
}
\examples{
# Welch Two Sample t-test (t.test())
visstat_core(mtcars, "mpg", "am")

## Wilcoxon rank sum test (wilcox.test())
grades_gender <- data.frame(
  Sex = as.factor(c(rep("Girl", 20), rep("Boy", 20))),
  Grade = c(
    19.3, 18.1, 15.2, 18.3, 7.9, 6.2, 19.4,
    20.3, 9.3, 11.3, 18.2, 17.5, 10.2, 20.1, 13.3, 17.2, 15.1, 16.2, 17.3,
    16.5, 5.1, 15.3, 17.1, 14.8, 15.4, 14.4, 7.5, 15.5, 6.0, 17.4,
    7.3, 14.3, 13.5, 8.0, 19.5, 13.4, 17.9, 17.7, 16.4, 15.6
  )
)
visstat_core(grades_gender, "Grade", "Sex")

## Welch's oneway ANOVA not assuming equal variances (oneway.test())
anova_npk <- visstat_core(npk, "yield", "block")
anova_npk # prints summary of tests

## Kruskal-Wallis rank sum test (kruskal.test())
visstat_core(iris, "Petal.Width", "Species")
visstat_core(InsectSprays, "count", "spray")

## Simple linear regression  (lm())
visstat_core(trees, "Girth", "Height", conf.level = 0.99)

## Pearson's Chi-squared test (chisq.test())
### Transform array to data.frame
HairEyeColorDataFrame <- counts_to_cases(as.data.frame(HairEyeColor))
visstat_core(HairEyeColorDataFrame, "Hair", "Eye")

## Fisher's exact test (fisher.test())
HairEyeColorMaleFisher <- HairEyeColor[, , 1]
### slicing out a 2 x2 contingency table
blackBrownHazelGreen <- HairEyeColorMaleFisher[1:2, 3:4]
blackBrownHazelGreen <- counts_to_cases(as.data.frame(blackBrownHazelGreen))
fisher_stats <- visstat_core(blackBrownHazelGreen, "Hair", "Eye")

}
\seealso{
The package's vignette
\code{vignette("visStatistics")} for a description of the 
decision logic, illustrated with numerous examples. The package is accompanied 
by its webpage
\url{https://shhschilling.github.io/visStatistics/}. The main function \code{\link{visstat}} for a detailed description of the return value.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_group_normality.R
\name{vis_group_normality}
\alias{vis_group_normality}
\title{Visualisation of the normality assumption for Welch ANOVA/t-test}
\usage{
vis_group_normality(
  samples,
  groups,
  conf.level = 0.95,
  samplename = "",
  groupname = "",
  cex = 1
)
}
\arguments{
\item{samples}{Numeric vector; the dependent variable.}

\item{groups}{Factor or vector; the grouping variable (2 to 8 groups for visual display).}

\item{conf.level}{Numeric; confidence level (default: 0.95). Used to determine 
alpha = 1 - conf.level for normality test interpretation.}

\item{samplename}{Character; label for the y-axis (default: "").}

\item{groupname}{Character; label for the x-axis (default: "").}

\item{cex}{Numeric; scaling factor for plot text and symbols (default: 1).}
}
\value{
A list containing:
\describe{
  \item{shapiro_tests}{List of Shapiro-Wilk test results for each group}
  \item{ad_tests}{List of Anderson-Darling test results for each group}
  \item{n_groups}{Number of groups}
  \item{group_names}{Names of the groups}
}
}
\description{
\code{vis_group_normality} checks for normality of each group separately using 
the Shapiro-Wilk and Anderson-Darling tests. The null hypothesis is that 
each group is normally distributed. The function generates histograms 
with normal distribution overlays and Q-Q plots to visually assess normality.
The layout is always 2 rows × k columns (histograms on top, Q-Q plots on bottom).
}
\details{
Layout is always 2 rows × k columns:
\itemize{
  \item Top row: Histograms with normal overlay for each group
  \item Bottom row: Q-Q plots for each group
}
For more than 8 groups, a tabular summary is provided instead of plots.
}
\examples{
# Two groups (like t-test)
vis_group_normality(ToothGrowth$len, ToothGrowth$supp)

# Three groups
ToothGrowth$dose <- as.factor(ToothGrowth$dose)
vis_group_normality(ToothGrowth$len, ToothGrowth$dose)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visstat_methods.R
\name{summary.visstat}
\alias{summary.visstat}
\title{Summary method for visstat objects}
\usage{
\method{summary}{visstat}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"visstat"}, returned by
\code{visstat()}.}

\item{...}{Currently unused. Included for S3 method compatibility.}
}
\value{
The object \code{object}, invisibly.
}
\description{
Displays the full statistical test results and, if available,
assumption tests and post hoc comparisons.
}
\details{
This method provides a full textual report of the statistical
test results returned by \code{visstat()}, and prints the contents of
\code{posthoc_summary} if present.
}
\examples{
anova <- visstat(npk$block, npk$yield)
summary(anova)

}
\seealso{
\code{\link{print.visstat}}, \code{\link{visstat}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pooled_normality_test.R
\name{pooled_normality_test}
\alias{pooled_normality_test}
\title{Test normality using pooled standardized residuals}
\usage{
pooled_normality_test(y, g, test = c("shapiro", "ad"), min_n = 3L)
}
\arguments{
\item{y}{Numeric vector of response values}

\item{g}{Factor or vector defining groups (must have at least 2 levels)}

\item{test}{Character, either "shapiro" or "ad" for Anderson-Darling}

\item{min_n}{Minimum sample size per group required (default 3)}
}
\value{
A list with test results (statistic, p.value, method, data.name)
}
\description{
Standardizes values within each group and applies a single normality 
test to the pooled standardized values. This avoids multiple testing 
issues when deciding between parametric and non-parametric methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_numeric.R
\name{vis_numeric}
\alias{vis_numeric}
\title{Visualize Numeric Relationships: Regression or Correlation Analysis}
\usage{
vis_numeric(
  y,
  x,
  correlation = FALSE,
  conf.level = 0.95,
  name_of_factor = character(),
  name_of_sample = character()
)
}
\arguments{
\item{y}{Numeric vector. The response variable (dependent variable) for regression analysis, 
or the y-axis variable for correlation analysis.}

\item{x}{Numeric vector. The predictor variable (independent variable) for regression analysis,
or the x-axis variable for correlation analysis. Must have the same length as y.}

\item{correlation}{Logical. If FALSE (default), performs regression analysis with
confidence and prediction bands. If TRUE, performs Spearman rank correlation analysis.}

\item{conf.level}{Numeric. Confidence level for statistical tests and intervals. 
Must be between 0 and 1. Default is 0.95 (95 percent confidence level).}

\item{name_of_factor}{Character string. Label for the x-axis (independent variable). 
If empty, defaults to the variable name.}

\item{name_of_sample}{Character string. Label for the y-axis (dependent variable). 
If empty, defaults to the variable name.}
}
\value{
A list containing analysis results and assumption checks. Content depends on analysis type.
  For regression analysis: analysis_type, summary_regression, assumptions, warnings, r_squared, adj_r_squared.
  For correlation analysis: analysis_type, correlation_test, correlation_coefficient, assumptions,
  warnings, method_used.
}
\description{
This function provides unified visualization for numeric relationships between two continuous variables.
It can perform either regression analysis (with confidence and prediction bands) or
Spearman rank correlation analysis with appropriate visualizations and statistical output.
For regression, statistical assumptions are checked and warnings are issued if violated, but analysis proceeds.
}
\details{
Statistical Assumptions Checked:
Regression: Normality of residuals (Shapiro-Wilk test) and
homoscedasticity (Breusch-Pagan test). All regression analyses proceed even if
assumptions are violated, but appropriate warnings are issued.
Correlation: Spearman rank correlation requires no distributional assumptions.
}
\examples{
\dontrun{
# Generate sample data
set.seed(123)
x <- rnorm(50, mean = 10, sd = 2)
y <- 2 * x + rnorm(50, mean = 0, sd = 1)

# Regression analysis (default)
result1 <- vis_numeric(y, x, 
                      name_of_factor = "Predictor", 
                      name_of_sample = "Response")

# Spearman rank correlation
result2 <- vis_numeric(y, x, correlation = TRUE)
}

}
\seealso{
\code{\link{cor.test}}, \code{\link{lm}}, \code{\link{shapiro.test}}
}
\author{
Sabine Schilling
}
